PRO CHANGE_MAP_RESOLUTION, map_in, map_out, beam_in=beam_in, beam_out=beam_out, nside_out=nside_out, lmax=lmax, iter=iter, $
                           globalfactor=globalfactor, ordering_in=ordering_in, ordering_out=ordering_out, double=double, $
                           nohdr=nohdr, nocheck=nocheck, overwrite=overwrite, help=help

  IF ((KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 2)) THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CHANGE_MAP_RESOLUTION: procedure to change the resolution of a map'
     PSM_INFO_MESSAGE, 'SYNTAX: CHANGE_MAP_RESOLUTION, map_in, map_out, beam_in=, beam_out=, nside_out=, lmax=, iter=, $'
     PSM_INFO_MESSAGE, '                               globalfactor=, ordering_in=, ordering_out=, /double, /nohdr'
     PSM_INFO_MESSAGE, 'beam_in and beam_out are structures for the input and output beam'
     PSM_INFO_MESSAGE, 'globalfactor is an optional global factor to apply to the map'
     PSM_INFO_MESSAGE, 'iter is the number of iterations used in the harmonic transforms [default = 0]'
     PSM_INFO_MESSAGE, 'set /overwrite to overwrite any already existing output'
     PSM_INFO_MESSAGE, 'set /double to force double precision for output alm'
     PSM_INFO_MESSAGE, 'set /nohdr to skip the writing of the correct header in intermediate and output files'
     PSM_INFO_MESSAGE, "set /nocheck to skip verification of consistency of beam in input file header and beam provided as 'beam_in'"
     PSM_INFO_MESSAGE, "set /overwrite to overwrite any existing output map"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ON_ERROR, 2

;  PSM_DEBUG_MESSAGE, 'Implement correct header in output file', routine='CHANGE_MAP_RESOLUTION'
;  PSM_DEBUG_MESSAGE, 'Check for existing output map in memory if overwrite keyword not set', routine='CHANGE_MAP_RESOLUTION'

  IF NOT KEYWORD_SET(overwrite) THEN overwrite=0

  IF DEFINED(map_out) THEN BEGIN
     IF overwrite EQ 0 THEN BEGIN
        s = SIZE(map_out,/type) 
        IF s EQ 7 THEN BEGIN
           exists=FILE_TEST(map_out)
           IF exists EQ 1 THEN ok=0 ELSE ok=1
        ENDIF ELSE BEGIN
           help, map_out, level=-1, output=help_string
           no_exist = strpos(help_string,'UNDEFINED = <Undefined>') 
           IF no_exist EQ -1 THEN ok=0 ELSE ok=1
        ENDELSE
     ENDIF ELSE ok=1
  ENDIF ELSE ok=1
  IF ok EQ 0 THEN BEGIN
     MESSAGE, 'output already exists, set /overwrite to erase it.'
  ENDIF 

  IF NOT KEYWORD_SET(nside_out) THEN BEGIN
     nside_out = GET_NSIDE(map_in)
  ENDIF

  s = SIZE(map_in,/type)
  IF s EQ 7 THEN BEGIN
     IF NOT DEFINED(lmax) THEN lmax = FITSFILE_LMAX(map_in)
     IF lmax EQ -1 THEN BEGIN
        IF PSM_CAREFULNESS() EQ 0 THEN BEGIN
           pix = FITSFILE_PIX(map_in)
           nsidein = pix.nside
           lmax = 2L*nsidein
        ENDIF ELSE BEGIN
           PSM_DEBUG_MESSAGE, 'Input value for lmax keyword requested', routine='CHANGE_MAP_RESOLUTION', infolimit=0, stoplimit=0
        ENDELSE
     ENDIF
     beam_file = FITSFILE_BEAM(map_in, info_exists=info_exists)
     IF DEFINED(beam_in) THEN BEGIN
        IF beam_file.type NE 'UNKNOWN' THEN BEGIN
           IF NOT KEYWORD_SET(nocheck) THEN BEGIN
              IF ISPOLARISED(map_in,/map) THEN dim=3 ELSE dim=1
              bl_in = PSM_GET_BL(beam_in,lmax=lmax,double=double,dim=dim)
              bl_file = PSM_GET_BL(beam_file,lmax=lmax,double=double,dim=dim)
              equal = 1B
              FOR i=0, lmax DO equal = equal AND (bl_in[i] EQ bl_file[i])
              IF NOT equal THEN BEGIN
                 MESSAGE, 'Beam associated to the input file is different from input beam supplied by the header -- check your code'
              ENDIF
           ENDIF ELSE BEGIN
              ;; Do nothing. The beam in the file will be neglected, and the beam provided by keyword will be used
           ENDELSE
        ENDIF
     ENDIF ELSE BEGIN
        beam_in = beam_file
     ENDELSE

     alm_in = PSM_TMPFILENAME(extension='.fits')
     IF ISANUMBER(beam_in) THEN input_beam=GET_BEAM_STRUCT('GAUSSIAN',beamsize=beam_in,sizeunits='arcmin') ELSE input_beam=beam_in    
     PSM_MAP2ALM, map_in, alm_in, input_beam=input_beam, output_beam=beam_out, ordering=ordering_in, double=double, nohdr=nohdr, lmax=lmax, iter=iter   
     s = SIZE(map_out,/type)
     IF s EQ 7 THEN BEGIN
        PSM_ALM2MAP, alm_in, map_out, ordering=ordering_out, hpx_nside=nside_out, double=double, overwrite=overwrite, lmax=lmax
        ;;PSM_ALM2MAP, alm_in, map_value, ordering=ordering_out, hpx_nside=nside_out, double=double, overwrite=overwrite
        ;;ispol=ISPOLARISED(map_value,/map)
        ;;IF ispol THEN WRITE_HPXMAP, map_out, beam=input_beam, tqustokes=map_value, ordering=ordering ELSE WRITE_HPXMAP, map_out, tstokes=map_value, beam=input_beam, ordering=ordering

     ENDIF ELSE BEGIN
        PSM_ALM2MAP, alm_in, map_out, ordering=ordering_out, hpx_nside=nside_out, double=double, overwrite=overwrite
     ENDELSE

     FILE_DELETE, alm_in

  ENDIF ELSE BEGIN

     IF NOT DEFINED(lmax) THEN BEGIN
        IF PSM_CAREFULNESS() EQ 0 THEN BEGIN
           PSM_DEBUG_MESSAGE, 'Fix the case where the input map is a pointer', routine='CHANGE_MAP_RESOLUTION'
           npixin = N_ELEMENTS(map_in[*,0])
           nsidein = NPIX2NSIDE(npixin)
           lmax = 2L*nsidein
        ENDIF ELSE BEGIN
           PSM_DEBUG_MESSAGE, 'Input value for lmax keyword requested', routine='CHANGE_MAP_RESOLUTION', infolimit=0, stoplimit=0
        ENDELSE
     ENDIF

     alm_in = PSM_TMPFILENAME(extension='.fits')
     PSM_MAP2ALM, map_in, alm_in, input_beam=beam_in, output_beam=beam_out, ordering=ordering_in, double=double, nohdr=nohdr, lmax=lmax, iter=iter

     s = SIZE(map_out,/type)
     IF s EQ 7 THEN BEGIN
        PSM_ALM2MAP, alm_in, map_value, ordering=ordering_out, hpx_nside=nside_out, double=double, overwrite=overwrite
        ispol=ISPOLARISED(map_value,/map)
        IF ispol THEN WRITE_HPXMAP, map_out, beam=input_beam, tqustokes=map_value, ordering=ordering ELSE WRITE_HPXMAP, map_out, tstokes=map_value, beam=input_beam, ordering=ordering
     ENDIF ELSE BEGIN
        PSM_ALM2MAP, alm_in, map_out, ordering=ordering_out, hpx_nside=nside_out, double=double, overwrite=overwrite
     ENDELSE
     FILE_DELETE, alm_in
  ENDELSE

END
