#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <complex.h>
/*#include "util.h"
#include "options.h"
#include "window_defines.h"
#include "kernels.h"
#include "fastsum.h"
#include "nfft3.h"*/
#include "chealpix.h"
#include "nfft3.h"
#include "nfft3conf.h"
#include "nfft3util.h"
#include "kernels.h"
#include "fastsum.h"

/********************************************************************/
void norm_angle(double* , double*);

void get_fourier_mode(double , double complex* ,double complex*);

void get_all_temp_map_(int*, 
                       double complex*, 
                       double complex*, double complex*, 
		       int*, int*, int*, 
                       double*,
		       double*, double*, double*, 
		       double*);

void get_temp_map_(int*, 
                   double complex*, 
                   double complex*, double complex*, 
		   int*, int*, int*, 
                   double*,
		   double*, double*, double*);


void get_lensed_temp_map_(int*, 
                          double complex*, 
		          int*, int*, int*, 
                          double*,
		          double*, double*, double*, 
		          double*);

void get_all_pol_map_(int*, 
                      double complex*, double complex*,
                      double complex*, double complex*,
		      int*, int*, int*,
                      double*, double*, double*,
		      double*, double*, double*, 
		      double*, double*, double*);

void get_pol_map_(int*, 
                  double complex*, double complex*,
                  double complex*,double complex*,
		  int*, int*, int*,
                  double*, double*, double*,
		  double*, double*, double*);

void get_lensed_pol_map_(int*, 
                         double complex*, double complex*,
		         int*, int*, int*,
                         double*, double*, double*,
		         double*, double*, double*, 
		         double*, double*, double*);

/********************************************************************/


void get_all_temp_map_(int *lm, double complex *tms, 
                       double complex *phms, double complex *gms, 
		       int *ns, int *sig, int *kap, 
                       double *ulmap,
		       double *phtp, double *gthtp, double *gphtp, 
		       double *lmap){


  /*Declaration of local variable*/

  int np;

  nfft_plan pl;
  double th, ph, thnew, phnew, gtp, sinc_gtp;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given nside*/

  np=nside2npix(*ns);

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computaion of non-equispaced nodes before lensing
  The the range of  new theta and phi are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(ph<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    /*norm_angle(&th,&ph);*/
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  timer2=time(NULL);

  printf(" time to compute healpix grid points for unlensed map %le\n",
                                             (double)(timer2-timer1));

  printf(" Ntotal, Mtotal  = %d %d\n",pl.N_total,pl.M_total);

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for unlensed map %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  } 
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for ulesned cmb %le\n",
                                      (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo for ulesned cmb %le\n",
                               (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero because temperature T(\hat n) 
  is a real field on the surface of a sphere*/

  for (i = 0;i<=(np-1);i++){
    ulmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in unlesned map %le\n",(double)(timer2-timer1));


  /*************************************************************/
  timer1=time(NULL);

  /*Computation of fourier modes for lensing potential before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(phms[p]);
     }
  } 
  /*get_fourier_mode(*lm,phms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensing potential %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for lensing potential before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo for lensing potential %le\n",
                                     (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of lensing potential at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature \Phi(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    phtp[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensing potential map %le\n",
                              (double)(timer2-timer1));


  /*************************************************************/

  timer1=time(NULL);

  /*Computation of fourier modes for deflection before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(gms[p]);
     }
  }
  /*get_fourier_mode(*lm,gms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for deflection field %le\n",
                                           (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of deflection at the non-equispaced nodes before lensing
   Real part of pl.f[i] is the \theta component of deflection field
   imaginary part of  pl.f[i] is the \phi component of deflection field*/  

  for (i = 0;i<=(np-1);i++){
    gthtp[i]=creal(pl.f[i]);
    gphtp[i]=cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in deflection field map %le\n",
                              (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of non-equispaced nodes after lensing
  New theta and phi are first computed from old theta and phi, and deflection field
  using the identities of spherical triangle. The the range of  new theta and phi 
  are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    gtp=sqrt(pow(gthtp[i],2)+pow(gphtp[i],2));
    if(gtp > 0.0){
      sinc_gtp=(sin(gtp)/gtp);
      thnew=acos(cos(gtp)*cos(th)-sinc_gtp*sin(th)*gthtp[i]);
      if(sqrt(sin(thnew)*sin(thnew))>pow(10.0,-10)){
        phnew=ph+asin(sinc_gtp*gphtp[i]/sin(thnew));
      }
      else{
        phnew=ph;
      }
    }
    else{
      thnew=th;
      phnew=ph;
    }

    if(thnew>=2.0*PI){
      thnew=thnew-2.0*PI;    
    }

    if(thnew<0.0){
      thnew=2.0*PI+thnew;
    }

    if(phnew>=2.0*PI){
      phnew=phnew-2.0*PI; 
    }
    
    if(phnew<0.0){
      phnew=2.0*PI+phnew;
    }
   
    thnew=-0.5+thnew/(2.0*PI);
    phnew=-0.5+phnew/(2.0*PI);
    /*norm_angle(&thnew,&phnew);*/
    pl.x[2*i]=phnew;
    pl.x[2*i+1]=thnew;
  }

  timer2=time(NULL);

  printf(" time to compute new theta and phi for lensed map %le\n",
                                         (double)(timer2-timer1));

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for lensed map %le\n",
                                           (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensed map %le\n",
                                     (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
    Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
    is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    lmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensed map %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft%le\n",(double)(timer2-timer1));
}

void get_all_pol_map_(int *lm, double complex *tms, double complex *pms,
                      double complex *phms,double complex *gms,
		      int *ns, int *sig, int *kap,
                      double *ultmap, double *ulqmap, double *ulumap,
		      double *phtp, double *gthtp, double *gphtp, 
		      double *ltmap, double *lqmap, double *lumap){

  int np;
  double *crot,*srot;
  double tamg;

  nfft_plan pl;
  double th, ph, thnew, phnew, gtp, sinc_gtp;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given te nside*/

  np=nside2npix(*ns); 

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  printf(" sigma = %d kappa = %d\n",*sig,*kap);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computaion of non-equisapced nodes before lensing
  The the range of  new theta and phi are scaled down to [-0.5,0.5]*[-0.5,0.5]*/

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(ph<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    /*norm_angle(&th,&ph);*/
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  timer2=time(NULL);

  printf(" time to compute healpix grid points for unlensed map %le\n",
                                             (double)(timer2-timer1));

  printf(" Ntotal, Mtotal  = %d %d\n",pl.N_total,pl.M_total);

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for unlensed map %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for unlensed temperture map %le\n",
                                                 (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    ultmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf("output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed for unlensed temperature map  %le\n",
                                   (double)(timer2-timer1));

  /*************************************************************/
  timer1=time(NULL);

  /*Computation of fourier modes for lensing potential before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(phms[p]);
     }
  }
  /*get_fourier_mode(*lm,phms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes lensing potential %le\n",
                                        (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for lensing potential before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of lensing potential at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature \Phi(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    phtp[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf("output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensing potential map %le\n",
                               (double)(timer2-timer1));


  /*************************************************************/


  timer1=time(NULL);

  /*Computation of fourier modes for polarization before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(pms[p]);
    }
  }
  /*get_fourier_mode(*lm,pms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for unlesned polarization map %le\n",
                                                   (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for polarization before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of polarization at the non-equispaced nodes before lensing*/

  for (i = 0;i<=(np-1);i++){
    ulqmap[i]= creal(pl.f[i]);
    ulumap[i]= cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in unlensed polarization map %le\n",
                                      (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for deflection field before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(gms[p]);
     }
  }
  /*get_fourier_mode(*lm,gms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for deflection field map %le\n",
                                               (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection vector*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of deflection at the non-equispaced nodes before lensing
   Real part of pl.f[i] is the \theta component of deflection field
   imaginary part of  pl.f[i] is the \phi component of deflection field*/

  for (i = 0;i<=(np-1);i++){
    gthtp[i]=creal(pl.f[i]);
    gphtp[i]=cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in deflection field  map %le\n",
                              (double)(timer2-timer1));

  timer1=time(NULL);

  crot=(double *)calloc(np,sizeof(double));
  srot=(double *)calloc(np,sizeof(double));

  /*Computation of non-equispaced nodes after lensing
  New theta and phi are first computed from old theta and phi, and deflection field
  using the identities of spherical triangle. The the range of  new theta and phi 
  are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    gtp=sqrt(pow(gthtp[i],2)+pow(gphtp[i],2));
    if(gtp > 0.0){
      sinc_gtp=(sin(gtp)/gtp);
      /*Computation of cosine and sin of rotation angle for polarization basis*/
      tamg=gphtp[i]*sin(th)/(gtp*gtp*sinc_gtp*cos(th)+gthtp[i]*cos(gtp)*sin(th));
      /*crot[i]=2.0*pow((gthtp[i]+gphtp[i]*tamg),2)/(gtp*gtp*(1+pow(tamg,2))) - 1.0;
      srot[i]=2.0*(gthtp[i]+gphtp[i]*tamg)*(gphtp[i]-gthtp[i]*tamg)/(gtp*gtp*(1.0+pow(tamg,2)));
      */
      crot[i]=2.0*pow((gthtp[i]+gphtp[i]*tamg)/gtp,2)/(1.0+pow(tamg,2)) - 1.0;
      srot[i]=2.0*((gthtp[i]+gphtp[i]*tamg)/gtp)*((gphtp[i]-gthtp[i]*tamg)/gtp)/(1.0+pow(tamg,2));
      /**/
      thnew=acos(cos(gtp)*cos(th)-sinc_gtp*sin(th)*gthtp[i]);
      if(sqrt(sin(thnew)*sin(thnew))>pow(10.0,-10)){
        phnew=ph+asin(sinc_gtp*gphtp[i]/sin(thnew));
      }
      else{
        phnew=ph;
      }
    }
    else{
      thnew=th;
      phnew=ph;
    }

    if(thnew>=2.0*PI){
      thnew=thnew-2.0*PI;    
    }

    if(thnew<0.0){
      thnew=2.0*PI+thnew;
    }

    if(phnew>=2.0*PI){
      phnew=phnew-2.0*PI;  
    }
    
    if(phnew<0.0){
      phnew=2.0*PI+phnew;
    }
   
    thnew=-0.5+thnew/(2.0*PI);
    phnew=-0.5+phnew/(2.0*PI);
    /*norm_angle(&thnew,&phnew);*/
    pl.x[2*i]=phnew;
    pl.x[2*i+1]=thnew;
  }

  timer2=time(NULL);

  printf(" time to compute new theta and phi for lensed map %le\n",
                                         (double)(timer2-timer1));

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for lensed map %le\n",
                                          (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }  
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/

  timer2=time(NULL);

  printf(" time to feed in fourier modes for lesned temperature map %le\n",
                                                (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes after lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere
  */

  for (i = 0;i<=(np-1);i++){
    ltmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed for lensed temperature map %le\n",
                    (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for polarization after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(pms[p]);
     }
  }
 
  /*get_fourier_mode(*lm,pms,pl.f_hat);*/

  timer2=time(NULL);

  printf(" time to feed in fourier modes for lesned polarization map %le\n",
(double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for polarization after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of polarization at the non-equispaced nodes after lensing*/

  for (i = 0;i<=(np-1);i++){
    lqmap[i]=crot[i]*creal(pl.f[i])-srot[i]*cimag(pl.f[i]);
    lumap[i]=srot[i]*creal(pl.f[i])+crot[i]*cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in lensed polarization map %le\n",
                                (double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft %le\n",(double)(timer2-timer1));

  free(crot);
  free(srot);

}

void norm_angle(double *th_ang, double *ph_ang){
  /*Computation of non-equispaced nodes:
  The the range of  new theta and phi are scaled down 
  to [-0.5,0.5]*[-0.5,0.5]*/

  double temp;

  if(*th_ang>=2.0*PI){
    temp=*th_ang;
    temp=temp-2.0*PI;
    *th_ang=temp;
  }
  if(*th_ang<0.0){
    temp=*th_ang;
    temp=2.0*PI+temp;
    *th_ang=temp;
  }
  if(*ph_ang>=2.0*PI){
    temp=*ph_ang;
    temp=temp-2.0*PI;
    *ph_ang=temp; 
  }
  if(*ph_ang<0.0){
    temp=*ph_ang;
    temp=2.0*PI+temp;
    *ph_ang=temp;
  }
  *th_ang=-0.5+*th_ang/(2.0*PI);
  *ph_ang=-0.5+*ph_ang/(2.0*PI);
}

void get_fourier_mode(double blimit, double complex *hmode,double complex *fmode){
  /*Computation of 2-d fourier modes from harmonic modes*/

  int m,s,p,q;
  int delp, mps;

  fmode[0]=0.0;

  for(s=0;s<=2*(blimit);s++){
    q=(s+1);
    fmode[q]=0.0;
  }

  for(m=0;m<=2*(blimit);m++){
    q=2*(blimit+1)*(m+1);
    fmode[q]=0.0;
  }

  for(m=0;m<=2*(blimit);m++){
    delp=(2*(blimit)+m+3);
    for(s=0;s<=2*(blimit);s++){
      p=(2*(blimit)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      fmode[q]=(double)(1-2*mps)*conj(hmode[p]);
     }
  }  

}



void get_temp_map_(int *lm, double complex *tms, 
                   double complex *phms, double complex *gms, 
		   int *ns, int *sig, int *kap, 
                   double *ulmap,
		   double *phtp, double *gthtp, double *gphtp){


  /*Declaration of local variable*/

  int np;

  nfft_plan pl;
  double th, ph;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given nside*/

  np=nside2npix(*ns);

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computaion of non-equispaced nodes before lensing
  The the range of  new theta and phi are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(ph<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    /*norm_angle(&th,&ph);*/
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  timer2=time(NULL);

  printf(" time to compute healpix grid points for unlensed map %le\n",
                                             (double)(timer2-timer1));

  printf(" Ntotal, Mtotal  = %d %d\n",pl.N_total,pl.M_total);

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for unlensed map %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  } 
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for ulesned cmb %le\n",
                                      (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo for ulesned cmb %le\n",
                               (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero because temperature T(\hat n) 
  is a real field on the surface of a sphere*/

  for (i = 0;i<=(np-1);i++){
    ulmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in unlensed map %le\n",(double)(timer2-timer1));


  /*************************************************************/
  timer1=time(NULL);

  /*Computation of fourier modes for lensing potential before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(phms[p]);
     }
  } 
  /*get_fourier_mode(*lm,phms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensing potential %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for lensing potential before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo for lensing potential %le\n",
                                     (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of lensing potential at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature \Phi(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    phtp[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensing potential map %le\n",
                              (double)(timer2-timer1));


  /*************************************************************/

  timer1=time(NULL);

  /*Computation of fourier modes for deflection before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(gms[p]);
     }
  }
  /*get_fourier_mode(*lm,gms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for deflection field %le\n",
                                           (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of deflection at the non-equispaced nodes before lensing
   Real part of pl.f[i] is the \theta component of deflection field
   imaginary part of  pl.f[i] is the \phi component of deflection field*/  

  for (i = 0;i<=(np-1);i++){
    gthtp[i]=creal(pl.f[i]);
    gphtp[i]=cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in deflection field map %le\n",
                              (double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft%le\n",(double)(timer2-timer1));
}

void get_lensed_temp_map_(int *lm, 
                          double complex *tms, 
		          int *ns, int *sig, int *kap, 
                          double *ulmap,
		          double *phtp, double *gthtp, double *gphtp, 
		          double *lmap){


  /*Declaration of local variable*/

  int np;

  nfft_plan pl;
  double th, ph, thnew, phnew, gtp, sinc_gtp;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given nside*/

  np=nside2npix(*ns);

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of non-equispaced nodes after lensing
  New theta and phi are first computed from old theta and phi, and deflection field
  using the identities of spherical triangle. The the range of  new theta and phi 
  are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    gtp=sqrt(pow(gthtp[i],2)+pow(gphtp[i],2));
    if(gtp > 0.0){
      sinc_gtp=(sin(gtp)/gtp);
      thnew=acos(cos(gtp)*cos(th)-sinc_gtp*sin(th)*gthtp[i]);
      if(sqrt(sin(thnew)*sin(thnew))>pow(10.0,-10)){
        phnew=ph+asin(sinc_gtp*gphtp[i]/sin(thnew));
      }
      else{
        phnew=ph;
      }
    }
    else{
      thnew=th;
      phnew=ph;
    }

    if(thnew>=2.0*PI){
      thnew=thnew-2.0*PI;    
    }

    if(thnew<0.0){
      thnew=2.0*PI+thnew;
    }

    if(phnew>=2.0*PI){
      phnew=phnew-2.0*PI; 
    }
    
    if(phnew<0.0){
      phnew=2.0*PI+phnew;
    }
   
    thnew=-0.5+thnew/(2.0*PI);
    phnew=-0.5+phnew/(2.0*PI);
    /*norm_angle(&thnew,&phnew);*/
    pl.x[2*i]=phnew;
    pl.x[2*i+1]=thnew;
  }

  timer2=time(NULL);

  printf(" time to compute new theta and phi for lensed map %le\n",
                                         (double)(timer2-timer1));

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for lesned map %le\n",
                                           (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensed map %le\n",
                                     (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
    Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
    is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    lmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensed map %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft%le\n",(double)(timer2-timer1));
}

void get_pol_map_(int *lm, 
                  double complex *tms, double complex *pms,
                  double complex *phms,double complex *gms,
		  int *ns, int *sig, int *kap,
                  double *ultmap, double *ulqmap, double *ulumap,
		  double *phtp, double *gthtp, double *gphtp){

  int np;

  nfft_plan pl;
  double th, ph, gtp, sinc_gtp;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given te nside*/

  np=nside2npix(*ns); 

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  printf(" sigma = %d kappa = %d\n",*sig,*kap);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computaion of non-equisapced nodes before lensing
  The the range of  new theta and phi are scaled down to [-0.5,0.5]*[-0.5,0.5]*/

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    if(th>=2.0*PI){
      th=th-2.0*PI;    
    }
    if(th<0.0){
      th=2.0*PI+th;
    }
    if(ph>=2.0*PI){
      ph=ph-2.0*PI; 
    }
    if(ph<0.0){
      ph=2.0*PI+ph;
    }
    th=-0.5+th/(2.0*PI);
    ph=-0.5+ph/(2.0*PI);
    /*norm_angle(&th,&ph);*/
    pl.x[2*i]=ph;
    pl.x[2*i+1]=th;
  }

  timer2=time(NULL);

  printf(" time to compute healpix grid points for unlensed map %le\n",
                                             (double)(timer2-timer1));

  printf(" Ntotal, Mtotal  = %d %d\n",pl.N_total,pl.M_total);

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for unlensed map %le\n",
                                            (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for unlensed temperture map %le\n",
                                                 (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    ultmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed for unlensed temperature map  %le\n",
                                   (double)(timer2-timer1));

  /*************************************************************/
  timer1=time(NULL);

  /*Computation of fourier modes for lensing potential before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */


  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(phms[p]);
     }
  }
  /*get_fourier_mode(*lm,phms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes lensing potential %le\n",
                                        (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for lensing potential before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of lensing potential at the non-equispaced nodes before lensing
  Imaginary part of pl.f[i] should be zero, because temperature \Phi(\hat n) 
  is a real field on the surface of sphere*/

  for (i = 0;i<=(np-1);i++){
    phtp[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed in lensing potential map %le\n",
                               (double)(timer2-timer1));


  /*************************************************************/


  timer1=time(NULL);

  /*Computation of fourier modes for polarization before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(pms[p]);
    }
  }
  /*get_fourier_mode(*lm,pms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for unlesned polarization map %le\n",
                                                   (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for polarization before lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of polarization at the non-equispaced nodes before lensing*/

  for (i = 0;i<=(np-1);i++){
    ulqmap[i]= creal(pl.f[i]);
    ulumap[i]= cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in unlensed polarization map %le\n",
                                      (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for deflection field before lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(gms[p]);
     }
  }
  /*get_fourier_mode(*lm,gms,pl.f_hat);*/
  timer2=time(NULL);

  printf(" time to feed in fourier modes for deflection field map %le\n",
                                               (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for deflection vector*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of deflection at the non-equispaced nodes before lensing
   Real part of pl.f[i] is the \theta component of deflection field
   imaginary part of  pl.f[i] is the \phi component of deflection field*/

  for (i = 0;i<=(np-1);i++){
    gthtp[i]=creal(pl.f[i]);
    gphtp[i]=cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in deflection field  map %le\n",
                              (double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft %le\n",(double)(timer2-timer1));

}


void get_lensed_pol_map_(int *lm, 
                         double complex *tms, double complex *pms,
 		         int *ns, int *sig, int *kap,
                         double *ultmap, double *ulqmap, double *ulumap,
		         double *phtp, double *gthtp, double *gphtp, 
		         double *ltmap, double *lqmap, double *lumap){

  int np;
  double *crot,*srot;
  double tamg;

  nfft_plan pl;
  double th, ph, thnew, phnew, gtp, sinc_gtp;
  int N[2],n[2];
  int i,m,s,p,q,mps,delp;
  time_t timer1,timer2;

  /*HEALPix C Subroutine to compute number of pixel given te nside*/

  np=nside2npix(*ns); 

  /* Dimensions of fourier modes */

  N[0]=2*(*lm+1);
  N[1]=N[0];
  n[0]=(*sig)*N[0];
  n[1]=(*sig)*N[1];

  timer1=time(NULL);

  printf(" sigma = %d kappa = %d\n",*sig,*kap);

  /*Initialization of NFFT*/

  nfft_init_guru(&pl,2, N, np, n,*kap,
		 PRE_PHI_HUT| PRE_PSI| MALLOC_F_HAT| MALLOC_X| MALLOC_F |
		 FFTW_INIT| FFT_OUT_OF_PLACE,
		 FFTW_ESTIMATE| FFTW_DESTROY_INPUT);

  timer2=time(NULL);

  printf(" time to initialize nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  crot=(double *)calloc(np,sizeof(double));
  srot=(double *)calloc(np,sizeof(double));

  /*Computation of non-equispaced nodes after lensing
  New theta and phi are first computed from old theta and phi, and deflection field
  using the identities of spherical triangle. The the range of  new theta and phi 
  are scaled down to [-0.5,0.5]*[-0.5,0.5]
  */

  for (i = 0;i<=(np-1);i++){
    pix2ang_ring(*ns,i,&th,&ph);
    gtp=sqrt(pow(gthtp[i],2)+pow(gphtp[i],2));
    if(gtp > 0.0){
      sinc_gtp=(sin(gtp)/gtp);
      /*Computation of cosine and sin of rotation angle for polarization basis*/
      tamg=gphtp[i]*sin(th)/(gtp*gtp*sinc_gtp*cos(th)+gthtp[i]*cos(gtp)*sin(th));
      /*crot[i]=2.0*pow((gthtp[i]+gphtp[i]*tamg),2)/(gtp*gtp*(1+pow(tamg,2))) - 1.0;
      srot[i]=2.0*(gthtp[i]+gphtp[i]*tamg)*(gphtp[i]-gthtp[i]*tamg)/(gtp*gtp*(1.0+pow(tamg,2)));
      */
      crot[i]=2.0*pow((gthtp[i]+gphtp[i]*tamg)/gtp,2)/(1.0+pow(tamg,2)) - 1.0;
      srot[i]=2.0*((gthtp[i]+gphtp[i]*tamg)/gtp)*((gphtp[i]-gthtp[i]*tamg)/gtp)/(1.0+pow(tamg,2));
      /**/
      thnew=acos(cos(gtp)*cos(th)-sinc_gtp*sin(th)*gthtp[i]);
      if(sqrt(sin(thnew)*sin(thnew))>pow(10.0,-10)){
        phnew=ph+asin(sinc_gtp*gphtp[i]/sin(thnew));
      }
      else{
        phnew=ph;
      }
    }
    else{
      thnew=th;
      phnew=ph;
    }

    if(thnew>=2.0*PI){
      thnew=thnew-2.0*PI;    
    }

    if(thnew<0.0){
      thnew=2.0*PI+thnew;
    }

    if(phnew>=2.0*PI){
      phnew=phnew-2.0*PI;  
    }
    
    if(phnew<0.0){
      phnew=2.0*PI+phnew;
    }
   
    thnew=-0.5+thnew/(2.0*PI);
    phnew=-0.5+phnew/(2.0*PI);
    /*norm_angle(&thnew,&phnew);*/
    pl.x[2*i]=phnew;
    pl.x[2*i+1]=thnew;
  }

  timer2=time(NULL);

  printf(" time to compute new theta and phi for lensed map %le\n",
                                         (double)(timer2-timer1));

  timer1=time(NULL);

  /*Checks a transform plan for frequently used bad parameter*/

  nfft_check(&pl);

  timer2=time(NULL);

  printf(" time to check nfft %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Precompute psi, the entries of the matrix B*/

  if(pl.nfft_flags & PRE_ONE_PSI) nfft_precompute_one_psi(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_precompute_one_psi for lensed map %le\n",
                                          (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for temperature after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(tms[p]);
     }
  }  
  /*get_fourier_mode(*lm,tms,pl.f_hat);*/

  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensed temperature map %le\n",
                                                (double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for temperature after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do  nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of temperature at the non-equispaced nodes after lensing
  Imaginary part of pl.f[i] should be zero, because temperature T(\hat n) 
  is a real field on the surface of sphere
  */

  for (i = 0;i<=(np-1);i++){
    ltmap[i]= creal(pl.f[i]);
    if(sqrt(cimag(pl.f[i])*cimag(pl.f[i]))>pow(10.0,-10)){
      printf(" output data is not real\n");
    }
  }

  timer2=time(NULL);

  printf(" time to feed for lensed temperature map %le\n",
                    (double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of fourier modes for polarization after lensing
  The extra factor (double)(1-2*mps) and complex conjugation are required  
  to go from [0,2*\pi]*[0,2*\pi] grid to [-0.5,0.5]*[-0.5,0.5] grid
  */

  pl.f_hat[0]=0.0;

  for(s=0;s<=2*(*lm);s++){
    q=(s+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    q=2*(*lm+1)*(m+1);
    pl.f_hat[q]=0.0;
  }

  for(m=0;m<=2*(*lm);m++){
    delp=(2*(*lm)+m+3);
    for(s=0;s<=2*(*lm);s++){
      p=(2*(*lm)+1)*m+s;
      q=p+delp;
      mps=(m+s)%2;
      pl.f_hat[q]=(double)(1-2*mps)*conj(pms[p]);
     }
  }
 
  /*get_fourier_mode(*lm,pms,pl.f_hat);*/

  timer2=time(NULL);

  printf(" time to feed in fourier modes for lensed polarization map %le\n",
(double)(timer2-timer1));

  timer1=time(NULL);

  /*Do nfft for polarization after lensing*/

  nfft_trafo(&pl);

  timer2=time(NULL);

  printf(" time to do nfft_trafo %le\n",(double)(timer2-timer1));

  timer1=time(NULL);

  /*Computation of polarization at the non-equispaced nodes after lensing*/

  for (i = 0;i<=(np-1);i++){
    lqmap[i]=crot[i]*creal(pl.f[i])-srot[i]*cimag(pl.f[i]);
    lumap[i]=srot[i]*creal(pl.f[i])+crot[i]*cimag(pl.f[i]);
  }

  timer2=time(NULL);

  printf(" time to feed in lensed polarization map %le\n",
                                (double)(timer2-timer1));

  timer1=time(NULL);

  /*Finalization of NFFT*/

  nfft_finalize(&pl);

  timer2=time(NULL);

  printf(" time to finalize nfft %le\n",(double)(timer2-timer1));

  free(crot);
  free(srot);

}
