PRO COADD_SKYINBAND_MAPS, maplist, outputmap, hdr=hdr, overwrite=overwrite, help=help

  nmaps = N_ELEMENTS(maplist)
  IF nmaps EQ 0 THEN BEGIN
     MESSAGE, 'Empty map list'
  ENDIF

  IF PSM_CAREFULNESS() GE 1 THEN BEGIN
     ;; Check that maps exist
     mapsok=1B
     FOR i=0, nmaps DO BEGIN
        ok = FILE_TEST(maplist[i])
        IF NOT ok THEN mapsok=0B
     ENDFOR
     IF mapsok EQ 0B THEN MESSAGE, 'Some maps are not defined'
  ENDIF

  IF nmaps EQ 1 THEN BEGIN
     IF NOT DEFINED(outputmap) THEN BEGIN
        READ_MAP, maplist[0], outputmap
        hdr = HEADFITS(maplist[0], exten=1)
     ENDIF ELSE BEGIN
        IF isstring(outputmap) THEN SPAWN, 'cp ' + maplist[0] + ' ' + outputmap
        hdr = HEADFITS(maplist[0], exten=1)
     ENDELSE
     GOTO, closing
  ENDIF

  ;; Check units
  ;;-------------
  h0 = HEADFITS(maplist[0],exten=1)
  units0 = STRTRIM(SXPAR(h0, 'TUNIT*', count=count),2)
  IF count EQ 0 THEN units0 = 'unknown'
  res = PSM_UNIQ(units0)
  IF N_ELEMENTS(res) NE 1 THEN MESSAGE, 'Incompatible units in input maps'

  FOR i=1, nmaps-1 DO BEGIN
     h = HEADFITS(maplist[i],exten=1)
     units = STRTRIM(SXPAR(h, 'TUNIT*', count=count),2)
     IF count EQ 0 THEN units = 'unknown'
     wh = WHERE(units NE units0[0], nwh)
     IF nwh NE 0 THEN MESSAGE, 'Incompatible units in input maps'
  ENDFOR

  ;; Check beams
  ;;-------------
  beam0 = FITSFILE_BEAM(maplist[0], info_exists=info_exists, exten=1)
  IF NOT info_exists THEN beam0 = GET_BEAM_STRUCT('UNKNOWN')
  FOR i=1, nmaps-1 DO BEGIN
     beam = FITSFILE_BEAM(maplist[i], info_exists=info_exists, exten=1)
     IF NOT info_exists THEN beam = GET_BEAM_STRUCT('UNKNOWN')
     ok = BEAMS_EQUAL(beam0, beam)
     DESTROY_DATA, beam
     IF NOT ok THEN BEGIN
        DESTROY_DATA, beam0
        MESSAGE, 'Incompatible beams in input maps'
     ENDIF
  ENDFOR
  
  ;; Check bands
  ;;-------------
  band0 = FITSFILE_BAND(maplist[0], info_exists=info_exists, exten=1)
  IF NOT info_exists THEN band0 = GET_BAND_STRUCT('UNKNOWN')
  FOR i=1, nmaps-1 DO BEGIN
     band = FITSFILE_BAND(maplist[i], info_exists=info_exists, exten=1)
     IF NOT info_exists THEN band = GET_BAND_STRUCT('UNKNOWN')
     ok = BANDS_EQUAL(band0, band)
     DESTROY_DATA, band
     IF NOT ok THEN BEGIN
        DESTROY_DATA, band0
        MESSAGE, 'Incompatible bands in input maps'
     ENDIF
  ENDFOR

  ;; Check lmax
  ;;------------
  lmax0 = FITSFILE_LMAX(maplist[0])
  FOR i=1, nmaps-1 DO BEGIN
     lmax = FITSFILE_LMAX(maplist[i])
     IF lmax NE lmax0 THEN MESSAGE, 'Incompatible lmax in input maps'
  ENDFOR

  ;; Check pixel window function
  ;;-----------------------------
  pxwin0 = FITSFILE_PXWIN(maplist[0])
  FOR i=1, nmaps-1 DO BEGIN
     pxwin = FITSFILE_PXWIN(maplist[i])
     IF pxwin NE pxwin0 THEN MESSAGE, 'Incompatible pixel windows in input maps'
  ENDFOR

  ;; Check Healpix header
  ;;----------------------
  pix0 = FITSFILE_PIX(maplist[0])
  FOR i=1, nmaps-1 DO BEGIN
     pix = FITSFILE_PIX(maplist[i])
     IF NOT STRUCT_EQUAL(pix,pix0) THEN MESSAGE, 'Incompatible pixelisations in input maps'
  ENDFOR

  ;; Co-adding maps
  ;;----------------
  pol = BYTARR(nmaps)
  FOR i=0, nmaps-1 DO pol[i] = ISPOLARISED(maplist[i],/map)

  polar = 0B
  FOR i=0, nmaps-1 DO polar=polar OR pol[i]
  
  whp = WHERE(pol,npol,complement=whn, ncomplement=nnot)

  IF polar THEN BEGIN
     READ_MAP, maplist[whp[0]], map, hdr=hh
  ENDIF ELSE BEGIN
     READ_MAP, maplist[whn[0]], map, hdr=hh
  ENDELSE

  ;: PSM header
    SPLIT_FITSHDR, hh, hout=hout, hpsm=hpsm, hend=hend
    EXTRACT_PSMHDRBLOCK, hpsm, hbase, blockname='base', whereblock=whbase
    UPDATEHDRPAR, hbase, 'PSM_VERS', PSM_VERSION()
    UPDATEHDRPAR, hbase, 'PSM_RNID', PSM_ID()
    PSM_ID_KEYGEN, key & UPDATEHDRPAR, hbase, 'PSM_FLID', key
    EXTRACT_PSMHDRBLOCK, hpsm, hobs, blockname='obs', whereblock=whobs
    comps = SXPAR(hobs, 'CMIX*', count=count)
    EXTRACT_PSMHDRBLOCK, hpsm, hband, blockname='band', whereblock=whband
    EXTRACT_PSMHDRBLOCK, hpsm, hbeam, blockname='beam', whereblock=whbeam
    EXTRACT_PSMHDRBLOCK, hpsm, hmap, blockname='map', whereblock=whmap
  ;;------------

  IF polar THEN BEGIN
     IF npol GE 2 THEN BEGIN
        FOR j=1,npol-1 DO BEGIN
           READ_MAP, maplist[whp[j]], maptoadd, hdr=hhtoadd
           map = TEMPORARY(map) + TEMPORARY(maptoadd)
           EXTRACT_PSMHDRBLOCK, hhtoadd, hobs2, blockname='obs', whereblock=whobs2
           IF whobs2[0] NE -1 THEN BEGIN
              compstoadd = SXPAR(hobs2, 'CMIX*', count=count)
              IF count NE 0 THEN comps = [comps,compstoadd] ELSE STOP
           ENDIF ELSE STOP
        ENDFOR
     ENDIF
     IF nnot NE 0 THEN BEGIN
        FOR j=0,nnot-1 DO BEGIN
           READ_MAP, maplist[whn[j]], maptoadd, hdr=hhtoadd
           map[*,0] = map[*,0] + TEMPORARY(maptoadd)
           EXTRACT_PSMHDRBLOCK, hhtoadd, hobs2, blockname='obs', whereblock=whobs2
           IF whobs2[0] NE -1 THEN BEGIN
              compstoadd = SXPAR(hobs2, 'CMIX*', count=count)
              IF count NE 0 THEN comps = [comps,compstoadd] ELSE STOP
           ENDIF ELSE STOP
        ENDFOR
     ENDIF
  ENDIF ELSE BEGIN
     IF nnot GE 2 THEN BEGIN
        FOR j=1,nnot-1 DO BEGIN
           READ_MAP, maplist[whn[j]], maptoadd, hdr=hhtoadd
           map[*,0] = map[*,0] + TEMPORARY(maptoadd)
           EXTRACT_PSMHDRBLOCK, hhtoadd, hobs2, blockname='obs', whereblock=whobs2
           IF whobs2[0] NE -1 THEN BEGIN
              compstoadd = SXPAR(hobs2, 'CMIX*', count=count)
              IF count NE 0 THEN comps = [comps,compstoadd] ELSE STOP
           ENDIF ELSE STOP
        ENDFOR
     ENDIF
  ENDELSE
  
  comps = STRTRIM(comps,2)
  PSM_ID_KEYGEN, obsid
  hobs = PSM_OBS_HDR(comps,obsid=obsid)

  MK_PSMHDR, hdrpsm, hbase, hobs, hmap, hband, hbeam
  hdr = MERGE_FITSHDRS(hout,hdrpsm)

  IF NOT DEFINED(outputmap) THEN BEGIN
     outputmap = TEMPORARY(map)
  ENDIF ELSE BEGIN

     ;; Output in file format
     ;;-----------------------
     IF ISSTRING(outputmap) THEN BEGIN
        exists = FILE_TEST(outputmap)
        IF exists THEN BEGIN
           IF NOT KEYWORD_SET(overwrite) THEN BEGIN
              PSM_INFO_MESSAGE, "outputmap already exists, set /overwrite to overwrite it"
              PSM_INFO_MESSAGE, "File NOT written"
              GOTO, closing
           ENDIF
           IF ISPOLARISED(map,/map) THEN WRITE_HPXMAP, outputmap, tqustokes=map, xhdr=hdr ELSE WRITE_HPXMAP, outputmap, tstokes=map, xhdr=hdr            
        ENDIF
     ENDIF

     ;; Output in array format
     ;;------------------------
     IF ISANUMBER(outputmap) THEN BEGIN
        IF NOT KEYWORD_SET(overwrite) THEN BEGIN
           PSM_INFO_MESSAGE, "outputmap already exists, set /overwrite to overwrite it"
           PSM_INFO_MESSAGE, "File NOT written"
           GOTO, closing
        ENDIF ELSE BEGIN
           outputmap = TEMPORARY(map)
        ENDELSE
     ENDIF
     STOP
  ENDELSE
  


closing:
END
