FUNCTION IS_ONE_DIMENSIONAL, comp

  answer = 1B
  IF comp.nlaw NE 1 THEN RETURN, 1B - answer

  params = EMLAW_PARAMLIST(comp.e1.law)
  FOR i=0, N_ELEMENTS(params)-1 DO answer = answer AND  (NOT ISFITSFILE(*(EMISSIONPARAM(comp,params[i]+'.value',1))))
  
  RETURN, answer

END
;;---------------------------------------------------------------------------------------------------------------------------


PRO ONE_DIM_COMP_IN_BAND, comp, band, output, e_index=e_index, units=units, lmax=lmax, hpx_nside=hpx_nside, dtfm=dtfm, pointer=pointer, double=double, polarised=polarised

  ;IF PSM_DEBUG_VALUE() LE 1 THEN ON_ERROR, 2


  ;; Case where we observe the first emission law 
  ;;----------------------------------------------
  IF NOT KEYWORD_SET(e_index) THEN BEGIN

     IF comp.nlaw NE 1 THEN MESSAGE, 'component must be one-dimensional in call to ONE_DIM_COMP_IN_BAND'
     
     file = *(comp.e1.ampl.value)
     file_dtfm = PSM_DATAFORMAT(file)
     
     CASE dtfm OF
        
        ;; Output format is an alm file
        'alm': BEGIN
           CASE file_dtfm OF
              ;; Amplitude provided in the form of an alm file
              'ALM': BEGIN 
                 READ_ALM, file, output, dtshp='array', lmax=lmax 
                 stop
              END
              ;; Amplitude provided in the form of a map file
              'MAP': BEGIN
                 STOP
              END
              ELSE: STOP
           ENDCASE
        END
        
        ;; Output format is a map file
        'map': BEGIN
           erasemapfilename = 0B
           CASE file_dtfm OF
              ;; Amplitude provided in the form of an alm file
              'ALM': BEGIN
                 mapfilename = PSM_TMPFILENAME(extension='.fits')
                 PSM_ALM2MAP, file, mapfilename, double=double, output_beam=GET_BEAM_STRUCT('GAUSSIAN',beamsize=SKY_RESOLUTION()), hpx_nside=hpx_nside, polar=polarised
                 erasemapfilename = 1B
              END
              ;; Amplitude provided in the form of a map file
              'MAP': BEGIN
                 mapfilename = file
              END
              ELSE: STOP
           ENDCASE
           READ_MAP, mapfilename, output, pointer=pointer
           IF erasemapfilename THEN FILE_DELETE, mapfilename
        END
        
     ENDCASE
     
     ;; Extrapolate into useful band if needed, and convert units
     ;;-----------------------------------------------------------
     IF DEFINED(*comp.e1.nuref.value) THEN BEGIN
        fac = CONVERSION_FACTOR(comp.e1.ampl.unit, 'MJy/sr',*comp.e1.nuref.value, double=double)
        params = EMLAW_PARAMLIST(comp.e1.law)
        FOR i=0, N_ELEMENTS(params)-1 DO ok = EXECUTE(params[i]+' = *comp.e1.'+params[i]+'.value')
        fac = fac * BANDINTEGRATED_EMLAW(band, emission_name=comp.e1.law, double=double, nuref=nuref, temp=temp, specind=specind, $
                                         curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, numin=*comp.e1.numin.value, numax=*comp.e1.numax.value)
        fac = fac * CONVERSION_FACTOR('MJy/sr', units, band, double=double)
     ENDIF ELSE BEGIN
        fac = CONVERSION_FACTOR(comp.e1.ampl.unit, units, band, double=double)
     ENDELSE
     
     IF pointer THEN (*output) = TEMPORARY(*output)*fac  ELSE output = TEMPORARY(output)*fac
     FIX_PRECISION, output, double=double


  ;; Case where we observe an emission law specified by e_index
  ;;------------------------------------------------------------
  ENDIF ELSE BEGIN

     ok = EXECUTE('file = *(comp.e'+NUMBER2STRING(e_index)+'.ampl.value)')
     file_dtfm = PSM_DATAFORMAT(file)

     CASE dtfm OF       
        ;; Output format is a map file
        'map': BEGIN
           erasemapfilename = 0B
           CASE file_dtfm OF
              ;; Amplitude provided in the form of a map file
              'MAP': BEGIN
                 mapfilename = file
              END
              ELSE: STOP
           ENDCASE
           READ_MAP, mapfilename, output, pointer=pointer
           IF erasemapfilename THEN FILE_DELETE, mapfilename
        END       
     ENDCASE

     ;; Extrapolate into useful band if needed, and convert units
     ;;-----------------------------------------------------------
     ok = EXECUTE('nurefdefined = DEFINED(*comp.e'+NUMBER2STRING(e_index)+'.nuref.value)')
     IF NOT ok THEN MESSAGE, 'Cannot find out whether nuref is defined'
     ok = EXECUTE('amplunit = comp.e'+NUMBER2STRING(e_index)+'.ampl.unit')
     IF NOT ok THEN MESSAGE, 'Cannot find out unit of input map'

     IF nurefdefined THEN BEGIN
        ok = EXECUTE('nuref = *comp.e'+NUMBER2STRING(e_index)+'.nuref.value')
        IF NOT ok THEN MESSAGE, 'Cannot find out value of reference frequency'
        ok = EXECUTE('numindefined = DEFINED(*comp.e'+NUMBER2STRING(e_index)+'.numin.value)')
        IF NOT ok THEN numin=0. ELSE BEGIN
           IF numindefined THEN BEGIN
              ok = EXECUTE('numin = *comp.e'+NUMBER2STRING(e_index)+'.numin.value')
              IF NOT ok THEN numin=0.
           ENDIF ELSE numin=0.
        ENDELSE
        ok = EXECUTE('numaxdefined = DEFINED(*comp.e'+NUMBER2STRING(e_index)+'.numax.value)')
        IF NOT ok THEN numax=FLOAT('Inf') ELSE BEGIN
           IF numaxdefined THEN BEGIN
              ok = EXECUTE('numax = *comp.e'+NUMBER2STRING(e_index)+'.numax.value')
              IF NOT ok THEN numax=FLOAT('Inf')
           ENDIF ELSE numax=FLOAT('Inf')
        ENDELSE
        fac = CONVERSION_FACTOR(amplunit, 'MJy/sr',nuref, double=double)
        ok = EXECUTE('law = comp.e'+NUMBER2STRING(e_index)+'.law')
        IF NOT ok THEN MESSAGE, 'Cannot find out emission law'
        params = EMLAW_PARAMLIST(law)
        FOR i=0, N_ELEMENTS(params)-1 DO ok = EXECUTE(params[i]+' = *comp.e'+NUMBER2STRING(e_index)+'.'+params[i]+'.value')
        fac = fac * BANDINTEGRATED_EMLAW(band, emission_name=law, double=double, nuref=nuref, temp=temp, specind=specind, $
                                         curvnuref=curvnuref, curvampl=curvampl, dlcomp=dlcomp, numin=numin, numax=numax)
        fac = fac * CONVERSION_FACTOR('MJy/sr', units, band, double=double)
     ENDIF ELSE BEGIN
        fac = CONVERSION_FACTOR(amplunit, units, band, double=double)
     ENDELSE
     
     IF pointer THEN (*output) = TEMPORARY(*output)*fac  ELSE output = TEMPORARY(output)*fac
     FIX_PRECISION, output, double=double
     

  ENDELSE

END  
  
;;---------------------------------------------------------------------------------------------------------------------------
;; MAIN PROGRAM
;;---------------------------------------------------------------------------------------------------------------------------

PRO COMP_IN_BAND, compname, band, output, units=units, alm=alm, map=map, lmax=lmax, hpx_nside=hpx_nside, $
                  dtfm=dtfm, dtshp=dtshp, pointer=pointer, double=double, polarised=polarised

  IF NOT KEYWORD_SET(lmax) THEN lmax = SKY_LMAX()
  IF NOT KEYWORD_SET(hpx_nside) THEN hpx_nside = SKY_NSIDE()
  hpx_npix = LONG(hpx_nside)^2*12L
  IF NOT KEYWORD_SET(outputshape) THEN outputshape = 'array'
  IF NOT DEFINED(double) THEN double = PRECISION_ISDOUBLE()

  ;; dttp (datatype) not set
  IF NOT KEYWORD_SET(dtfm) THEN BEGIN

     IF KEYWORD_SET(alm) THEN BEGIN
        dttp='alm' 
        IF KEYWORD_SET(map) THEN MESSAGE, 'Incompatible input keywords'
     ENDIF ELSE BEGIN
        IF KEYWORD_SET(map) THEN BEGIN
           dttp='map' 
        ENDIF ELSE BEGIN
           dttp='map' 
           IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, 'COMP_IN_BAND: Assuming an output map by default'
        ENDELSE
     ENDELSE

  ;; dttp (datatype) defined
  ENDIF ELSE BEGIN
     IF KEYWORD_SET(alm) THEN IF dtfm NE 'alm' THEN MESSAGE, 'Incompatible input keywords'
     IF KEYWORD_SET(map) THEN IF dtfm NE 'map' THEN MESSAGE, 'Incompatible input keywords'
  ENDELSE

  ;; Restore component
  RESTORE_COMP, comp, compname=compname
 
 
  ;; For one-dimensional components 
  ;; (dipole, cmb, simple freefree, simple spindust, non-relativistic thermalsz, kineticsz ...)
  ;;--------------------------------------------------------------------------------------------
  IF IS_ONE_DIMENSIONAL(comp) THEN BEGIN
     ONE_DIM_COMP_IN_BAND, comp, band, output, units=units, lmax=lmax, hpx_nside=hpx_nside, dtfm=dtfm, pointer=pointer, double=double, polarised=polarised

  ;; For complex multidimensional components      
  ;;-----------------------------------------
  ENDIF ELSE BEGIN

     CASE compname OF
        
        ;; Observe SZ (relativistic case)
        ;;--------------------------------

        'thermalsz': BEGIN
           ;; First term (non-relativistic, Kompaneets approximation)
           ONE_DIM_COMP_IN_BAND, comp, band, output, e_index=1, units=units, lmax=lmax, hpx_nside=hpx_nside, dtfm=dtfm, pointer=pointer, double=double, polarised=polarised
           ;; Other terms, if any
           nlaw = comp.nlaw
           IF nlaw GE 2 THEN BEGIN
              FOR ii=2, nlaw DO BEGIN
                 ONE_DIM_COMP_IN_BAND, comp, band, outputi, e_index=ii, units=units, lmax=lmax, hpx_nside=hpx_nside, dtfm=dtfm, pointer=pointer, double=double, polarised=polarised
                 *output = *output + TEMPORARY(*outputi)
                 PTR_FREE, outputi
              ENDFOR
           END
        END

        ;; Observe Synchrotron
        ;;---------------------
        
        'synchrotron': BEGIN
           
           EMLAWSTRUCT_IN_BAND, comp.e1, band, output, polarised=polarised, double=double
           fac = CONVERSION_FACTOR(comp.e1.ampl.unit,units,band,double=double)
           output = TEMPORARY(output)*fac
           IF pointer THEN output = PTR_NEW(TEMPORARY(output)) 
           
        END
        
        ;; Observe thermal dust
        ;;----------------------
        
        'thermaldust': BEGIN
           
           EMLAWSTRUCT_IN_BAND, comp.e1, band, output, polarised=polarised, double=double
           fac = CONVERSION_FACTOR(comp.e1.ampl.unit,units,band,double=double)
           output = TEMPORARY(output)*fac
           EMLAWSTRUCT_IN_BAND, comp.e2, band, output2, polarised=polarised, double=double
           fac = CONVERSION_FACTOR(comp.e2.ampl.unit,units,band,double=double)
           IF ISPOLARISED(output,/map) EQ ISPOLARISED(output2,/map) THEN output = TEMPORARY(output) + TEMPORARY(output2)*fac $
           ELSE MESSAGE, 'Case not implemented'
           IF pointer THEN output = PTR_NEW(TEMPORARY(output)) 
           
        END

        ;; Observe FIRB
        ;;--------------

        'firb': BEGIN

           EMLAWSTRUCT_IN_BAND, comp.e1, band, output, polarised=polarised, double=double
           fac = CONVERSION_FACTOR(comp.e1.ampl.unit,units,band,double=double)
           output = TEMPORARY(output)*fac

           IF comp.nlaw GE 2 THEN BEGIN
              FOR i=2, comp.nlaw DO BEGIN
                 ok = EXECUTE('em = comp.e' + NUMBER2STRING(i))
                 EMLAWSTRUCT_IN_BAND, em, band, output2, polarised=polarised, double=double
                 fac = CONVERSION_FACTOR(em.ampl.unit,units,band,double=double)
                 output = TEMPORARY(output) + TEMPORARY(output2)*fac
              ENDFOR
           ENDIF

           IF pointer THEN output = PTR_NEW(TEMPORARY(output)) 

        END
                  
        ;; Observe CO
        ;;------------

        'co': BEGIN

           EMLAWSTRUCT_IN_BAND, comp.e1, band, output, polarised=polarised, double=double
           fac = CONVERSION_FACTOR(comp.e1.ampl.unit,units,band,double=double)
           output = TEMPORARY(output)*fac
           FOR i=2, comp.nlaw DO BEGIN
              ok = EXECUTE('em = comp.e' + NUMBER2STRING(i))
              EMLAWSTRUCT_IN_BAND, em, band, output2, polarised=polarised, double=double
              fac = CONVERSION_FACTOR(em.ampl.unit,units,band,double=double)
              output = TEMPORARY(output) + TEMPORARY(output2)*fac
           ENDFOR
           IF pointer THEN output = PTR_NEW(TEMPORARY(output)) 

        END
                  
     ENDCASE
  ENDELSE

  DESTROY_DATA, comp

  FIX_PRECISION, output, double=double
   
END
  
