PRO OBSERVE_SKY, groups, instrument

  nchannel = instrument.nchannel
  FOR i=0, nchannel-1 DO BEGIN
     channelnumber = i+1
     inputdir = GETPSMDIR('skyinbands') + CHANNELPARAM(instrument,'INST', channelnumber) + '/' + CHANNELPARAM(instrument,'NAME', channelnumber)
     outputdir = GETPSMDIR('observations') + CHANNELPARAM(instrument,'INST', channelnumber) + '/' + CHANNELPARAM(instrument,'NAME', channelnumber)
     exists = FILE_TEST(outputdir,/directory)
     IF NOT exists THEN FILE_MKDIR, outputdir ELSE BEGIN
        IF (PSM_CAREFULNESS() GE 2) THEN BEGIN
           PSM_INFO_MESSAGE, 'Do you really want to erase everything in directory :'
           PSM_INFO_MESSAGE, outputdir
           PSM_INFO_MESSAGE, 'Execution of the PSM run stopped. Type .c to erase everything and continue'
           STOP
        ENDIF
        FILE_DELETE, outputdir, /allow_nonexistent, /quiet, /recursive
        FILE_MKDIR, outputdir
     ENDELSE
     wh = WHERE(STRLEN(groups) NE 0, ngroups)
     ok = EXECUTE('channel = instrument.channel_'+NUMBER2STRING(channelnumber))
     IF ngroups NE 0 THEN BEGIN
        GET_LUN, unit
        OPENW, unit, FIX_SEPARATOR(outputdir)+'README'
        FOR g=0, ngroups-1 DO PRINTF, unit, 'COMPONENT GROUP '+NUMBER2STRING(g+1)+': '+groups[g]
        CLOSE, unit
        FREE_LUN, unit
	knoise=0
        FOR g=0, ngroups-1 DO BEGIN

           allcomp = STRSPLIT(groups[g],/extract)

           whall = WHERE((allcomp EQ 'allsky') OR (allcomp EQ 'all'), nwhall)
           whnoise = WHERE((allcomp EQ 'noise') OR (allcomp EQ 'all'), nwhnoise)

           IF nwhall EQ 0 THEN BEGIN
              inputmaplist = FIX_SEPARATOR(inputdir) $
                             + allcomp + '_map_'+ CHANNELPARAM(instrument,'NAME', channelnumber) + '.fits'
           ENDIF ELSE BEGIN
              inputmaplist = FILE_SEARCH(FIX_SEPARATOR(inputdir),'*' $
                                         + '_map_'+ CHANNELPARAM(instrument,'NAME', channelnumber) + '.fits', count=ccc)
           ENDELSE

           mapsexist = FILE_TEST(inputmaplist)
           whok = WHERE(mapsexist,nmaps)

           ;; Do the noise map (if requested)
           ;;---------------------------------
           instrname = channel.inst
           COMMON obs, obser
           donoise = 'none'
           ok = EXECUTE('donoise = obser.'+instrname+'.noise')
           
           IF (nwhnoise GE 1) AND (donoise EQ 'nominal') AND (knoise EQ 0) THEN BEGIN

              COMMON psm, param
              polar = (param.global.fields EQ 'TP' AND SKY_POLARISED() AND channel.stokes EQ 'TQU')
              noisemap = INSTRUMENTAL_NOISE(channel, polar=polar)
	      knoise=1

           ENDIF

           ;; Do the sky map
           ;;----------------
           IF nmaps NE 0 THEN BEGIN
              maplist = inputmaplist[whok]
              COADD_SKYINBAND_MAPS, maplist, outputmap, hdr=hdr
             
              ;; Convert units if needed
              hdrunits = STRTRIM(SXPAR(hdr,'TUNIT*'),2)
              wh = WHERE(hdrunits NE CHANNELPARAM(instrument,'obs_units',channelnumber), nwh)
              IF nwh NE 0 THEN BEGIN
                 FOR z=0, nwh-1 DO BEGIN
                    STOP
                 ENDFOR
                 STOP ;; do the conversion here and update the header
              ENDIF

              cmix = GET_CMIX(maplist[0])
              IF N_ELEMENTS(maplist) GT 1 THEN FOR k=1, N_ELEMENTS(maplist)-1 DO cmix=[cmix,GET_CMIX(maplist[k])]
              ;; filename = instrument.name+'_'
              ;; FOR k=0, N_ELEMENTS(cmix)-1 DO
              ;; filename=filename+cmix[k]+'_'
              filename = ''
              filename = filename+'group'+NUMBER2STRING(g+1)+'_map_'
              filename = filename+CHANNELPARAM(instrument,'NAME', channelnumber)+'.fits'

              ;; Get pixelisation
              channelpix = CHANNELPARAM(instrument,'pix',i+1)
              skypix = SKY_PIX()
              
              CASE channelpix.pixtype OF
                 'HEALPIX': BEGIN
                    
                    skybeam = GET_BEAM_STRUCT(SKY_RESOLUTION())
                    channelbeam = CHANNELPARAM(instrument,'beam',i+1)

                    IF STRUCT_EQUAL(channelpix,skypix) THEN BEGIN
                       IF BEAMS_EQUAL(skybeam,channelbeam) THEN BEGIN
                          ;; Add noise
                          ;;;;IF ((nwhnoise GE 1) AND (donoise EQ 'nominal')) THEN outputmap = TEMPORARY(outputmap) + TEMPORARY(noisemap)  
                          IF ((nwhnoise GE 1) AND (donoise EQ 'nominal')) THEN BEGIN
                            ADD_CMIX, hdr, 'noise'
                            IF NOT DEFINED(outputmap) THEN outputmap = noisemap $
                                                      ELSE outputmap = TEMPORARY(outputmap) + noisemap 
                          ENDIF
                          ;; Write observed map
                          IF ISPOLARISED(outputmap,/map) THEN WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tqustokes=outputmap, xhdr=hdr $
                          ELSE WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tstokes=outputmap, xhdr=hdr
                       ENDIF ELSE BEGIN
                          ;; Change resolution
                          CHANGE_MAP_RESOLUTION, TEMPORARY(outputmap), map, beam_in=skybeam, beam_out=channelbeam, double=PRECISION_ISDOUBLE(), $
                                                 ordering_in=skypix.ordering, ordering_out=channelpix.ordering, /overwrite
                          ;; Change beam in header
                          block = PSM_BEAM_HDR(channelbeam)
                          UPDATE_PSMHDRBLOCK, hdr, block
                          ;; Add noise
                          IF ((nwhnoise GE 1) AND (donoise EQ 'nominal')) THEN BEGIN
                             ADD_CMIX, hdr, 'noise'
                             map = TEMPORARY(map) + noisemap  
                          ENDIF
                          ;; Write observed map
                          IF ISPOLARISED(map,/map) THEN WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tqustokes=TEMPORARY(map), xhdr=hdr $
                          ELSE WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tstokes=TEMPORARY(map), xhdr=hdr
                       ENDELSE
                    ENDIF ELSE BEGIN
                       ;; Change resolution
                       CHANGE_MAP_RESOLUTION, TEMPORARY(outputmap), map, beam_in=skybeam, beam_out=channelbeam, double=PRECISION_ISDOUBLE(), $
                                              ordering_in=skypix.ordering, ordering_out=channelpix.ordering, nside_out=channelpix.nside, /overwrite
                       ;; Update Healpix header
                       SXADDPAR, hdr, 'ORDERING', channelpix.ordering
                       SXADDPAR, hdr, 'NSIDE', channelpix.nside
                       SXADDPAR, hdr, 'NPIX', NSIDE2NPIX(channelpix.nside)
                       SXADDPAR, hdr, 'LASTPIX', NSIDE2NPIX(channelpix.nside)-1
                       ;; Change beam in header
                       block = PSM_BEAM_HDR(channelbeam)
                       UPDATE_PSMHDRBLOCK, hdr, block
                       ;; Add noise
                       ;;;;IF  ((nwhnoise GE 1) AND (donoise EQ 'nominal')) THEN outputmap = TEMPORARY(outputmap) + TEMPORARY(noisemap)  
                       IF  ((nwhnoise GE 1) AND (donoise EQ 'nominal')) THEN BEGIN
                          ADD_CMIX, hdr, 'noise'
                          IF NOT DEFINED(outputmap) THEN outputmap = noisemap $
                                                    ELSE outputmap = TEMPORARY(outputmap) + noisemap 
                       ENDIF 
                       ;; Write observed map
                       IF ISPOLARISED(map,/map) THEN WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tqustokes=TEMPORARY(map), xhdr=hdr, ordering=channelpix.ordering $
                       ELSE WRITE_HPXMAP, FIX_SEPARATOR(outputdir)+filename, tstokes=TEMPORARY(map), xhdr=hdr, ordering=channelpix.ordering
                    ENDELSE
                    
                 END
                 ELSE: STOP
              ENDCASE

              ;; Free memory
              DESTROY_DATA, outputmap

           ENDIF ELSE BEGIN

              IF (nwhnoise GE 1) AND (donoise EQ 'nominal') THEN BEGIN
              
                 filename = ''
                 filename = filename+'group'+NUMBER2STRING(g+1)+'_map_'
                 filename = filename+CHANNELPARAM(instrument,'NAME', channelnumber)+'.fits'
                 outname = FIX_SEPARATOR(outputdir)+filename
                 
                 ;; Write the map
                 PSM_ID_KEYGEN, key
                 IF ISPOLARISED(noisemap,/map) THEN WRITE_HPXMAP, outname, tqustokes=noisemap, datatyp='OBS', psm_dtid=key, beam=channel.beam, $
                    coordsys=channel.pix.coordsys, ordering=channel.pix.ordering, units=channel.obs_units, lmax=SKY_LMAX() $
                 ELSE WRITE_HPXMAP, outname, tstokes=noisemap, datatyp='OBS', psm_dtid=key, beam=channel.beam, $
                                    coordsys=channel.pix.coordsys, ordering=channel.pix.ordering, units=channel.obs_units, lmax=SKY_LMAX()
                 
                 ;; Get the header
                 hh = HEADFITS(outname,exten=1)
                 
                                ;: update PSM header
                 SPLIT_FITSHDR, hh, hout=hout, hpsm=hpsm, hend=hend
                 EXTRACT_PSMHDRBLOCK, hpsm, hbase, blockname='base', whereblock=whbase
                 UPDATEHDRPAR, hbase, 'PSM_VERS', PSM_VERSION()
                 UPDATEHDRPAR, hbase, 'PSM_RNID', PSM_ID()
                 PSM_ID_KEYGEN, key & UPDATEHDRPAR, hbase, 'PSM_FLID', key
                 hobs = PSM_OBS_HDR(['noise'])
                 hband = PSM_BAND_HDR(channel.band)
                 EXTRACT_PSMHDRBLOCK, hpsm, hbeam, blockname='beam', whereblock=whbeam
                 EXTRACT_PSMHDRBLOCK, hpsm, hmap, blockname='map', whereblock=whmap
                 
                 MK_PSMHDR, hdrpsm, hbase, hobs, hmap, hband, hbeam
                 hdr = MERGE_FITSHDRS(hh,hdrpsm)
                 MODFITS, outname, 0, hdr, EXTEN_NO = 1
                 
              ENDIF
           ENDELSE
        ENDFOR
	;; Free memory
        DESTROY_DATA, noisemap
     ENDIF
  ENDFOR
  
END 

