PRO EXTRACT_PSMHDRBLOCK, hdr, block, blockname=blockname, whereblock=whereblock, wherenotblock=wherenotblock, silent=silent

  ;; First check that the fits header is OK
  ok = CHECK_FITSHDR(hdr)
  IF NOT ok THEN MESSAGE, 'Input header is not valid'

  ;; Make a template header for the block of interest
  CASE blockname OF
     'base': templatehdr = PSM_BASE_HDR()
     'comp': templatehdr = PSM_COMP_HDR()
     'obs': BEGIN
        res=SXPAR(hdr, 'CMIX*', count=ncomp, /silent)
        templatehdr = PSM_OBS_HDR(ncomp=ncomp)
     END
     'alm': templatehdr = PSM_ALM_HDR()
     'map': templatehdr = PSM_MAP_HDR()
     'cl': templatehdr = PSM_CL_HDR()
     'cat': templatehdr = PSM_CAT_HDR()
     'beam': BEGIN
        beamtype = STRTRIM(SXPAR(hdr,'BEAMTYPE',count=count),2)
        IF count NE 0 THEN templatehdr = PSM_BEAM_HDR(GET_BEAM_STRUCT(beamtype)) ELSE templatehdr = PSMHDRBLOCK_HEADLINE('beam')
     END
     'band': BEGIN
        bandshape = STRTRIM(SXPAR(hdr,'BD_SHP', count=count),2)
        IF count NE 0 THEN templatehdr = PSM_BAND_HDR(GET_BAND_STRUCT(bandshape)) ELSE templatehdr = PSMHDRBLOCK_HEADLINE('band')
     END
  ENDCASE
  
  ;; Define that the block is complete by default (contains all header tags and the block header comment line)
  complete = 1

  ;; By default the block is nowhere to be found
  whblock = [-1]

  ;; Look for the first line of the block header
  IF SIZE(templatehdr[0],/type) NE 7 THEN PSM_DEBUG_MESSAGE, 'Problem with template header', routine='EXTRACT_PSMHDRBLOCK'
  wh = WHERE(hdr EQ templatehdr[0],nwhead)
  
  ;; If the block headline has been found, add its index to whblock (otherwise the block is not complete)
  IF nwhead NE 0 THEN BEGIN
     whblock = [whblock,wh]  
     ;; For all tags of the block header, get the corresponding line and add the index to whblock
     nel = N_ELEMENTS(templatehdr)
     IF nel GE 2 THEN FOR i=1,nel-1 DO BEGIN
        wh = WHERE(STRMID(hdr,0,8) EQ STRMID(templatehdr[i],0,8),nwh)         
        IF nwh NE 0 THEN whblock = [whblock,wh] ELSE complete=0
     ENDFOR
  ENDIF

  IF N_ELEMENTS(whblock) GT 1 THEN BEGIN
     whereblock = whblock[1:*]                       ; remove first value of -1
     block = hdr(whereblock)                         ; extract block value
     
     ;; Find the complementary lines (to define wherenotblock)
     keep = REPLICATE(1,N_ELEMENTS(hdr))
     keep(whereblock) = 0
     wherenotblock = WHERE(keep EQ 1)
     IF NOT complete THEN BEGIN
        IF ((PSM_CAREFULNESS() GE 2) AND (PSM_VERBOSITY() GE 2)) THEN PSM_INFO_MESSAGE, 'EXTRACT_PSMHDRBLOCK: WARNING, '+ blockname +' header block not complete in header'
        PSM_DEBUG_MESSAGE, 'Input header incomplete', routine = 'EXTRACT_PSMHDRBLOCK'
     ENDIF
     
  ENDIF ELSE BEGIN
     whereblock = -1L
     wherenotblock = INDGEN(N_ELEMENTS(hdr))
     IF NOT KEYWORD_SET(silent) THEN BEGIN
        IF ((PSM_VERBOSITY() GE 2) AND (PSM_CAREFULNESS() GE 1)) THEN PSM_INFO_MESSAGE, 'EXTRACT_PSMHDRBLOCK: '+ blockname +' header block not found in header'
     ENDIF
  ENDELSE

END 
