FUNCTION PSM_BASE_HDR, datatype=datatype, dataform=dataform, setval=setval, help=help

  ;; datatype can be 'comp' or 'obs'
  ;; dataform can be 'map' or 'alm' or 'cl' or 'cat'

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_BASE_HDR: function to generate the basic PSM header information'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_BASE_HDR(datatype=, dataform=, /setval)'
     PSM_INFO_MESSAGE, "datatype can be 'comp' for COMPONENT or 'obs' for OBSERVATION"
     PSM_INFO_MESSAGE, "dataform can be 'map' or 'alm' or 'cl' or 'cat'"
     PSM_INFO_MESSAGE, "set /setval to assign default values to PSM_VERS and PSM_RNID"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ;; Minimal header
  ;;----------------

  hd = [PSMHDRBLOCK_HEADLINE('base')]
  hd = PSM_STRING_TO_LENGTH(hd,80)
  psmvers = PSM_VERSION()
  SXADDPAR, hd, 'PSM_VERS', psmvers, "PSM version used to create the data"
  psmid = PSM_ID()
  SXADDPAR, hd, 'PSM_RNID', psmid, "ID key of PSM run which produced the data"
  IF DEFINED(datatype) THEN SXADDPAR, hd, 'PSM_DTTP', STRUPCASE(datatype), "PSM data type ('COMP' or 'OBS')" $
  ELSE SXADDPAR, hd, 'PSM_DTTP', '', "PSM data type ('COMP' or 'OBS')"
  IF DEFINED(dataform) THEN SXADDPAR, hd, 'PSM_DTFM', STRUPCASE(dataform), "PSM data format ('MAP', 'ALM', 'CL' or 'CAT')" $
  ELSE SXADDPAR, hd, 'PSM_DTFM', '', "PSM data format ('MAP', 'ALM', 'CL' or 'CAT')"
  SXADDPAR, hd, 'PSM_DTID', '', "ID key of PSM data"
  PSM_ID_KEYGEN, key
  SXADDPAR, hd, 'PSM_FLID', key, "ID key of this file"
  
  IF KEYWORD_SET(setval) NE 0 THEN BEGIN
     SXADDPAR, hd, 'PSM_VERS', PSM_VERSION()
     SXADDPAR, hd, 'PSM_RNID', PSM_ID()
  ENDIF

  hd = [hd[0:N_ELEMENTS(hd)-2]]

RETURN, hd

END
