FUNCTION PSM_BEAM_HDR, beam, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_BEAM_HDR: function to generate PSM beam header information array'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_BEAM_HDR(beam)'
     PSM_INFO_MESSAGE, "beam can be any valid beam recognized by psm_get_bl.pro (default is 0)"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  hd = PSMHDRBLOCK_HEADLINE('beam') & hd=[hd]

  IF DEFINED(beam) THEN BEGIN
     beamvartype = SIZE(beam,/type)
     IF beamvartype LE 5 THEN BEGIN
        bmtp = 'GAUSSIAN'
        bmsz = FLOAT(beam)      
     ENDIF
     IF beamvartype EQ 8 THEN BEGIN
        bmtp = beam.type
        IF bmtp EQ 'GAUSSIAN' THEN BEGIN
           bmsz = beam.size
           bmsz = bmsz*CONVERSION_FACTOR(beam.sizeunits,'arcmin')
        ENDIF
     ENDIF
     SXADDPAR, hd, 'BEAMTYPE', bmtp
     CASE bmtp OF
        'GAUSSIAN': BEGIN
           SXADDPAR, hd, 'BEAMSIZE', bmsz, "Beam size in arcminutes" 
        END
        'TABULATED': BEGIN
           PSM_DEBUG_MESSAGE, 'Implement tabulated beam headers', routine='PSM_BEAM_HDR'
           stop
        END
        'INSTR': BEGIN
           SXADDPAR, hd, 'BM_INSTR', beam.instr, "Instrument reference for the beam"
           SXADDPAR, hd, 'BM_VERS', beam.version, "Version reference for the beam or instrument"
           SXADDPAR, hd, 'BM_CHAN', beam.channel, "Channel reference for the beam"
        END
        'NO_BEAM': BEGIN
           bmtp = 'NO_BEAM'
           SXADDPAR, hd, 'BEAMTYPE', bmtp
        END
        'UNKNOWN': BEGIN
           bmtp = 'UNKNOWN'
           SXADDPAR, hd, 'BEAMTYPE', bmtp
        END
     ENDCASE
  ENDIF ELSE BEGIN
     bmtp = 'UNKNOWN'
     SXADDPAR, hd, 'BEAMTYPE', bmtp
  ENDELSE
     
  hd = hd[0:N_ELEMENTS(hd)-2]

RETURN, hd
END
