FUNCTION PSM_OBS_HDR, complist, ncomp=ncomp, obsid=obsid, help=help
  
  ;; obsid can be shared by several files corresponding to the same obs. e.g. a CMB and a galaxy observed by the same band)

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_OBS_HDR: function to generate PSM observation header information array'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_OBS_HDR(complist, obsid=, band=)'
     PSM_INFO_MESSAGE, "complist is the list of components present in the mix"
     PSM_INFO_MESSAGE, "ncomp is used only if complist is not defined, to generate a template header"
     PSM_INFO_MESSAGE, "obsid is the 16 ASCII character ID key of the observation"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ;hd = ['COMMENT  --- observation header ---']
  ;hd = PSM_STRING_TO_LENGTH(hd,80,character='-')
  hd = [PSMHDRBLOCK_HEADLINE('obs')]

  IF DEFINED(obsid) THEN SXADDPAR, hd, 'PSM_OBID', obsid, "PSM observation ID key" ELSE $
     SXADDPAR, hd, 'PSM_OBID', '', "PSM observation ID key"
  hd = hd[0:N_ELEMENTS(hd)-2]

  compadded=0
  numcomp=0

  IF DEFINED(complist) THEN BEGIN
     allcomp = PSM_ALLCOMP()
     allcomp = [allcomp, 'noise']
     nc = N_ELEMENTS(allcomp)
     FOR i=0, nc-1 DO BEGIN
        wh = WHERE(complist EQ allcomp[i], nwh)
        IF nwh EQ 1 THEN BEGIN
           compadded=1
           numcomp=numcomp+1
           SXADDPAR, hd, 'CMIX'+NUMBER2STRING(numcomp), allcomp[i], "Component included in observation"
        ENDIF
     ENDFOR
  ENDIF ELSE BEGIN
     IF KEYWORD_SET(ncomp) NE 0 THEN BEGIN
        compadded=1
        FOR i=1, ncomp DO SXADDPAR, hd, 'CMIX'+NUMBER2STRING(i), '', "Component included in observation"
     ENDIF
  ENDELSE

  IF compadded THEN hd = hd[0:N_ELEMENTS(hd)-2]

RETURN, hd
END
