FUNCTION PSM_PIX_HDR, pix_param, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSM_PIX_HDR: function to generate the standard header for a given pixelisation'
     PSM_INFO_MESSAGE, 'SYNTAX: result = PSM_PIX_HDR(pix_param)'
     PSM_INFO_MESSAGE, 'pix_param is a standard structure describing a pixelisation in the PSM'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  PSM_DEBUG_MESSAGE, 'Fix implementation of pixelisation headers', routine='PSM_PIX_HDR'

;; Check information about map pixelisation
;;------------------------------------------
  pixtype = pix_param.pixtype

;; Pixelisation dependent header
;;-------------------------------
  CASE pixtype OF

     ;; HEALPIX
     ;;---------
     'HEALPIX': BEGIN

        ;; first a few comment lines
        hd = ['COMMENT  *** HEALPIX header ***']
        hd = PSM_STRING_TO_LENGTH(hd,80,character='*')

        ;; Now write standard HEALPIX keywords
        SXADDPAR, hd, 'PIXTYPE', 'HEALPIX', 'HEALPIX pixelisation'       
        SXADDPAR, hd, 'ORDERING', pix_param.ordering, 'Pixel ordering scheme, either RING or NESTED'
        SXADDPAR, hd, 'NSIDE', pix_param.nside, 'Healpix resolution parameter'
        SXADDPAR, hd, 'NPIX', pix_param.npix, 'Total number of pixels'
        SXADDPAR, hd, 'OBJECT', pix_param.object, 'Sky coverage, either FULLSKY or PARTIAL'
        SXADDPAR, hd, 'FIRSTPIX', pix_param.firstpix, 'First pixel # (zero based)'
        SXADDPAR, hd, 'LASTPIX', pix_param.lastpix, 'Last pixel # (zero based)'
        SXADDPAR, hd, 'INDXSCHM', pix_param.indxschm, 'Indexing : IMPLICIT or EXPLICIT'
        SXADDPAR, hd, 'GRAIN', 0, 'GRAIN = 0: No index'
        SXADDPAR, hd, 'COORDSYS', pix_param.coordsys, 'Pixelization coordinate system'
        nel = N_ELEMENTS(hd)
        hd = [hd[0:nel-2],PSM_STRING_TO_LENGTH('COMMENT         G = Galactic, E = ecliptic, C = celestial = equatorial',80),hd[nel-1]]

     END
     ELSE: BEGIN
        MESSAGE, pixtype + ' pixelisation not implemented'
     END
  ENDCASE

  RETURN, hd
END
