FUNCTION FITSFILE_BEAM, fitsfile, info_exists=info_exists, exten=exten

  PSM_DEBUG_MESSAGE, 'Read beam in all extensions', routine = 'FITSFILE_BEAM', infolimit=2, stoplimit=3
  IF NOT DEFINED(exten) THEN exten=1
  hd = HEADFITS(fitsfile, exten=exten)
  beamtype = STRTRIM(STRUPCASE(SXPAR(hd, 'BEAMTYPE', count=count)),2)

  info_exists = 1B

  IF count EQ 0 THEN BEGIN
     info_exists = 0B
     beamtype='UNKNOWN'
     beam = GET_BEAM_STRUCT('UNKNOWN')
  ENDIF

  CASE beamtype OF
     'NO_BEAM': BEGIN
        beam = {type:'NO_BEAM'}
     END
     'GAUSSIAN': BEGIN
        beam = GET_BEAM_STRUCT(0)
        beam.size = SXPAR(hd, 'BEAMSIZE')
     END
     'TABULATED': BEGIN
        MESSAGE, 'Case not implemented'
     END
     'INSTR': BEGIN
        instr = STRTRIM(SXPAR(hd, 'BM_INSTR'),2)
        version = STRTRIM(SXPAR(hd, 'BM_VERS'),2)
        channel = STRTRIM(SXPAR(hd, 'BM_CHAN'),2)
        beam = GET_BEAM_STRUCT('INSTR',instr=instr,version=version,channel=channel)
     END
     ELSE: BEGIN
        IF psm_carefulness() GE 2 THEN PSM_INFO_MESSAGE, 'Beam info not found in file '+fitsfile
        IF psm_carefulness() GE 3 THEN BEGIN
           PSM_INFO_MESSAGE, 'Beam info not found in file '+fitsfile
           PSM_INFO_MESSAGE, 'Stopping the execution. Type .c to assume no beam and continue'
           STOP
        ENDIF
        beam = GET_BEAM_STRUCT('UNKNOWN')
     ENDELSE
  ENDCASE

RETURN, beam
END
