FUNCTION FITSFILE_LMAX, fitsfile, exten=exten, ok=ok, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, ''
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'FITSFILE_LMAX: function to get the maximum multipole stored in a map or alm fits file'
     PSM_INFO_MESSAGE, 'SYNTAX: result = FITSFILE_LMAX(fitsfile, exten=, ok=)'
     PSM_INFO_MESSAGE, 'By default the extension checked is exten=1, set exten to something else if necessary.'
     PSM_INFO_MESSAGE, 'The status is returned in the OK output keyword.'
     PSM_INFO_MESSAGE, 'If lmax was found, then OK=1B, otherwise OK=0B.'
     PSM_INFO_MESSAGE, 'fitsfile can be an array, in which case result and ok are arrays of same size'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF NOT DEFINED(exten) THEN exten=1

  IF ISARRAY(fitsfile) THEN BEGIN
     lmax = REPLICATE(-1L, N_ELEMENTS(fitsfile))
     ok = BYTARR(N_ELEMENTS(fitsfile))
     FOR i=0, N_ELEMENTS(fitsfile)-1 DO BEGIN
        lmax[i] = FITSFILE_LMAX(fitsfile[i], exten=exten, ok=oki)
        ok[i] = oki
     ENDFOR
  ENDIF ELSE BEGIN
     oki = FILE_TEST(fitsfile)
     IF NOT oki THEN BEGIN
        RETURN, -1L
     ENDIF ELSE BEGIN
        hd = HEADFITS(fitsfile, exten=exten)
        lmax = LONG(HEADER_LMAX(hd, ok=ok))
     ENDELSE
  ENDELSE

  RETURN, lmax

END
