PRO READ_ALM, fitsfile, alm, index, dtshp=dtshp, signal=signal, lmax=lmax, pointer=pointer, hdr=hdr, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'READ_ALM: procedure to read alm coefficients in a fits file'
     PSM_INFO_MESSAGE, 'SYNTAX: READ_ALM, fitsfile, alm, index, dtshp=, signal=, lmax=, /pointer'
     PSM_INFO_MESSAGE, "dtshp (data shape) can be either 'structure' (created with alm_structure.pro), or 'array' [n_index,2,n_fields]"
     PSM_INFO_MESSAGE, "signal is the signal to be read: can be 'T' or 'E' or 'B' or 'TEB' or 'ALL'"
     PSM_INFO_MESSAGE, "set /pointer to return a pointer rather than a memory array"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  PSM_DEBUG_MESSAGE, 'To be implemented seriously', routine='READ_ALM', infolimit=2, stoplimit=3

  IF NOT DEFINED(dtshp) THEN dtshp='array'

  polar = ISPOLARISED(fitsfile, /alm)
  IF NOT KEYWORD_SET(signal) THEN BEGIN
     IF polar THEN signal_to_read='ALL' ELSE signal_to_read='T'
  ENDIF ELSE BEGIN
     CASE signal OF
        'T': signal_to_read='T'
        'E': IF polar THEN signal_to_read='E'
        'B': IF polar THEN signal_to_read='B'
        'TEB': IF polar THEN signal_to_read='ALL'
        'ALL': IF polar THEN signal_to_read='ALL' ELSE signal_to_read='T'
     ENDCASE
     
  ENDELSE

  CASE dtshp OF
     'array': BEGIN
        FITS2ALM, index, alm, fitsfile, signal_to_read, lmax=lmax
        IF DEFINED(lmax) THEN BEGIN
           ni = LMAX2NALM(lmax)
           IF ni NE N_ELEMENTS(index) THEN BEGIN
              MESSAGE, "Check file '"+fitsfile+"'", /info 
              MESSAGE, 'Input alm file looks corrupt (number of alm does not seem to match LMAX in header !!'
           ENDIF
        ENDIF
     END
     'structure':BEGIN
        MESSAGE, 'Format not implemented yet'
     END
  ENDCASE
  
  IF KEYWORD_SET(pointer) NE 0 THEN alm = PTR_NEW(alm,/no_copy)

END
