PRO WRITE_ALM, alm, fitsfile, xhdr1=xhdr1, xhdr2=xhdr2, xhdr3=xhdr3, beam=beam, units=units, coordsys=coordsys, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WRITE_ALM: procedure to write a fits file given spherical harmonics coefficents alm'
     PSM_INFO_MESSAGE, 'SYNTAX: WRITE_ALM, alm, fitsfile, xhdr1=, xhdr2=, xhdr3=, beam=, coordsys=coordsys, units='
     PSM_INFO_MESSAGE, 'alm can be either a structure created with alm_structure.pro, or an array [n_index,2,n_fields]'
     PSM_INFO_MESSAGE, "set 'beam' to the beam of the map, to be written in the map header"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  ;; Check whether the fitsfile already exists
  ;;-------------------------------------------
  exists = FILE_TEST(fitsfile)
  IF exists THEN BEGIN
     IF PSM_CAREFULNESS() GE 2 THEN BEGIN
        PSM_INFO_MESSAGE, 'Do you really want to overwrite existing file '+ fitsfile +' ?'
        PSM_INFO_MESSAGE, 'Execution of the PSM run stopped. Type .c to erase and continue'
        STOP
     ENDIF
        
  ENDIF

  almtype = SIZE(alm,/type)

  ;; If the alm are provided as a structure
  ;;----------------------------------------
  IF almtype EQ 8 THEN BEGIN
     ALM_STRUCT2ARRAY, alm, almarr, index
     ALM2FITS, index, TEMPORARY(almarr), fitsfile
  ENDIF


  ;; If the alm are provided as an array
  ;;----------------------------------------
  IF almtype LE 5 THEN BEGIN
     dim = SIZE(alm,/dimensions)
     lmax = NALM2LMAX(dim[0])
     index = ALM_INDEX(0,lmax)     
     ALM2FITS, index, alm, fitsfile
  ENDIF

  ;; Update the headers if needed
  ;;------------------------------
  FITS_INFO, fitsfile, n_ext=n_ext, /silent

  FOR i=1, n_ext DO BEGIN
     upd = 0
     ok = EXECUTE("IF KEYWORD_SET(xhdr"+ NUMBER2STRING(i) + ") NE 0 THEN upd = 1")
     IF DEFINED(beam) THEN upd=1
     IF DEFINED(units) THEN upd=1
     IF DEFINED(coordsys) THEN upd=1
     ;; IF KEYWORD_SET(xhdr) THEN BEGIN
     ;;    upd = 1
     ;;    newhdr = xhdr
     ;;    IF DEFINED(beam) THEN BEGIN
     ;;       beamhdr = PSM_BEAM_HDR(beam)
     ;;       newhdr = MERGE_FITSHDRS(newhdr,beamhdr)
     ;;    ENDIF
     ;;    ok = EXECUTE("IF KEYWORD_SET(xhdr"+ NUMBER2STRING(i) + $
     ;;                 ") NE 0 THEN MESSAGE, 'Too many input headers, use either xhdr (common to all extensions) or xhdr1, xhdr2, ...'")
     ;; ENDIF ELSE BEGIN
        
        IF upd THEN BEGIN 
           IF DEFINED(beam) THEN BEGIN
              beamhdr = PSM_BEAM_HDR(beam)
              ok = EXECUTE("IF KEYWORD_SET(xhdr"+ NUMBER2STRING(i) + ") NE 0 THEN newhdr = xhdr"+ NUMBER2STRING(i))
              IF DEFINED(newhdr) THEN newhdr = MERGE_FITSHDRS(newhdr,beamhdr) ELSE newhdr=beamhdr
           ENDIF ELSE BEGIN
              ok = EXECUTE("IF KEYWORD_SET(xhdr"+ NUMBER2STRING(i) + ") NE 0 THEN newhdr = xhdr"+ NUMBER2STRING(i))
           ENDELSE

           h = HEADFITS(fitsfile,exten=i)
           IF DEFINED(coordsys) THEN SXADDPAR, h, 'COORDSYS', coordsys

           IF DEFINED(newhdr) THEN h = MERGE_FITSHDRS(newhdr, h)
           
           IF DEFINED(units) THEN BEGIN
              SXADDPAR, h, 'TUNIT2', units, after='TFORM2'
              SXADDPAR, h, 'TUNIT3', units, after='TFORM3'
           ENDIF
           MODFITS, fitsfile, 0, h, exten_no=i
        ENDIF

     ;; ENDELSE

  ENDFOR


END
