PRO WRITE_CL, cl, fitsfile, pxwin=pxwin, beam=beam, xhdr=xhdr, help=help

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 2) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'WRITE_CL: procedure to write a fits file given power spectrum cl'
     PSM_INFO_MESSAGE, 'SYNTAX: WRITE_CL, cl, fitsfile, pxwin=pxwin'
     PSM_INFO_MESSAGE, 'cl is a structure created with cl_structure.pro'
     PSM_INFO_MESSAGE, "pxwin gives info about the pixel window function applied to the data"
     PSM_INFO_MESSAGE, "pxwin can be 0 (no pixel window function), or, 1, 2, 4, 8, ... (nside of the pixel window)"
     PSM_INFO_MESSAGE, "(default is pxwin=0)"
     PSM_INFO_MESSAGE, "Set xhdr to an array of additional header lines to be included, if any"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  COMMON psm, param
  IF DEFINED(param) THEN  carefulness = param.run.carefulness ELSE carefulness=0
  IF DEFINED(param) THEN  verbosity = param.run.verbosity ELSE verbosity=0

  exists = FILE_TEST(fitsfile)
  IF exists THEN BEGIN
     IF carefulness GE 2 THEN BEGIN
        PSM_INFO_MESSAGE, 'File already exists: Writing of the cl STOPPED'
        PSM_INFO_MESSAGE, 'Type .c to  delete existing file ('+fitsfile+') and continue'
        STOP
     ENDIF ELSE BEGIN
        IF verbosity GE 2 THEN FILE_DELETE, fitsfile, /verbose ELSE FILE_DELETE, fitsfile, /quiet
     ENDELSE
  ENDIF

  MWRFITS, cl, fitsfile

  ;; Fix the header
  ;;----------------
  hfits = HEADFITS(fitsfile, exten=1)

  IF KEYWORD_SET(xhdr) THEN BEGIN
     hfits = [hfits,xhdr]
     REORDER_FITSHDR, hfits
  ENDIF

  EXTRACT_PSMHDRBLOCK, hfits, block, blockname='base', whereblock=whereblock

  IF whereblock[0] EQ -1 THEN BEGIN
     hbase = PSM_BASE_HDR(dataform='cl', /setval)
     MK_PSMHDR, hpsm, hbase
     hfits = [hfits,hpsm]
     REORDER_FITSHDR, hfits
  ENDIF
  PSM_ID_KEYGEN, key
  SXADDPAR, hfits, 'PSM_DTID', key, "ID key of PSM data"

  EXTRACT_PSMHDRBLOCK, hfits, block, blockname='cl', whereblock=whereblock
  IF whereblock[0] EQ -1 THEN BEGIN
     hcl = PSM_CL_HDR(lmax=MAX(cl.l))
     MK_PSMHDR, hpsm, hcl
     hfits = [hfits,hpsm]
     REORDER_FITSHDR, hfits
  ENDIF

  IF DEFINED(beam) NE 0 THEN BEGIN
     hbeam = PSM_BEAM_HDR(beam)
     MK_PSMHDR, hpsm, hbeam
     hfits = [hfits,hpsm]
     REORDER_FITSHDR, hfits
  ENDIF

  MODFITS, fitsfile, 0, hfits, EXTEN_NO = 1

END
