; Copyright 2007 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; This program defines the system variable !const
; which sets values for useful physical constants: c (speed of light)
; h, h_bar, k (boltzmann), G (gravitation), m_e (electron mass),
; e (electron charge, m_p (proton mass), eps0 (permitivity of free space),
; n_a (Avogadro number), m_s (solar mass)
;
; @categories tool
;  
; @history <p>13/07/2007: First version, Jacques Delabrouille</p>
; @history <p>19/01/2009: Now output parameter structure (optional), Jacques Delabrouille</p>
; @copyright <p>Jacques Delabrouille, CNRS, 2007
;-

PRO DEFINE_PHYSICAL_CONSTANTS

  ;; Defines a system variable named !phyconst
  ;; Contained fixed physical parameters for
  ;;   centralized definition of some useful constants (in mks units)
  ;;   (structure encapsulation avoids namespace pollution
  ;; Values here are taken from the Particle Physics data booklet
  ;;
  phyconst = {physical_constants, $
              c:299792458d, $           ; Speed of light
              h:6.6260693d-34, $        ; Planck constant
              h_bar:1.05457168d-34, $   ;
              k:1.3806505d-23, $        ; Boltzmann constant
              G:6.6742d-11, $           ; Gravitational constant
              m_e:9.1093826d-31, $      ; Electron mass (kg)
              e:1.60217653d-19, $       ; Electron charge (Coulomb)
              m_p:1.67262171d-27, $     ; Proton mass (kg)
              eps0:8.854187817d-12, $   ; Permitivity of free space
              n_a:6.0221415d23, $       ; Avogadro constant
              ;;tcmb:2.725, $           ; CMB temperature (K)
              m_s:1.98844d30 $          ; Solar Mass (kg)
             }
  DEFSYSV, '!phyconst', phyconst

END
