PRO PROHELP, routine_name, path=path
  
  ;; Help about prohelp itself is further down

  r=RECALL_COMMANDS()

  type = SIZE(routine_name, /type)
  IF type NE 7 THEN BEGIN
     cmd_elts = STRSPLIT(r[0],',',/extract)
     routine = STRTRIM(cmd_elts[1],2)
  ENDIF ELSE routine = STRTRIM(routine_name,2)

  routine = PSM_STR_REPLACE(STRLOWCASE(routine),'.pro','')

  nm = STRUPCASE(routine)
  list_dir = STRSPLIT(!PATH,':',/extract)

  ndir = N_ELEMENTS(list_dir)
  FOR i=0, ndir-1 DO BEGIN
     dir = FIX_SEPARATOR(list_dir[i])
     res = FILE_SEARCH(dir+'*')
     files = PSM_STR_REPLACE(res,dir,'')
     wh = WHERE(STRLOWCASE(files) EQ STRLOWCASE(routine)+'.pro', nwh)
     IF nwh EQ 1 THEN BEGIN
        fullfilename = FIX_SEPARATOR(dir)+files[wh]
        GOTO, continue_help
     ENDIF
  ENDFOR

  PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
  PSM_INFO_MESSAGE, nm + ': Routine not found in IDL path'
  PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
  GOTO, closing

  continue_help:

  ;; Help about prohelp itself
  ;;---------------------------
  IF STRLOWCASE(routine) EQ 'prohelp' THEN BEGIN
     IF KEYWORD_SET(path) NE 0 THEN BEGIN
        HELP, /procedures, output=longlist
        nl = N_ELEMENTS(longlist)
        list = STRARR(nl)
        FOR i=0, nl-1 DO BEGIN
           pieces = STRSPLIT(longlist[i],' ',/extract)
           list[i] = STRTRIM(pieces[0],2)
        ENDFOR
        wh = WHERE(list EQ nm, nwh)
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
        PRINT, STRUPCASE(routine)+' source code = '+fullfilename 
     ENDIF
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PROHELP: procedure to print out help info about a specified routine'
     PSM_INFO_MESSAGE, 'SYNTAX: PROHELP, routine_name, /path'
     PSM_INFO_MESSAGE, 'Set the /path keyword to get info about the location of the source code'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF
  ;;-----------------------------

  RESOLVE_ROUTINE, routine, /either, /compile_full_file 

  ;; Get help if routine is a procedure
  HELP, /procedures, output=longlist
  nl = N_ELEMENTS(longlist)
  list = STRARR(nl)
  FOR i=0, nl-1 DO BEGIN
     pieces = STRSPLIT(longlist[i],' ',/extract)
     list[i] = STRTRIM(pieces[0],2)
  ENDFOR
  wh = WHERE(list EQ 'PSM_INFO_MESSAGE', nwh)
  IF nwh EQ 0 THEN RESOLVE_ROUTINE, 'psm_info_message', /either, /compile_full_file
  wh = WHERE(list EQ nm, nwh)
  IF nwh EQ 1 THEN BEGIN
     IF KEYWORD_SET(path) NE 0 THEN BEGIN
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
        PRINT, STRUPCASE(routine)+' source code = '+fullfilename 
     ENDIF
     ok = EXECUTE(routine + ', /help')
     IF NOT OK THEN BEGIN
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
        PSM_INFO_MESSAGE, 'No help available for procedure ' + nm
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     ENDIF
     GOTO, closing
  ENDIF

  ;; Get help if routine is a function
  HELP, /functions, output=longlist
  nl = N_ELEMENTS(longlist)
  list = STRARR(nl)
  FOR i=0, nl-1 DO BEGIN
     pieces = STRSPLIT(longlist[i],' ',/extract)
     list[i] = STRTRIM(pieces[0],2)
  ENDFOR
  wh = WHERE(list EQ nm, nwh)
;  wh = WHERE(STRMID(list,0,STRLEN(nm)) EQ nm, nwh)
  IF nwh EQ 1 THEN BEGIN
     IF KEYWORD_SET(path) NE 0 THEN BEGIN
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
        PRINT, STRUPCASE(routine)+' source code = '+fullfilename 
     ENDIF
     ok = EXECUTE('out = ' + routine + '(/help)')
     IF NOT OK THEN BEGIN
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
        PSM_INFO_MESSAGE, 'No help available for function ' + nm
        PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     ENDIF 
     GOTO, closing
  ENDIF

  PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
  PSM_INFO_MESSAGE, nm + ': Routine not found in IDL path'
  PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
   

closing:
END
