PRO PSMHELP, template, firstletter=firstletter, list=list, help=help
  
  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'PSMHELP: procedure to print out the list of PSM procedures and functions'
     PSM_INFO_MESSAGE, 'SYNTAX: PSMHELP, template, firstletter='
     PSM_INFO_MESSAGE, "'template' is an optional input that limits the list to those procedures and functions"
     PSM_INFO_MESSAGE, "that contain the template in their names."
     PSM_INFO_MESSAGE, "The 'firstletter' keyword limits the printed list to those procedures"
     PSM_INFO_MESSAGE, "and functions starting by that letter"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  list_dir = STRSPLIT(!PATH,':',/extract)
  psmdir = GETPSMDIR('psmsoft')
  wh = WHERE(STRMID(list_dir,0,STRLEN(psmdir)) EQ psmdir, ndir)

  IF ndir EQ 0 THEN BEGIN

  ENDIF ELSE BEGIN
     list_dir = list_dir[wh]
  ENDELSE

  all_pro_files = ['']

  IF NOT DEFINED(template) THEN BEGIN
     template='*' 
  ENDIF ELSE BEGIN
     IF FIRSTCHAR(template) NE '*' THEN template='*'+template
     IF LASTCHAR(template) NE '*' THEN template=template+'*'
  ENDELSE

  FOR i=0, ndir-1 DO BEGIN
     dir = FIX_SEPARATOR(list_dir[i])
     res = FILE_SEARCH(dir+template+'.pro')
     all_pro_files = [all_pro_files,res]
  ENDFOR

  wh = WHERE(STRLEN(all_pro_files) NE 0, nwh)

  all_pro_files = all_pro_files[wh]
  dirlist = STRARR(nwh)
  prolist = STRARR(nwh)

  ;; Eliminate obsolete software from list
  ;;---------------------------------------
  all_pro_files = PSM_STR_REPLACE(all_pro_files,psmdir,'')
  wh = WHERE(FIRSTCHAR(all_pro_files,8) NE 'obsolete', nwh)
  all_pro_files = all_pro_files[wh]

  ;; Make list of directories and list of procedures
  ;;-------------------------------------------------
  FOR i=0, N_ELEMENTS(all_pro_files)-1 DO BEGIN
     dirlist[i] = FILE_DIRNAME(all_pro_files[i])
     prolist[i] = FILE_BASENAME(all_pro_files[i])
     prolist[i] = PSM_STR_REPLACE(STRUPCASE(prolist[i]),'.PRO','')
  ENDFOR

  ;; Select procedures beginning by first character (if defined)
  ;;-------------------------------------------------------------
  IF DEFINED(firstletter) THEN BEGIN
     wh = WHERE(FIRSTCHAR(prolist) EQ STRUPCASE(firstletter), nwh)
     IF nwh NE 0 THEN BEGIN
        prolist = prolist[wh] 
        dirlist = dirlist[wh]
     ENDIF ELSE BEGIN 
        prolist=['']
     ENDELSE
  ENDIF

  ;; Print programs
  ;;----------------
  maxlen = MAX(STRLEN(prolist))
  ncol = (LINE_LENGTH()-3) / (maxlen+2)

  alphabetical=1
  IF alphabetical THEN BEGIN
     s = SORT(prolist)
     prolist = prolist[s]
     dirlist = dirlist[s]
     alpha = ALPHABET()
     FOR i=0, 25 DO BEGIN
        wh = WHERE(FIRSTCHAR(prolist) EQ alpha[i], nwh)
        IF nwh NE 0 THEN BEGIN
           PRINT, PSM_STRING_TO_LENGTH(alpha[i]+' ',LINE_LENGTH(),character='-')
           plist = prolist[wh]
           nline = CEIL(FLOAT(nwh)/ncol)
           n2add = nline*ncol - nwh
           IF n2add NE 0 THEN plist = [plist,REPLICATE('',n2add)]
           FOR k=0, N_ELEMENTS(plist)-1 DO plist[k] = PSM_STRING_TO_LENGTH('  '+plist[k],LINE_LENGTH()/ncol)
           plist = REFORM(plist,nline,ncol)
           PRINT, TRANSPOSE(plist)
        ENDIF
     ENDFOR
  ENDIF ELSE BEGIN
     s = SORT(dirlist)
     prolist = prolist[s]
     dirlist = dirlist[s]
     alldir = PSM_UNIQ(dirlist)
  ENDELSE


  PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
   

closing:
END
