PRO PSM_SKYSIM, param_table, debug=debug, check_param=check_param

  COMMON psm, param
  COMMON sky, model

  PSM_INFO_TITLE, "MODELLING SKY EMISSION"

  ;; Generate sky components
  ;;-------------------------
  IF SKY_TASK() NE 'restore' THEN BEGIN

     ;; Assign values to sky parameters if there is a new sky simulation performed
     ;;----------------------------------------------------------------------------
     model = READ_SKY_PARAM(param_table)
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, "MODEL PARAMETERS SUCCESFULLY READ"
     IF KEYWORD_SET(check_param) NE 0 THEN BEGIN
        PRTSTRUCT, model, name='MODEL'
        GOTO, closing
     ENDIF ELSE BEGIN
        IF PSM_VERBOSITY() GE 2 THEN PRTSTRUCT, model, name='MODEL'
     ENDELSE

     ;; Bibliography
     ;;--------------
     PSM_ADD2CITE, 'Modelled maps of sky emission are obtained using the '+PSM_STR_REPLACE(PSM_VERSION(),'_','.')+' version of the Planck Sky Model (PSM).'
     PSM_ADD2CITE, 'The original software can be downloaded from:'
     PSM_ADD2CITE, 'http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html'

     ;; COSMO PIPE
     ;;------------
     docosmo = ((STRLOWCASE(CMB_MODEL()) NE 'no_cmb') OR (STRLOWCASE(SZ_MODEL()) NE 'no_sz') OR (model.global.run_camb EQ 'yes') OR (model.global.run_class EQ 'yes')) AND SKY_TASK() EQ 'new'
     IF docosmo THEN BEGIN
        IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "GENERATING COSMOLOGICAL CL AND PK"
        COSMO_PIPE
     ENDIF

     ;; DIPOLE PIPE
     ;;-------------
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING CMB DIPOLE"
     IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "Dipole model: " + DIPOLE_MODEL()      
     IF param.tasks.update_dipole THEN BEGIN
        CLEAR_PSM_DIRECTORY, 'dipole'
        IF DIPOLE_MODEL() NE 'no_dipole' THEN DIPOLE_PIPE, test=test, debug=debug
     ENDIF

     ;; CMB PIPE
     ;;----------
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING CMB EMISSION"
     IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "CMB model: " + CMB_MODEL()      
     IF param.tasks.update_cmb THEN BEGIN
        IF CMB_MODEL() NE 'no_cmb' THEN BEGIN
           CMB_PIPE, debug=debug
           HEAP_GC
           mem = MEMORY()
           IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, 'CMB GENERATED. ' + NUMBER2STRING((mem(0)/1000L)/1000.) + ' MB OF MEMORY USED'
           IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_MESSAGE, 'MAX MEMORY: ' + NUMBER2STRING((mem(3)/1000L)/1000.) + ' MB'
        ENDIF
     ENDIF
  
     ;; GALACTIC PIPE
     ;;---------------  
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING GALACTIC EMISSION"
     do_gal = (model.gal.synchrotron.model+model.gal.freefree.model+model.gal.spindust.model+model.gal.thermaldust.model+model.gal.co.model) NE 0
     IF do_gal THEN BEGIN
        IF PSM_VERBOSITY() GE 2 THEN BEGIN
           PSM_INFO_MESSAGE, "Galactic model: synchrotron model number " + NUMBER2STRING(model.gal.synchrotron.model)
           PSM_INFO_MESSAGE, "Galactic model: free-free model number " + NUMBER2STRING(model.gal.freefree.model)
           PSM_INFO_MESSAGE, "Galactic model: thermal dust model number " + NUMBER2STRING(model.gal.thermaldust.model)
           PSM_INFO_MESSAGE, "Galactic model: spinning dust model number " + NUMBER2STRING(model.gal.spindust.model)
        ENDIF
        GALACTIC_PIPE, debug=debug
        HEAP_GC
        mem = MEMORY()
        PSM_INFO_MESSAGE, 'GALAXY GENERATED. ' + NUMBER2STRING((mem(0)/1000L)/1000.) + ' MB OF MEMORY USED'
        PSM_INFO_MESSAGE, 'MAX MEMORY: ' + NUMBER2STRING((mem(3)/1000L)/1000.) + ' MB'
     ENDIF ELSE BEGIN
        IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "Galactic model: no_galaxy"
     ENDELSE

     ;; SZ PIPE
     ;;---------
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING SUNYAEV-ZELDOVICH EMISSION"
     IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "SZ model: " + SZ_MODEL()      
     IF SZ_MODEL() NE 'no_sz' THEN BEGIN
        SZ_PIPE, debug=debug
        HEAP_GC
        mem = MEMORY()
        PSM_INFO_MESSAGE, 'SZ GENERATED. ' + NUMBER2STRING((mem(0)/1000L)/1000.) + ' MB OF MEMORY USED'
        PSM_INFO_MESSAGE, 'MAX MEMORY: ' + NUMBER2STRING((mem(3)/1000L)/1000.) + ' MB'
     ENDIF

     ;; POINT SOURCE PIPE
     ;;-------------------  
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING POINT SOURCES"
     IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "PS model: " + model.ps.model      
     IF model.ps.model NE 'no_ps' THEN BEGIN
        PS_PIPE, debug=debug
        HEAP_GC
        mem = MEMORY()
        PSM_INFO_MESSAGE, 'POINT SOURCES GENERATED. ' + NUMBER2STRING((mem(0)/1000L)/1000.) + ' MB OF MEMORY USED'
        PSM_INFO_MESSAGE, 'MAX MEMORY: ' + NUMBER2STRING((mem(3)/1000L)/1000.) + ' MB'
     ENDIF

     ;; FIRB PIPE
     ;;-----------  
     IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_TITLE, "MODELLING INFRARED BACKGROUND"
     IF PSM_VERBOSITY() GE 2 THEN PSM_INFO_MESSAGE, "FIRB model: " + model.firb.model      
     IF model.firb.model NE 'no_firb' THEN BEGIN
        FIRB_PIPE, debug=debug
        HEAP_GC
     ENDIF

     SAVE_PSM_SEEDS
     SAVE_SKY_MODEL
     
     PSM_INFO_SUBTITLE, 'END OF THE GENERATION OF THE MODEL SKY'

  ENDIF

closing:

END
