; Copyright 2007 ----

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Tool to handle in a consistent way seeds for random number generators in the PSM.
; The GETSEED function returns an unsigned long integer which can be used to re-initialise the
; generic state of the seed of IDL random number generators.
; GETSEED() returns seeds in 1024 different chains, each of which is named (e.g. 'cmb-pipe'. 
; A call result = GETSEED('cmb-pipe') returns the next unused seed in the 'cmb-pipe' chain, and sets the current chain to 'cmb-pipe'.
; A call result = GETSEED() returns the next unused seed in the current chain.
;
; @keyword {in}{optional}{string} A name of a chain
;
; @history <p>31/01/2007: First version, Marc Betoule </p>
; @history <p>19/01/2009: Now !current_seed = !current_seed + 10 instead of +1 
;   to make sure no seed is in common between pipes for all components, Jacques Delabrouille </p>
; @history <p>05/02/2009: Added verbosity level handling, Jacques Delabrouille </p>
; @history <p>05/09/2009: Now !current_seed = !current_seed + 250 instead of +10 (more seeds needed), Jacques Delabrouille </p>
; @history <p>09/09/2009: Fully rewritten, introducing the concept of chains, Jacques Delabrouille </p>
; 
;-
FUNCTION GETSEED, chain_name=chain_name

COMMON psm, param

  DEFSYSV, '!psm_seeds', exists=exists
  IF NOT exists THEN BEGIN
     IF GET_CAREFULNESS() EQ 1 THEN PSM_INFO_MESSAGE, 'Need to initialize the !psm_seeds system variable using PSM_SEEDINIT routine prior to any call to GETSEED'
     IF GET_CAREFULNESS() GE 2 THEN MESSAGE, 'Need to initialize the !psm_seeds system variable using PSM_SEEDINIT routine prior to any call to GETSEED'
     PSM_SEEDINIT 
  ENDIF

;;-----------------------------------------------------------------------------------------
;; READ THE PSM_SEEDS STRUCTURE AND ASSIGN NEW CURRENT VALUES
;;-----------------------------------------------------------------------------------------
   
  ;; If chain keyword is set
  IF KEYWORD_SET(chain_name) NE 0 THEN SET_PSM_SEEDCHAIN, chain_name
  
  ;; number by which we should shift the current seed value at each call
  shiftseed = N_ELEMENTS(!psm_seeds.chain_name)


;  IF DEFINED(param) THEN BEGIN
;     verbosity = param.run.verbosity
;     IF verbosity EQ 1 THEN PRTSTRUCT, !psm_seeds.current, name='!PSM_SEED.CURRENT'
;     IF verbosity GE 2 THEN PSM_SEEDSTATUS
;  ENDIF

  returnseed = !psm_seeds.current.value
  !psm_seeds.current.value = returnseed + shiftseed
  !psm_seeds.chain_value[!psm_seeds.current.chain] = returnseed + shiftseed

  RETURN, returnseed

  closing:
END








;;   verbosity = 0
;;   DEFSYSV, '!psm_info_param', exists=exists
;;   IF exists THEN verbosity = !psm_info_param.verbosity

;;   DEFSYSV, '!current_seed', exist=exist
;;   IF NOT exist THEN DEFSYSV, '!current_seed', 1
;;   !current_seed = !current_seed + 250L
;;   IF verbosity GE 2 THEN MESSAGE, 'seed = ' + NUMBER2STRING(!current_seed), /info
;;   RETURN, !current_seed

;; END
