PRO WRITE_CMB_ALM, alm, fitsfile, cmb

  ;; cmb is the cmb component structure (input)

  file_exists = FILE_TEST(fitsfile)
  IF file_exists THEN BEGIN
     IF PSM_CAREFULNESS() GE 2 THEN BEGIN
        PSM_INFO_MESSAGE, 'CMB alm file already exists: STOPPING THE PSM RUN'
        PSM_INFO_MESSAGE, 'Type .c to delete existing file ('+cmb_alm_file+') and continue'
        STOP
        FILE_DELETE, fitsfile
     ENDIF ELSE BEGIN
        FILE_DELETE, fitsfile
     ENDELSE
  ENDIF        
  IF PSM_VERBOSITY() GE 1 THEN PSM_INFO_SUBTITLE, 'WRITING THE CMB ALM'
  CMB_ALM_HEADER, hdr, id=cmb.id
  PSM_ID_KEYGEN, cmbalmkey
  UPDATEHDRPAR, hdr, 'PSM_DTID', cmbalmkey

  IF ISPOINTER(alm) THEN BEGIN
     nalm = (size(*alm,/dimensions))[0]
  ENDIF ELSE BEGIN
     nalm = (size(alm,/dimensions))[0]
  ENDELSE
  lmax = NALM2LMAX(nalm)
  UPDATEHDRPAR, hdr, 'PSM_LMAX', lmax
  hdrt = hdr & hdre = hdr & hdrb = hdr
  UPDATEHDRPAR, hdrt, 'ALM_FLD', 'T       '
  UPDATEHDRPAR, hdre, 'ALM_FLD', 'E       '
  UPDATEHDRPAR, hdrb, 'ALM_FLD', 'B       '

  skypix = SKY_PIX()
  WRITE_ALM, alm, fitsfile, coordsys=skypix.coordsys
  FITS_INFO, fitsfile, n_ext=n_ext, /silent

  FOR i=1, n_ext DO BEGIN
     h = HEADFITS(fitsfile, exten=i)
     extname = SXPAR(h,'EXTNAME',count=count)
     CASE extname OF
        'TEMPERATURE alm': hdr = hdrt
        'GRAD/ELECTRIC alm': hdr = hdre
        'CURL/MAGNETIC alm': hdr = hdrb
     ENDCASE

     ;; Add units
     SXADDPAR, h, 'TUNIT1', 'unitless', after='TFORM1'
     SXADDPAR, h, 'TUNIT2', cmb.e1.ampl.unit, after='TFORM2'
     SXADDPAR, h, 'TUNIT3', cmb.e1.ampl.unit, after='TFORM3'
     ;; Add PSM HEADER
     nel = N_ELEMENTS(h)
     h = [h[0:nel-2],hdr,h[nel-1]]
     MODFITS, fitsfile, 0, h, exten_no=i
  ENDFOR

END
