; This procedure is called in CREATE_SZ_SIMULATION in order to compute
; the cluster velocities using the box Universe

;-------------------------------------------------------------------------------------------------------
PRO VEL_XYZ_2_RTHPHI, x,y,z,vx,vy,vz, vr,vt,vp, input_rlonlat=input_rlonlat

IF NOT KEYWORD_SET(input_rlonlat) THEN BEGIN
	CARTESIAN2SPHERICAL, x,y,z,r,th,phi, /radian
ENDIF ELSE BEGIN
	r=x
	th=!pi/2-lat*!dpi/180d0
	phi=lon*!dpi/180d0
ENDELSE
vr = (x*vx + y*vy + z*vz)/r
vt = vx*COS(th)*COS(phi) + vy*COS(th)*SIN(phi) - vz*SIN(th)
vp = -vx*SIN(phi) + vy*COS(phi)
END
;-------------------------------------------------------------------------------------------------------
PRO GET_VELOCITY_BOXES, box, cell_size, f_box, D_g_box, redshift_box, vx, vy, vz, $
	                  cosmo=cosmo

;the generation is made according to Peeble's formula (5.117) p. 116
;v = (2/3)*(f/omega_m/H)*g, where g is the peculiar gravitationnal acceleration
;g = -grad(phi)/a and laplacian(phi) = 4 pi G rho_background a^2 delta
;so that in Fourier Space: 	g = -ik.(-4 pi G rho_background a delta(k)/|k|^2)
;with    					rho_background a 	= rho_present/(1+z)^2  <- c'est l'inverse
;or						rho_background a 	= rho_crit*omega_m/(1+z)^2 <- tjs l'inverse
;in the end v = (2/3)*(1/H)*(4*pi*G*rho_crit) * f/(1+z)^2 * delta(k) * k/|k|^2 
;OR, replacing rho_crit by its value
;v = f H/(1+z) * delta(k) * ik/|k|^2   <- le resultat est juste
;note that here H = H(z) and delta = delta(z)
;so that in the end we still multiply by D_g and replace h by h(z)
	
fftdelta = FFT(box)
dimensions = [N_ELEMENTS(fftdelta(*,0,0)),N_ELEMENTS(fftdelta(0,*,0)),N_ELEMENTS(fftdelta(0,0,*))]
ndim = N_ELEMENTS(dimensions)			;number of dimensions of the result
kinc = 2.*!pi/cell_size/(dimensions>1.)	;increment in k for each dimension (in Mpc^(-1))

;Generate k2 values
k2 = 0.*box
k2(*,*,0) = (DIST(dimensions(0),1)#REPLICATE(1,dimensions(1))*kinc(0))^2 $
	+ (REPLICATE(1,dimensions(0))#DIST(dimensions(1),1)*kinc(1))^2
IF dimensions(2) GT 1 THEN BEGIN & $
     distz2 = (DIST(dimensions(2),1)*kinc(2))^2 & $
	FOR iz=1, dimensions(2)-1 DO k2(*,*,iz) = k2(*,*,0) + distz2(iz) & $
ENDIF
k2=FLOAT(k2)

;Generate kx values
kx = 0.*box
FOR i=0, dimensions(0)/2 DO kx(i,*,*)=i
IF dimensions(0) GT 2 THEN FOR i=1, dimensions(0) - dimensions(0)/2 - 1 DO kx(dimensions(0)-i,*,*) = -i
kx = kx*kinc(0)
kx = FLOAT(kx)

;Generate ky values
ky = 0.*box
FOR i=0, dimensions(1)/2 DO ky(*,i,*)=i
IF dimensions(1) GT 2 THEN FOR i=1, dimensions(1) - dimensions(1)/2 - 1 DO ky(*,dimensions(1)-i,*) = -i
ky = ky*kinc(1)
ky = FLOAT(ky)
;Generate kz values
kz = 0.*box
FOR i=0, dimensions(2)/2 DO kz(*,*,i)=i
IF dimensions(2) GT 2 THEN FOR i=1, dimensions(2) - dimensions(2)/2 - 1 DO kz(*,*,dimensions(2)-i) = -i
kz = kz*kinc(2)
kz = FLOAT(kz)

minik2 = MIN(kinc^2)
vx = fftdelta*kx/(k2>minik2)*COMPLEX(0,1)
vy = fftdelta*ky/(k2>minik2)*COMPLEX(0,1)
vz = fftdelta*kz/(k2>minik2)*COMPLEX(0,1)
vx = FLOAT(FFT(vx,1))
vy = FLOAT(FFT(vy,1))
vz = FLOAT(FFT(vz,1))

vx = HUBBLE_AT_Z(redshift_box, cosmo=cosmo)* $
     f_box*D_g_box*vx/(1+redshift_box)
vy = HUBBLE_AT_Z(redshift_box, cosmo=cosmo)* $
     f_box*D_g_box*vy/(1+redshift_box)
vz = HUBBLE_AT_Z(redshift_box, cosmo=cosmo)* $
     f_box*D_g_box*vz/(1+redshift_box)
END
