PRO ANG2SIZE, ang, size, z, reverse=reverse, cosmo=cosmo
;+
; NAME:
;     ANG2SIZE
; ONE LINE HELP:
;     program to convert angular diameter (degrees) to physical size (Mpc)
; PURPOSE:
;     program to convert angular diameter (degrees) to physical size (Mpc)
; CALLING SEQUENCE:
;     ANG2SIZE, ang, size, z, reverse=reverse, cosmo=cosmo
; INPUT:
;     ang : the angle in degrees
;     OR
;     size : the size in Mpc
;     z : the redshift
; OUTPUT:
;     size : the size in Mpc
;     OR
;     ang : the angle in degrees
; KEYWORDS:
;     reverse : set this keyword to compute ang from size
;     cosmo : the structure containing the cosmological parameters
; EXAMPLE :
;     
; RESTRICTIONS:
;     Not fully tested yet. Report any bugs to Jacques Delabrouille
;     j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;     ANG_DIST
;
; REVISION HISTORY
;     Written, Jean-Baptiste Melin & Jacques Delabrouille April 2001
;     Distances are now in Mpc, JBM, May 30 2001
;     Small bug on da fixed da[0]->da, JBM, Sept 13 2001
;     Keyword cosmo introduced, JBM, Sept 27 2001
;-

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : ANG2SIZE, ang, size, z'
        print,'Keywords : reverse=reverse, cosmo=cosmo'
     GOTO, closing
ENDIF

; compute the angular distance of the object
da = ANG_DIST(z, cosmo=cosmo)

IF KEYWORD_SET(reverse) EQ 0 THEN BEGIN
   size = ang * !dtor * da
ENDIF ELSE BEGIN
   ang = size / !dtor / da
ENDELSE

closing:
END
