;FUNCTION COM_DIST, z_array, cosmo=cosmo
;+
; NAME:
;    COM_DIST
; ONE LINE HELP:
;    give the comoving distance of a source (Mpc)
; PURPOSE:
;    give the comoving distance of a source (Mpc)
; CALLING SEQUENCE:
;    com_dist_array = COM_DIST(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    com_dist_array : the array of comoving distances corresponding to z_array
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jean-Baptiste Melin
;    jean-baptiste.melin@cea.fr
;
; PROCEDURES CALLED:
;    MATTIG, HUBBLE_AT_Z
;
; REVISION HISTORY
;    Written from ang_dist.pro, Jean-Baptiste Melin, June 30 2009
;
;    ang_dist.pro revision history
;    Written, Jean-Baptiste Melin April 2001
;    Keyword /DOUBLE added in QROMB and new example in header, JBM May 2001
;    Correction of a small bug in the order of the result, JBM May 21 2001
;    Does not return a 1 element array anymore
;      if z_array is not an array at input, JBM+JD, may 29 2001
;    Loops in LONG, JBM and JD, may 29 2001
;    Distances in Mpc, JBM, May 30 2001
;    HUBBLE_AT_Z introduced, JBM, Sept 24 2001
;    Keyword cosmo introduced, JBM, Sept 27 2001
;    Bug (H0 forgotten) fixed for spherical and hyperbolic cosmo, JBM, Nov 22 2002
;    Last loop in LONG, JBM, April 12 2006
;-

;------------------------------------------------------------------------------
FUNCTION MATTIG, u
;-----------------
  COMMON MATTIG_COSMO, n_cosmo 
  hz = HUBBLE_AT_Z(u-1., cosmo=n_cosmo)
  RETURN, 1./hz
END
;------------------------------------------------------------------------------


FUNCTION COM_DIST, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : co_dis = COM_DIST(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

c_light = 2.9979246d+08 ; in m/s

; giving the values of common variables
COMMON MATTIG_COSMO, n_cosmo
n_cosmo = cosmo

save_z = z_array
z_array = z_array[UNIQ(z_array, SORT(z_array))]
n = N_ELEMENTS(z_array)
indices = SORT(z_array)
new_z = z_array[indices]
new_z = [new_z,new_z(n-1)]
integrale = QROMB('mattig', 1., 1.+new_z(0),/DOUBLE)
result = DBLARR(n)

FOR i=0L, n-1 DO BEGIN
  result[i] = c_light/1000.*integrale
  integrale = integrale + QROMB('mattig', 1.+new_z[i], 1.+new_z[i+1],/DOUBLE)
ENDFOR

uresult = DBLARR(n)
uresult[indices] = result
res = DBLARR(N_ELEMENTS(save_z))
FOR i=0L, n-1 DO BEGIN
  wh = WHERE(z_array[i] EQ save_z)
  res[wh]=result[i]
ENDFOR
z_array = save_z
IF NOT ISARRAY(z_array) AND N_ELEMENTS(res) EQ 1 THEN res = res(0)
RETURN, res

closing:
END
