;FUNCTION DERIV_HUBBLE_AT_Z, z_array, cosmo=cosmo
;+
; NAME:
;    DERIV_HUBBLE_AT_Z
; ONE LINE HELP:
;    Compute the value of the derivative of the Hubble parameter dH/da(z) at redshift z for a QCDM model
; PURPOSE:
;    Compute the value of the derivative of the Hubble parameter dH/da(z) at redshift z for a QCDM model
; CALLING SEQUENCE:
;    D_H_array = DERIV_HUBBLE_AT_Z(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    D_H_array : the array of derivative of the Hubble parameters
;                (in respect to the scale factor a) dH/da corresponding to z_array
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters (cf. DEFINE_COSMO)
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written from HUBBLE_AT_Z, Jean-Baptiste Melin, May 10 2005
;    Bug in derivation fixed, JBM, August 26 2005
;-

FUNCTION DERIV_HUBBLE_AT_Z, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : DERIV_HUBBLE_AT_Z(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

h = cosmo.h
omega_m = cosmo.omega_m
omega_q = cosmo.omega_de
w = cosmo.w_dark_energy

result_check = CHECK_MATH()
output = 100. * h * (1/2.) * 1/SQRT(omega_q*(1.+z_array)^(3.*(1+w)) $
                                    + (1.-omega_m-omega_q)*(1.+z_array)^2. $
                                    + omega_m*(1.+z_array)^3.) $
         * (-1) * (3.*(1+w)*omega_q*(1.+z_array)^(3.*(1+w)+1)+2*(1.-omega_m-omega_q)*(1.+z_array)^3.+3*omega_m*(1.+z_array)^4.)
result_check = CHECK_MATH()
IF result_check NE 0 THEN MESSAGE, 'Numerical error in calculation -- Please report bug to PSM coordinators'

RETURN, output
closing:
END
