;FUNCTION HUBBLE_AT_Z, z_array, cosmo=cosmo
;+
; NAME:
;    HUBBLE_AT_Z
; ONE LINE HELP:
;    Compute the value of the Hubble parameter H(z) at redshift z
;    (km/s/Mpc) for a QCDM model
; PURPOSE:
;    Compute the value of the Hubble parameter H(z) at redshift z
;    (km/s/Mpc) for a QCDM model
; CALLING SEQUENCE:
;    H_array = HUBBLE_AT_Z(z_array, cosmo=cosmo)
; INPUT:
;    z_array : an array of redshift values
;
; OUTPUT:
;    H_array : the array of Hubble parameters corresponding to z_array
;    
; KEYWORDS:
;    cosmo : the structure containing the cosmological parameters
; EXAMPLE:
;    
; RESTRICTIONS:
;    Not fully tested yet. Report any bugs to Jacques Delabrouille
;    j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;
; REVISION HISTORY
;    Written from HUBBLE_AT_Z, Jean-Baptiste Melin, July 1 2004
;-

FUNCTION HUBBLE_AT_Z, z_array, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        print,'CALL IS : HUBBLE_AT_Z(z_array)'
        print,'Keyword : cosmo=cosmo'
     GOTO, closing
ENDIF

h = cosmo.h
omega_m = cosmo.omega_m
omega_q = cosmo.omega_de
w = cosmo.w_dark_energy

result_check = CHECK_MATH()
output = 100.*h*SQRT(omega_q*(1.+z_array)^(3.*(1+w))+(1.-omega_m-omega_q)*(1.+z_array)^2.+omega_m*(1.+z_array)^3.)
result_check = CHECK_MATH()
IF result_check NE 0 THEN MESSAGE, 'Numerical error in calculation -- Please report bug to PSM coordinators'

RETURN, output
closing:
END
