;FUNCTION SHETH_TORMEN, delta_c, sigma, z, cosmo=cosmo
;
;+
; NAME:
;  SHETH_TORMEN
; ONE LINE HELP:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Sheth and Tormen 1999 fitting
; PURPOSE:
;  give M/rho0*dNclu/(d_covolume*dln(sigma^(-1))) using the Sheth and Tormen 1999 fitting
; CALLING SEQUENCE:
;  result = SHETH_TORMEN(delta_c, sigma, z, cosmo=cosmo)
;
; INPUT:
;  delta_c : the critical density contrast at redshift z
;  sigma : fluctuation amplitude for scale M and redshift z
;  z : the redshift of computation
; OUTPUT:
;  M/rho0*dNclu/(d_covolume*dln(sigma^(-1)))
;    
; KEYWORDS:
;  cosmo : the structure containing the cosmological parameters
; EXAMPLES:
;  
; RESTRICTIONS:
;  Not fully tested yet. Report any bugs to Jacques Delabrouille
;  j.delabrouille@cdf.in2p3.fr
;
; PROCEDURES CALLED:
;  No procedure called 
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, September 11 2001
;-

FUNCTION SHETH_TORMEN, delta_c, sigma, z, cosmo=cosmo

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
    print,'  CALL IS : SHETH_TORMEN(delta_c, sigma, z)'
    print,'  KEYWORD : cosmo=cosmo'
    GOTO, closing
ENDIF

nu = delta_c/sigma

ga = 0.3222
a = 0.707
p = 0.3

nu_lim = 40.

nu = nu < nu_lim 

RETURN, ga*SQRT(2*a/!pi)*(1+1./(a*nu^2)^p)*nu*EXP(-a*nu^2/2.)

closing:
END
