; Copyright 2009 Jacques Delabrouille, CNRS

; This file is part of the Planck Sky Model.
;
; The Planck Sky Model is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; version 2 of the License.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY, without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model. If not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

;+
; Reads the specific parameters used for the generation of the CMB dipole in the PSM
;
; @categories pipe
;   
; @param table {in}{required}{type=strarr(*,2)} vector of couple (keyword, value)
; @keyword carefulness {in}{optional}{type=integer} level of carefulness in handling missing or repeated keywords in the table. 
;
; @uses fill_struct
; 
; @history <p>22/09/2009: First version, Jacques Delabrouille </p>
; @copyright <p>Jacques Delabrouille, CNRS, 2009</p>
;-
FUNCTION READ_DIPOLE_PARAM, table, carefulness=carefulness, restore=restore


  ;; File where parameters will be written
  ;;---------------------------------------
  file = GETPSMDIR('private',/create)+'dipole_parameters.sav'

  dipole_param = {dipole_model: 'no_dipole', $
                  dipole_glon:263.99, $
                  dipole_glat:48.26, $
                  dipole_ampl:3.355, $
                  dipole_glon_error:0.14, $
                  dipole_glat_error:0.03, $
                  dipole_ampl_error:0.008, $
                  save_dipole_map:'yes' $
                 }  
  

  IF NOT KEYWORD_SET(restore) THEN BEGIN
     FILL_STRUCT, table, dipole_param, carefulness=carefulness
     
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     ;; Dipole pipeline parameters
     ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
     CASE dipole_param.dipole_model OF
        'no_dipole': BEGIN
           dipole = {model:dipole_param.dipole_model}
        END
        'prediction': BEGIN
           dipole = {model:dipole_param.dipole_model, $
                     glon:dipole_param.dipole_glon, $
                     glat:dipole_param.dipole_glat, $
                     ampl:dipole_param.dipole_ampl, $
                     save_map:dipole_param.save_dipole_map $
                    }
        END
        'generic': BEGIN
           dipole = {model:dipole_param.dipole_model, $
                     glon:dipole_param.dipole_glon, $
                     glat:dipole_param.dipole_glat, $
                     ampl:dipole_param.dipole_ampl, $
                     glon_error:dipole_param.dipole_glon_error, $
                     glat_error:dipole_param.dipole_glat_error, $
                     ampl_error:dipole_param.dipole_ampl_error, $
                     save_map:dipole_param.save_dipole_map $
                    }
           
        END
        ELSE: MESSAGE, 'DIPOLE MODEL NOT ALLOWED'
     ENDCASE

     ;; Write the parameters
     ;;----------------------
     SAVE, filename=file, dipole

  ENDIF ELSE BEGIN
     RESTORE, filename=file
  ENDELSE
     

  RETURN, dipole

END
