PRO MAKE_FIRB_MIX

  COMMON psm, param
  COMMON sky, model
  
  dir = GETPSMDIR('psmdata')+'ancillary/models/firb/PEP-GC/'
  
  nurefghz=[0., 100, 143, 217., 353., 545., 857., 1874., 3000.]
  nurefs=nurefghz*1e9
  
  CASE model.firb.add_firb_component OF
     'no':                      ;do nothing
     'yes': BEGIN
        cibcl = PEP_FIRB_CL(lmax=SKY_LMAX())
        fixcl = FLTARR(SKY_LMAX()+1,6)
        fixcl[*,0] = CL_VALUE(cibcl, '100GHz', '100GHz')
        fixcl[*,1] = CL_VALUE(cibcl, '143GHz', '143GHz')
        fixcl[*,2] = CL_VALUE(cibcl, '217GHz', '217GHz')
        fixcl[*,3] = CL_VALUE(cibcl, '353GHz', '353GHz')
        fixcl[*,4] = CL_VALUE(cibcl, '545GHz', '545GHz')
        fixcl[*,5] = CL_VALUE(cibcl, '857GHz', '857GHz')
        blsq = PSM_GET_BL(SKY_RESOLUTION(),lmax=SKY_LMAX()) & blsq=blsq^2
        FOR i=0,5 DO fixcl[*,i] = fixcl[*,i]*model.firb.add_firb_percent_power[i]/100.
        cl = CL_STRUCTURE(nfields=1,lmax=SKY_LMAX()) & cl.c_1_1[*]=1
        PSM_CL2ALM, cl, alm, seed=GETSEED()
     END
  ENDCASE

  FOR i=1, n_elements(nurefs)-1 DO BEGIN
     em = EMLAW_PARAMSTRUCT('POWERLAW',/include_nurange,/include_ampl)
     *em.numax.value = nurefs[i]
     *em.numin.value = nurefs[i-1]
     *em.nuref.value = nurefs[i]
     
     ok = TEST_PSM_DATA(dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits',/get)
     IF ok THEN BEGIN
        CASE model.firb.add_firb_component OF
           'no':BEGIN
              CHANGE_MAP_RESOLUTION, dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', $
                                     beam_out = SKY_RESOLUTION(), nside_out=SKY_NSIDE(), lmax=SKY_LMAX(), ordering_out='RING' 
              ;;GNOMVIEW, dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits'
              ;;GNOMVIEW, GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits'
              ;;cmd ='cp '+dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits '+GETPSMDIR('components')+'firb/'
              ;;spawn, cmd
           END
           'yes':BEGIN
              IF ((i GE 1) AND (i LE 6)) THEN BEGIN
                 ;; Get original map and proper header
                 ;;------------------------------------
                 originalalm = PSM_TMPFILENAME(extension='.fits')
                 PSM_MAP2ALM, dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', originalalm, lmax=SKY_LMAX()
                 hdr = HEADFITS(originalalm,exten=1)
                 ;; Read the original alm file
                 ;;----------------------------
                 READ_ALM, originalalm, firbalm
                 FILE_DELETE, originalalm
                 firbalm[1:*] = firbalm[1:*]*SQRT(1-model.firb.add_firb_percent_power[i-1]/100.)
                 index = ALM_INDEX(0,SKY_LMAX())
                 INDEX2L_ONLY, index, lll
                 ;; Add the new firb component
                 ;;----------------------------
                 firbalm[1:*,0] = firbalm[1:*,0] + alm[1:*,0]*SQRT(fixcl[lll[1:*],i-1])
                 firbalm[1:*,1] = firbalm[1:*,1] + alm[1:*,1]*SQRT(fixcl[lll[1:*],i-1])
                 modifiedalm = PSM_TMPFILENAME(extension='.fits')
                 WRITE_ALM, firbalm, modifiedalm, xhdr1=hdr
                 ;; Change beam
                 ;;-------------
                 beamedalm = PSM_TMPFILENAME(extension='.fits')
                 CHANGE_ALM_RESOLUTION, modifiedalm, beamedalm, beam_out = SKY_RESOLUTION(), lmax=SKY_LMAX()
                 FILE_DELETE, modifiedalm
                 ;; Make map
                 ;;----------
                 PSM_ALM2MAP, beamedalm, GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', $
                              hpx_nside=SKY_NSIDE(), lmax=SKY_LMAX(), ordering='RING'
                 FILE_DELETE, beamedalm
              ENDIF ELSE BEGIN
                 CHANGE_MAP_RESOLUTION, dir+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits', $
                                        beam_out = SKY_RESOLUTION(), nside_out=SKY_NSIDE(), lmax=SKY_LMAX(), ordering_out='RING' 
              ENDELSE
           END
        ENDCASE
     ENDIF ELSE BEGIN
        MESSAGE, 'Input data for FIRB model not available'
     ENDELSE
     
     *em.ampl.value = 'firb_ref_map_'+NUMBER2STRING(nurefghz[i])+'GHz.fits'
     em.ampl.unit = 'MJy/sr'
     
     specindfile = 'firb_specind_e'+NUMBER2STRING(i)+'.fits'
     
     *em.specind.value = specindfile
     ok = EXECUTE('e'+NUMBER2STRING(i)+'=em')
     
  ENDFOR  
  
  ;; Fix frequency limits for first and last emission law
  ;; *e1.numin.value = 0.
  *e8.numax.value = FLOAT('INF')

  comp = COMPONENT_STRUCT(e1,e2,e3,e4,e5,e6,e7,e8, name=param.names.firb_basename, class='diffuse')


  ;; Compute spectral index maps and zero levels
  ;;---------------------------------------------
  tfreqs=[0, 100, 143, 217, 353, 545, 857, 1874, 3000]
  IF param.run.visu GE 2 THEN zerolevels = FLTARR(8)
  i=1
  read_fits_map, GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(tfreqs[i])+'GHz.fits', map_min
  IF param.run.visu GE 2 THEN zerolevels[i-1] = AVG(map_min)
  FOR i=2, 8 DO BEGIN
     read_fits_map, GETPSMDIR('components')+'firb/'+'firb_ref_map_'+NUMBER2STRING(tfreqs[i])+'GHz.fits', map_max
     
     specind = ALOG(map_max/map_min)/alog(tfreqs[i]*1./tfreqs[i-1])
     map_min = TEMPORARY(map_max)
     IF param.run.visu GE 2 THEN zerolevels[i-1] = AVG(map_min)
     wh=WHERE(finite(specind) eq 0, nwh)
     IF nwh ne 0 THEN specind[wh]=1

     specindfile = GETPSMDIR('components')+'firb/'+'firb_specind_e'+NUMBER2STRING(i)+'.fits'

     WRITE_HPXMAP, specindfile, tstokes=specind, ordering='RING'
     IF i eq 2 THEN WRITE_HPXMAP, GETPSMDIR('components')+'firb/'+'firb_specind_e1.fits', tstokes=specind, ordering='RING'
  ENDFOR

  ;; Do the figures
  ;;----------------
  IF param.run.visu GE 1 THEN BEGIN
     figdir = GETPSMDIR('components')+'firb/figures/'
     ok = FILE_TEST(figdir, /directory)
     IF NOT ok THEN FILE_MKDIR, figdir
     FOR i=1, 8 DO BEGIN
        ampfile = 'firb_ref_map_'+NUMBER2STRING(tfreqs[i])+'GHz.fits'
        specindfile = 'firb_specind_e'+NUMBER2STRING(i)+'.fits'
        CASE param.run.output_visu OF 
           'ps': BEGIN
              plotampfile = figdir+PSM_STR_REPLACE(ampfile,'.fits','.ps')
              plotspecindfile = figdir+PSM_STR_REPLACE(specindfile,'.fits','.ps')
              MOLLVIEW, GETPSMDIR('firb')+ampfile, title='FIRB AMPLITUDE AT '+NUMBER2STRING(tfreqs[i])+'GHz', units='MJy/sr', ps=plotampfile
              MOLLVIEW, GETPSMDIR('firb')+specindfile, title='FIRB SPECTRAL INDEX BETWEEN '+NUMBER2STRING(tfreqs[i-1])+'GHz AND '+NUMBER2STRING(tfreqs[i])+'GHz', units='unitless', ps=plotspecindfile
           END
           'png': BEGIN
              plotampfile = figdir+PSM_STR_REPLACE(ampfile,'.fits','.png')
              plotspecindfile = figdir+PSM_STR_REPLACE(specindfile,'.fits','.png')
              MOLLVIEW, GETPSMDIR('firb')+ampfile, title='FIRB AMPLITUDE AT '+NUMBER2STRING(tfreqs[i])+'GHz', units='MJy/sr', png=plotampfile
              WDELETE
              MOLLVIEW, GETPSMDIR('firb')+specindfile, title='FIRB SPECTRAL INDEX BETWEEN '+NUMBER2STRING(tfreqs[i-1])+'GHz AND '+NUMBER2STRING(tfreqs[i])+'GHz', units='unitless', png=plotspecindfile
              WDELETE
           END
        ENDCASE
     ENDFOR
     IF param.run.visu GE 2 THEN BEGIN
        CASE param.run.output_visu OF 
           'ps': BEGIN
              ;; STOP
              ;; plotname = !d.name
              ;; SET_PLOT, 'ps'
              ;; DEVICE, filename=GETPSMDIR('firb')+'figures/firb_Inu.ps', xsize=15, ysize=10, xoffset=1, yoffset=1
              ;; fr = (FINDGEN(90)+11)*1e11
              ;; inu = FIRB_NU_INU(fr,/megajysr,/bestfit2012)/fr
              ;; PLOT, fr/1e9, inu, xtitle='Freg (GHz)', ytitle='I (MJy/sr)', charthick=3, charsize=2, /xlog
              ;; DEVICE, /close
              ;; SET_PLOT, plotname
           END
           'png': BEGIN
           END
        ENDCASE
     ENDIF
  ENDIF

  ;; Write the component structure
  ;;-------------------------------
  IF FIRB_MODEL() NE 'no_firb' THEN BEGIN
     SAVE_COMP, comp
     PRTSTRUCT, comp, name='FIRB', file=GETPSMDIR('firb')+'firb.txt'
     DESTROY_DATA, comp
  ENDIF







END
