; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SAVE_SKY_MODEL
;
; PURPOSE:
;	This procedure is an I/O tool for saving to file the current
;	sky model, hiding the technical detail of the format of the
;	model representation, as well as the format and location of
;	the file saved. It uses at some point (either directly, or in
;	called procedures and functions) the PSM and SKY COMMON
;	parameters (which must be defined). The saved model can then
;	be restored if needed with RESTORE_SKY_MODEL.
;
; CATEGORY:
;	SKY_MODEL
;
; CALLING SEQUENCE:
;	SAVE_SKY_MODEL
;
; INPUTS:
;	None
;
; COMMON BLOCKS:
;	SKY:	model
;
; SIDE EFFECTS:
;	Creates a file in the current PSM output directory
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; MODIFICATION HISTORY:
; 	29/10/2009:	Jacques Delabrouille, First version.
; 	06/11/2009:	JD, Added sophistication to handle model updates
; 	18/12/2009:	JD, Simplify routine (supressed option of
; 	                model updates makes the simplification possible).
; 	22/01/2011:	JD, Some cleaning.
; 	06/01/2012:	JD, doc and PSM license header.
;-
PRO SAVE_SKY_MODEL

  COMMON sky, model
  IF DEFINED(model) THEN SAVE, filename = GETPSMDIR('components',/create)+'sky_model.sav', model

END
