; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
FUNCTION GET_PS_ID_CAT, lon, lat, pos_err, flux=flux, help=help

 IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GET_PS_ID_CAT: function to create a basic catalog for use with source_identification function'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GET_PS_ID_CAT(lon, lat, pos_err, flux=)'
     PSM_INFO_MESSAGE, 'lon, lat and pos_err in radians'
     PSM_INFO_MESSAGE, 'flux in whatever you want, as long as you use the same unit for each catalog'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF


 nsrc = N_ELEMENTS(lon)
     IF N_ELEMENTS(lat) NE nsrc THEN BEGIN
        print, 'number of lat and lon elements do not agree'
        RETURN, 0
     ENDIF

     IF N_ELEMENTS(pos_err) NE nsrc THEN BEGIN
        IF N_ELEMENTS(pos_err) NE 1 THEN BEGIN
           print, 'number of elements in pos_err must be 1 or same as lat and lon'
           RETURN, 0
        ENDIF
     ENDIF

     creation_string = "cat = {lon:PTR_NEW(/allocate_heap),lat:PTR_NEW(/allocate_heap),pos_err:PTR_NEW(/allocate_heap)"


     IF DEFINED(flux) THEN BEGIN
        IF N_ELEMENTS(flux) NE nsrc THEN BEGIN
           print, 'number of lat and flux elements do not agree'
           return, 0
        ENDIF
        creation_string = creation_string + ",flux:PTR_NEW(/allocate_heap)"
     ENDIF

     creation_string = creation_string + '}'
     ok = EXECUTE(creation_string)


     *cat.lon = lon
     *cat.lat = lat
     *cat.pos_err = pos_err
     IF DEFINED(flux) THEN *cat.flux = flux

     RETURN, cat

END
