; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FILL_SZ_MODEL_CAT
;
; PURPOSE:
;	 Function to fill a sz catalog.
;
; CATEGORY:
;	SZ effect
;
; CALLING SEQUENCE:
;	FILL_SZ_MODEL_CAT, Formatted_cat, Simulated_cat
;
; INPUTS:
;	Formatted_cat:	Formatted catalogue.
;	Simulated_cat:	Simulated catalogue.
;	
; KEYWORD PARAMETERS:
;	SZP:	SZ parameters structure
;
; OUTPUTS:
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	14/04/2009:	Jacques Delabrouille and Jean-Baptiste Melin,
; 	                First version.
;	18/04/2009:	JD, Now coordinate conversion from degree to
;	                radian in double precision.
;	11/02/2010:	JBM, Structure made compatible with all the cases, bug for cy_500 fixed.
;	06/01/2012:	JD, Updated doc header, included PDM licence header.
;-
PRO FILL_SZ_MODEL_CAT, formatted_cat, simulated_cat, szp=szp

model = szp.cluster_profile
bounds = szp.profile_bounds

CASE STRLOWCASE(model) OF
   'beta': useparam='beta'
   'xmm': useparam='alphabetagamma'
   'chandra': useparam='alphabetagamma'
ENDCASE

nclu_simu = N_ELEMENTS(simulated_cat)
nclu_form = N_ELEMENTS(formatted_cat)
IF nclu_simu NE nclu_form THEN BEGIN
  print, 'The formatted catalogue does not have the same number of elements as the input catalogue! Returning...'
  goto, prob
ENDIF

nclu = nclu_simu
whn = WHERE(strcompress(simulated_cat.name EQ '',/rem), nwhn)
IF nwhn GE 1 THEN BEGIN
  defname=STRTRIM(STRING(LINDGEN(nclu)),1)
  formatted_cat[whn].name=defname[whn]
ENDIF
        
dtor = !DPI/180D0

CASE useparam OF
   'beta': BEGIN
      formatted_cat.name = simulated_cat.name
      formatted_cat.glon_rad = simulated_cat.lon * dtor
      formatted_cat.glat_rad = simulated_cat.lat * dtor
      formatted_cat.z = simulated_cat.z
      formatted_cat.mass = simulated_cat.mass*1e15
      formatted_cat.v_radial = simulated_cat.v_radial
      formatted_cat.v_transverse = simulated_cat.v_transverse
      formatted_cat.v_angle_rad = simulated_cat.angle
      formatted_cat.theta_c = simulated_cat.tc
      formatted_cat.theta_v = simulated_cat.tv
      formatted_cat.cy = simulated_cat.cy
   END
   'alphabetagamma': BEGIN
      IF bounds EQ 'rvir' THEN BEGIN
        formatted_cat.name = simulated_cat.name
        formatted_cat.glon_rad = simulated_cat.lon * dtor
        formatted_cat.glat_rad = simulated_cat.lat * dtor
        formatted_cat.z = simulated_cat.z
        formatted_cat.mass = simulated_cat.mass*1e15
        formatted_cat.v_radial = simulated_cat.v_radial
        formatted_cat.v_transverse = simulated_cat.v_transverse
        formatted_cat.v_angle_rad = simulated_cat.angle
        formatted_cat.theta_s = simulated_cat.ts
        formatted_cat.theta_v = simulated_cat.tv
        formatted_cat.cy = simulated_cat.cy
      ENDIF ELSE BEGIN
        formatted_cat.name = simulated_cat.name
        formatted_cat.glon_rad = simulated_cat.lon * dtor
        formatted_cat.glat_rad = simulated_cat.lat * dtor
        formatted_cat.z = simulated_cat.z
        formatted_cat.mass_500 = simulated_cat.mass500*1e15
        formatted_cat.v_radial = simulated_cat.v_radial
        formatted_cat.v_transverse = simulated_cat.v_transverse
        formatted_cat.v_angle_rad = simulated_cat.angle
        formatted_cat.theta_s = simulated_cat.ts
        formatted_cat.theta_500 = simulated_cat.t500
        formatted_cat.cy_5r500 = simulated_cat.cyn500
      ENDELSE
   END
ENDCASE

prob:

END
