; -----------------------------------------------------------------------------
;
; Copyright (C) 2002, 2006
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     TCCY2Y0
; ONE LINE HELP:
;     Compute y0 if theta_c and Y are given.
; PURPOSE:
;     Compute y0 if theta_c and Y are given.
; CALLING SEQUENCE:
;     y0 = TCCY2Y0(theta_c, cy, cosmo=cosmo, cluster=cluster)
; INPUT:
;     theta_c : theta core array (in arcmin)
;     cy : Y array (in arcmin^2)
; OUTPUT:
;     y0 : central decrement
; KEYWORDS:
;     cosmo : the structure containing the cosmological parameters
;     cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jacques Delabrouille
;     delabrouille@apc.univ-paris7.fr
; PROCEDURES CALLED:
;     FAST_Y0TC2ZM, FAST_TCCY2ZM
; REVISION HISTORY
;     Written, Jean-Baptiste Melin, June 26 2002
;     cluster keyword added, Feb 3 2006
;     QUESTION : Does the result depend on cosmo and cluster ?, JBM, Feb 3 2006
;-
FUNCTION TCCY2Y0, theta_c, cy, cosmo=cosmo, cluster=cluster


IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        PRINT, 'CALL IS : y0 = TCCY2Y0(theta_c, cy)'
        PRINT, 'Keywords : cosmo=cosmo, cluster=cluster'
     GOTO, closing
ENDIF

FAST_TCCY2ZM, theta_c, cy, z, mass, cosmo=cosmo, cluster=cluster

y0 = theta_c
y0[*] = -1.

wh = WHERE(z GE 0, nwh)
IF nwh GE 1 THEN BEGIN
  mas=mass[wh]
  red=z[wh]
  FAST_Y0TC2ZM, sub_y0, sub_tc, red, mas, cosmo=cosmo, cluster=cluster, /reverse
  y0[wh]=sub_y0
ENDIF

RETURN, y0
closing:
END
