; -----------------------------------------------------------------------------
;
; Copyright (C) 2001, 2006
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;  THETA_CORE
; ONE LINE HELP:
;  compute the theta core values of the clusters of masses mass and reshift z
; PURPOSE:
;  compute the theta core values of the clusters of masses mass and reshift z
; CALLING SEQUENCE:
;  result = THETA_CORE(z, mass, cosmo=cosmo, cluster=cluster)
;
; INPUT:
;  z : the array containing the cluster redshifts
;  mass : the array containing cluster masses (in 1d15*M_sun units)
;
; OUTPUT:
;  The theta core values (i.e. core radii over angular distances) (arcmin)
;
; KEYWORD:
;  cosmo : the structure containing the cosmological parameters
;  cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
; EXAMPLES:
;
; RESTRICTIONS:
;  Report any bugs to Jacques Delabrouille
;  delabrouille@apc.univ-paris7.fr
;
; PROCEDURES CALLED:
;  CLUSTER_MODEL, ANG_DIST
; REVISION HISTORY
;  Written, Jean-Baptiste Melin, November 23 2001
;  cluster keyword added, JBM, Feb 3 2006 
;-

FUNCTION THETA_CORE, z, mass, cosmo=cosmo, cluster=cluster

cat = CLUSTER_MODEL(z, mass, cosmo=cosmo, cluster=cluster, /rc)
r_c = cat.rc
ANG2SIZE, coreang, r_c, z, cosmo=cosmo, /reverse
theta_c = 60.*coreang

RETURN, theta_c

END
