; -----------------------------------------------------------------------------
;
; Copyright (C) 2002-2006
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;     Y0TC2CY
; ONE LINE HELP:
;     Compute Y (in arcmin^2) if y0 and theta_c are given.
; PURPOSE:
;     Compute Y (in arcmin^2) if y0 and theta_c are given.
; CALLING SEQUENCE:
;     cy = Y0TC2CY(y0, theta_c, cosmo=cosmo, cluster=cluster)
; INPUT:
;     theta_c : theta core array (in arcmin)
;     y0 : y0 array
; OUTPUT:
;     Y (in arcmin^2)
; KEYWORDS:
;     cosmo : the structure containing the cosmological parameters
;     cluster : the structure containing the structure formation parameters (cf. DEFINE_CLUSTER)
; EXAMPLES:
;     
; ALGORITHM:
;     
; SIDE EFFECTS:
;     
; RESTRICTIONS:
;     Report any bugs to Jacques Delabrouille
;     delabrouille@apc.univ-paris7.fr
; PROCEDURES CALLED:
;     FAST_Y0TC2ZM, FAST_TCCY2ZM
; REVISION HISTORY
;     Written, Jean-Baptiste Melin, June 26 2002
;     cluster keyword added, Feb 3 2006
;     QUESTION : Does the result depend on cosmo and cluster ?, JBM, Feb 3 2006     
;-
FUNCTION Y0TC2CY, y0, theta_c, cosmo=cosmo, cluster=cluster


IF ( N_PARAMS() EQ 0 ) THEN BEGIN
        PRINT, 'CALL IS : cy = Y0TC2CY(y0, theta_c)'
        PRINT, 'Keywords : cosmo=cosmo, cluster=cluster'
     GOTO, closing
ENDIF

FAST_Y0TC2ZM, y0, theta_c, z, mass, cosmo=cosmo, cluster=cluster

FAST_TCCY2ZM, tc, cy, z, mass, cosmo=cosmo, cluster=cluster, /reverse

RETURN, cy
closing:
END
