; -----------------------------------------------------------------------------
;
; Copyright (C) 2006-2011
;   Jacques Delabrouille
;   Guillaume Castex
;   Jean-Baptiste Melin
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
;-
; BE CAREFUL : This procedure is adapted from CREATE_SZ_SIMULATION. It
; includes all sky (not correlated) maps but is not fully tested.
; This procedure is not the final one. The aim is to create a
; CREATE_SZ_SIMULATION with different map types. For the moment,
; there is two different procedures.

; May 29 2006
; input_catalog keyword added, JBM
; June 13 2006
; polar keyword checked for random all-sky maps, JBM
; March 8 2007
; Known clusters are now distributed at |b|>20deg, JBM
; March 20 2007
; "theta_c" field changed as "tc" to be consistent with the cluster extraction algorithm, JBM
; March 21 2007
; z_inf changed from 0.001 to 0.01, JBM
; July 17 2007
; rv (virial radius) and tv (theta virial) included, JBM
; added m_inf keywords, JD, 18 july 2007
; bug in replacing some clusters by the input ones fixed, JBM, feb 15 2008
; chandra, xmm and norm keywords added, JBM, July 24 2008
; z_inf keyword added, JBM, September 24 2008
; buf in m_inf keyword fixed, JBM, September 24 2008
; nr500 keyword added, JBM, Dec 4 2008
; cm500 included in DO_CLUSTER_CATALOG, JBM, Dec 11 2008
; speed increased in PK_SIGMA_VELOCITY call, JBM, Dec 12 2008
; nomap keyword added, JBM, April 17 2009
; not empty cluster name added in order to avoid bugs with mwrfits, JBM, June 15 2009
; use the nearest cluster to change with input_catalog elements, JBM, July 11 2011
; nstd keyword added, JBM, June 14 2012

PRO DO_CLUSTER_CATALOG, z_list, box, D_g_box, cell_size, map_hd, mf_cat, $
	     		  m_inf = m_inf, map_type=map_type, $
                          cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
        lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
        ;lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
        solid_angle = map_ang_size_x * map_ang_size_y * !dtor^2
ENDIF
IF STRLOWCASE(map_type) EQ 'half-sky' THEN solid_angle = 2.*!pi
IF STRLOWCASE(map_type) EQ 'all-sky' THEN solid_angle = 4.*!pi
dimensions = [N_ELEMENTS(box(*,0,0)),N_ELEMENTS(box(0,*,0)),N_ELEMENTS(box(0,0,*))]
deltarho_rho = 0.*z_list

z_cut = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX
boxpix = cell_size/!dtor/((1+z_cut)*ANG_DIST(z_cut, cosmo=cosmo))
mappix = lon_map[1,0]-lon_map[0,0]
underpix_pix = FIX(boxpix/mappix)>1
s = SIZE(lon_map)
map_dim_x = s[1]
map_dim_y = s[2]
underpix_pix = PREVIOUS_EVEN(underpix_pix) ; nx and ny must be both even
underpix_mappix = underpix_pix*mappix
underpix_map_dim_x = CEIL(map_dim_x/FLOAT(underpix_pix))
underpix_map_dim_y = CEIL(map_dim_y/FLOAT(underpix_pix))
underpix_lon_map = (FINDGEN(underpix_map_dim_x)-(underpix_map_dim_x-1.)/2.)# $
                   REPLICATE(underpix_mappix,underpix_map_dim_y)
underpix_lat_map = REPLICATE(underpix_mappix,underpix_map_dim_x)# $
                   (FINDGEN(underpix_map_dim_y)-(underpix_map_dim_y-1.)/2.)
delta_map = FLTARR(underpix_map_dim_x,underpix_map_dim_y)

FOR i=0, N_ELEMENTS(z_list)-1 DO BEGIN
        redshift = z_list[i]
        COMOVING2OBSERVABLE, x_map, y_map, z_map, underpix_lon_map, underpix_lat_map, redshift, $
                               cosmo=cosmo, /reverse
	BOX_PIX2COORD, px, py, pz, x_map, y_map, z_map, dimensions, cell_size, $
                         map_type='small-map',/reverse
	delta_map = box(px,py,pz)*D_g_box(px,py,pz)
        deltarho_rho(i) = AVG(delta_map)
	; HERE SHOULD STILL HAVE PUT IN EFFECT OF LINEAR BIAS!!
ENDFOR

mf_cat = CLUSTER_CATALOG(solid_angle, M_inf, z_list=z_list, deltarho_rho=deltarho_rho, $
			   cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500)
END
;-------------------------------------------------------------------------------------------------------

PRO CREATE_SZ_SIMULATION_ALLSKY, map_hd, catalog, y_map, taubeta_map, taubeta2_q_map, taubeta2_u_map, $
			     m_inf = m_inf, z_inf=z_inf, map_type=map_type, chandra=chandra, norm=norm, xmm=xmm, nr500=nr500, $
			     cosmo=cosmo, cluster=cluster, $
                             mass_function=mass_function, iseed=iseed, nocorrelation=nocorrelation, $
                             input_catalog=input_catalog, $
                             polar=polar, nomap=nomap, nstd=nstd

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF NOT KEYWORD_SET(M_inf) THEN M_inf = 0.1 ; in units of 10^15 solar masses

IF NOT KEYWORD_SET(mass_function) THEN BEGIN
   print, 'Choosing Press-Schechter mass function...'
   mass_function='press_schechter'
ENDIF
				
IF DEFINED(map_type) EQ 0 THEN map_type = 'small-map'
IF DEFINED(map_hd) EQ 0 THEN map_hd=GENERATE_HEADER(nx=360, ny=360, lon_c=0, lat_c=0, $
		pixdeg=1./120., angle=0., coord='G')

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
        IF (NOT (nx/2 EQ nx/2.)) OR (NOT (ny/2 EQ ny/2.)) THEN BEGIN
           PRINT, 'nx and ny must be both even !'
           ; see the repixelisation in add_positions_to_catalog
           GOTO, closing
        ENDIF
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
	lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
	lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
ENDIF

IF NOT KEYWORD_SET(nocorrelation) THEN BEGIN

; case : correlation

;we generate a density contrast box of appropriate dimensions, in comoving coordinates
;-------------------------------------------------------------------------------------

PRINT, '  GENERATING THE DENSITY CONTRAST BOX'

DO_DENSITY_CONTRAST_BOX, map_ang_size_x, map_ang_size_y, dimensions, cell_size, $
			 box, D_g_box, f_box, lon_box, lat_box, redshift_box, $
			 map_type = map_type, $
			 cosmo=cosmo, cluster=cluster, iseed=iseed ;, smooth_size=5*cosmo.h

lon_box=0B
lat_box=0B ; to save memory space

HELP, box
HELP, cell_size

IF NOT KEYWORD_SET(z_inf) THEN z_inf = 0.01 ; if z_inf LE 0.01 they might be some big cluster at very low redshift
z_sup = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX and in the other part of the IF nocorrelation loop.
z_step = 0.001
z_list = (FINDGEN(CEIL((z_sup-z_inf)/z_step)+1)*z_step)+z_inf

;generate the cluster catalog
;----------------------------

PRINT,''
PRINT, '  GENERATING THE CLUSTER CATALOG'

IF KEYWORD_SET(xmm) OR KEYWORD_SET(chandra) THEN BEGIN
  ; Get the value of cm500 to do the mass conversion M_jenkins -> M_vir in cluster_catalog.pro
  get_cm500 = CLUSTER_MODEL(0., 1., /rs, xmm=xmm, chandra=chandra, norm=norm, cosmo=cosmo, cluster=cluster, cm500=cm500, nstd=nstd)
ENDIF

DO_CLUSTER_CATALOG, z_list, box, D_g_box, cell_size, map_hd, mf_cat, $
		      map_type=map_type, $
                      cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500

;add cluster positions
;---------------------

bias = GET_CLUSTERING_BIAS(mf_cat.mass, mf_cat.z, cosmo=cosmo, cluster=cluster)

show=0
ADD_POSITIONS_TO_CATALOG, mf_cat, lon_map, lat_map, box, D_g_box, cell_size, bias, show=show, $
		            cosmo=cosmo, map_type=map_type, iseed=iseed

CATALOG_DRAW_LONLAT, mf_cat, map_hd=map_hd, iseed=iseed

;get cluster velocities
;----------------------
PRINT,''
PRINT, '  GENERATING VELOCITIES'
;PRINT, '  WARNING, STILL NEED TO FIX REDSHIFT DEPENDENCE'
GET_VELOCITY_BOXES, box, cell_size, f_box, D_g_box, redshift_box, vx_box, vy_box, vz_box, $
	              cosmo=cosmo

box=0B
f_box=0B
redshift_box=0B
D_g_box=0B

COMOVING2OBSERVABLE, x, y, z, mf_cat.lon, mf_cat.lat, mf_cat.z, $
                       cosmo=cosmo, /reverse
BOX_PIX2COORD, px, py, pz, x, y, z, dimensions, cell_size, map_type=map_type, /reverse

vc_x = vx_box(px,py,pz)
vc_y = vy_box(px,py,pz)
vc_z = vz_box(px,py,pz)

vx_box=0B
vy_box=0B
vz_box=0B

ENDIF ELSE BEGIN

; case : nocorrelation

; get cluster catalog
;--------------------

IF NOT KEYWORD_SET(z_inf) THEN z_inf = 0.01 ; if z_inf LE 0.01 they might be some big cluster at very low redshift
z_sup = 5. ; be careful redondancy with z_sup in DO_DENSITY_CONTRAT_BOX and in the other part of the IF nocorrelation loop.
z_step = 0.001
z_list = (FINDGEN(CEIL((z_sup-z_inf)/z_step)+1)*z_step)+z_inf

IF NOT KEYWORD_SET(iseed) THEN iseed=LONG(systime(1))

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
        lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
        ;lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
        solid_angle = map_ang_size_x * map_ang_size_y * !dtor^2
ENDIF
IF STRLOWCASE(map_type) EQ 'half-sky' THEN solid_angle = 2.*!pi
IF STRLOWCASE(map_type) EQ 'all-sky' THEN solid_angle = 4.*!pi
deltarho_rho = 0.*z_list

PRINT,''
PRINT, '  GENERATING THE CLUSTER CATALOG'

IF KEYWORD_SET(xmm) OR KEYWORD_SET(chandra) THEN BEGIN
  ; Get the value of cm500 to do the mass conversion M_jenkins -> M_vir in cluster_catalog.pro
  get_cm500 = CLUSTER_MODEL(0., 1., /rs, xmm=xmm, chandra=chandra, norm=norm, cosmo=cosmo, cluster=cluster, cm500=cm500, nstd=nstd)
ENDIF

;M_inf = 0.1 ; bug : this line must be removed because M_inf is now specified by the keyword
mf_cat = CLUSTER_CATALOG(solid_angle, M_inf, z_list=z_list, deltarho_rho=deltarho_rho, $
			   cosmo=cosmo, cluster=cluster, mass_function=mass_function, iseed=iseed, cm500=cm500)

;add cluster positions
;---------------------

n_mf_cat = N_ELEMENTS(mf_cat)
tagnames = TAG_NAMES(mf_cat)
;catalog = CREATE_STRUCT(mf_cat(0),'lon',0.,'lat',0.,'pixel',0L)
catalog = CREATE_STRUCT(mf_cat(0),'lon',0.,'lat',0.)
catalog = REPLICATE(catalog,n_mf_cat)
FOR i=0, N_ELEMENTS(tagnames)-1 DO catalog.(i) = mf_cat.(i)

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN
	nx=SXPAR(map_hd,'NAXIS1')
	ny=SXPAR(map_hd,'NAXIS2')
	map_ang_size_x = nx*ABS(SXPAR(map_hd,'CDELT1'))
	map_ang_size_y =  ny*ABS(SXPAR(map_hd,'CDELT2'))
        lon_map = (FINDGEN(nx)-(nx-1.)/2.)#REPLICATE(ABS(SXPAR(map_hd,'CDELT1')),ny)
        lat_map = REPLICATE(ABS(SXPAR(map_hd,'CDELT2')),nx)#(FINDGEN(ny)-(ny-1.)/2.)
        pixel_cat = LONG(nx*ny*RANDOMU(iseed, n_mf_cat))
        catalog.lon = lon_map[pixel_cat]
        catalog.lat = lat_map[pixel_cat]
        catalog.pixel = pixel_cat
        CATALOG_DRAW_LONLAT, catalog, map_hd=map_hd, iseed=iseed
ENDIF
IF STRLOWCASE(map_type) EQ 'half-sky' THEN BEGIN
        nside=SXPAR(map_hd,'NSIDE')
ENDIF
IF STRLOWCASE(map_type) EQ 'all-sky' THEN BEGIN
        ;nside=SXPAR(map_hd,'NSIDE')
        posi = RANDOMU(iseed, 2, n_mf_cat)
        lon_cat = REFORM(360*posi[0,*])
        lat_cat = REFORM(ASIN(2*posi[1,*]-1)/!dtor) ; uniform random draw on the sphere
        posi=0B
        ;theta = !pi/2.-!dtor*lat_cat
        ;phi = !dtor*lon_cat
        ;ang2pix_ring, nside, theta, phi, pixel_cat
        ;theta = 0B
        ;phi = 0B
        catalog.lon = lon_cat
        catalog.lat = lat_cat
        ;catalog.pixel = pixel_cat
        lon_cat = 0B
        lat_cat = 0B
        ;pixel_cat = 0B
ENDIF

mf_cat = catalog

IF KEYWORD_SET(input_catalog) THEN BEGIN

PRINT,''
PRINT, '  REPLACING SIMULATED CLUSTERS WITH INPUT CATALOG CLUSTERS'

catalog = mf_cat

whlat = WHERE(ABS(catalog.lat) GE 20., nwhlat)

n_i_cat = N_ELEMENTS(input_catalog)
;tagnames = TAG_NAMES(input_catalog)
;cat = CREATE_STRUCT(input_catalog(0),'pixel',0L)
;cat = REPLICATE(cat,n_i_cat)
;FOR i=0, N_ELEMENTS(tagnames)-1 DO cat.(i) = input_catalog.(i)

;theta = !pi/2.-!dtor*cat.lat
;phi = !dtor*cat.lon
;ang2pix_ring, nside, theta, phi, pixel_cat
;theta = 0B
;phi = 0B
;cat.pixel = pixel_cat
cat = input_catalog

perc_z = 0.5 ; 50% bin
perc_M = 0.5 ; 50% bin
nocount = 0L
;PRINT, STRTRIM(STRING(n_i_cat),1)+' clusters in the input_catalog'
already_changed = INTARR(N_ELEMENTS(catalog))
FOR clu_num=0L, n_i_cat-1 DO BEGIN  
  ;PRINT, clu_num+1, n_i_cat
  whz = WHERE(catalog[whlat].z GE cat[clu_num].z*(1-perc_z) AND catalog[whlat].z LT cat[clu_num].z*(1+perc_z), nwhz)
  IF nwhz GE 1 THEN BEGIN
    whm = WHERE(ALOG(catalog[whlat[whz]].mass) GE  ALOG(cat[clu_num].mass*(1-perc_M)) AND $
                ALOG(catalog[whlat[whz]].mass) LT ALOG(cat[clu_num].mass*(1+perc_M)), nwhm)
    IF nwhm GE 1 THEN BEGIN
      whc = WHERE(already_changed[whlat[whz[whm]]] EQ 0, nwhc)
      IF nwhc GE 1 THEN BEGIN
        ;ok = MIN(ABS(ALOG(catalog[whlat[whz[whm[whc]]]].mass)-ALOG(cat[clu_num].mass)),whmin)
        dival=2*ASIN(SQRT((SIN((cat[clu_num].lat*!dtor-catalog[whlat[whz[whm[whc]]]].lat*!dtor)/2))^2+COS(cat[clu_num].lat*!dtor)*COS(catalog[whlat[whz[whm[whc]]]].lat*!dtor)*(SIN((cat[clu_num].lon*!dtor-catalog[whlat[whz[whm[whc]]]].lon*!dtor)/2))^2))/!dtor*60.
        ok = MIN(dival, whmin)
        catalog[whlat[whz[whm[whc[whmin]]]]]=cat[clu_num]
        already_changed[whlat[whz[whm[whc[whmin]]]]] = 1
      ENDIF ELSE BEGIN
        catalog = [catalog, cat[clu_num]]
        nocount=nocount+1
      ENDELSE
    ENDIF ELSE BEGIN
      ;PRINT, 'No cluster mass in the specified range.'
      ;PRINT, 'I add the cluster to the catalog without removing any other cluster.'
      ;PRINT, cat[clu_num].mass
      catalog = [catalog, cat[clu_num]]
      nocount=nocount+1
    ENDELSE
  ENDIF ELSE BEGIN
    ;PRINT, 'No cluster redshift in the specified range.'
    ;PRINT, 'I add the cluster to the catalog without removing any other cluster.'
    ;PRINT, cat[clu_num].z
    catalog = [catalog, cat[clu_num]]
    nocount=nocount+1
  ENDELSE
ENDFOR

mf_cat = catalog
n_mf_cat = N_ELEMENTS(mf_cat)

ENDIF

rho = 2.76e-4*cosmo.omega_m*(cosmo.h)^2
; in units of 10^15 solar masses/Mpc^3

;allmass = mf_cat.mass
;umass = allmass[UNIQ(allmass, SORT(allmass))]
;num = N_ELEMENTS(umass)
;uradii  = (3.*umass/(4.*!pi*rho))^(1./3) ; Mpc
;sigma_val = 0.*mf_cat.z
;FOR i=0L, num-1 DO BEGIN
;  whm = WHERE(allmass EQ umass[i], nwhm)
;  z_cat = mf_cat[whm].z
;  uzcat = z_cat[UNIQ(z_cat, SORT(z_cat))]
;  resu = PK_SIGMA_VELOCITY(uradii[i], uzcat, cosmo=cosmo, cluster=cluster)
;  nuz = N_ELEMENTS(uzcat)
;  FOR j=0, nuz-1 DO BEGIN
;    whz = WHERE(z_cat EQ uzcat[j], nwhz)
;    sigma_val[whm[whz]] = resu[j]
;  ENDFOR
;ENDFOR

sigma_val = 0.*mf_cat.z

min_mass = MIN(mf_cat.mass)
max_mass = MAX(mf_cat.mass)
step_log10 = 20.
log10_mass_list = (FINDGEN(CEIL((ALOG10(max_mass)-ALOG10(min_mass))*step_log10)+1)/step_log10)+ALOG10(min_mass)
mass_list = EXP(ALOG(10)*log10_mass_list)
numm = N_ELEMENTS(mass_list)
radii  = (3.*mass_list/(4.*!pi*rho))^(1./3) ; Mpc
z_cat = mf_cat.z
uzcat = z_cat[UNIQ(z_cat, SORT(z_cat))]
numz = N_ELEMENTS(uzcat)

sigma_val_mass_z = FLTARR(numm,numz)
FOR i=0L, numm-1 DO BEGIN
  ;print, i, numm
  resu = PK_SIGMA_VELOCITY(radii[i], uzcat, cosmo=cosmo, cluster=cluster)
  sigma_val_mass_z[i,*] = resu
ENDFOR
FOR i=0L, numz-1 DO BEGIN
  whz = WHERE(z_cat EQ uzcat[i], nwhz)
  sigma_val[whz] = INTERPOL(sigma_val_mass_z[*,i], mass_list, mf_cat[whz].mass)
ENDFOR

veloc = RANDOMN(iseed, 3, n_mf_cat)
vc_x = sigma_val/SQRT(3)*veloc[0,*]
vc_y = sigma_val/SQRT(3)*veloc[1,*]
vc_z = sigma_val/SQRT(3)*veloc[2,*]
veloc = 0B

ENDELSE

; end cor/nocor

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN

;add velocities to mf_cat
;------------------------
catalog = CREATE_STRUCT(mf_cat(0),'v_radial',0d,'v_transverse',0d,'angle',0.)
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i =0, nnn-1 DO catalog.(i) = mf_cat.(i)

catalog.v_radial = vc_x 				
catalog.v_transverse = sqrt(vc_y^2+vc_z^2)	
catalog.angle = ATAN(vc_z,vc_y)

mf_cat = catalog

ENDIF

IF STRLOWCASE(map_type) EQ 'all-sky' THEN BEGIN

catalog = CREATE_STRUCT(mf_cat(0),'v_radial',0d,'v_transverse',0d,'angle',0.)
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i =0, nnn-1 DO catalog.(i) = mf_cat.(i)

; change coordinates from global to local following HEALPIX convention
; see "HEALPIX Primer"
n_vc_z = vc_x*COS(!dtor*mf_cat.lat)*COS(!dtor*mf_cat.lon) $
        +vc_y*COS(!dtor*mf_cat.lat)*SIN(!dtor*mf_cat.lon) $
        +vc_z*SIN(!dtor*mf_cat.lat)
; e_nz = e_r
n_vc_x = vc_x*SIN(!dtor*mf_cat.lat)*COS(!dtor*mf_cat.lon) $
        +vc_y*SIN(!dtor*mf_cat.lat)*SIN(!dtor*mf_cat.lon) $
        -vc_z*COS(!dtor*mf_cat.lat)
; e_nx = e_theta
n_vc_y = -vc_x*SIN(!dtor*mf_cat.lon)+vc_y*COS(!dtor*mf_cat.lon)
; e_ny = e_phi

vc_x = 0B
vc_y = 0B
vc_z = 0B

catalog.v_radial = n_vc_z 				
catalog.v_transverse = sqrt(n_vc_x^2+n_vc_y^2)	
catalog.angle = ATAN(n_vc_y,n_vc_x)
; this is the angle between the local south direction (x axis) and the transverse velocity 
; towards east (y axis) i.e. counterclockwise
; (see definition of the local tangential frame in the HEALPIX Primer)

n_vc_x = 0B
n_vc_y = 0B
n_vc_z = 0B

mf_cat = catalog

ENDIF

;add cluster parameters
;----------------------
PRINT,''
PRINT, '  MODELLING THE CLUSTERS'
model = CLUSTER_MODEL(mf_cat.z, mf_cat.mass, cosmo=cosmo, cluster=cluster, $
	/Te, /ne0, /tau0, /y0, chandra=chandra, norm=norm,xmm=xmm, c500=c500, cm500=cm500, nstd=nstd)
;catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'tau0',0d,'y0',0d,'theta_c',0.,'cy',0d)
;catalog =
;CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'tau0',0d,'y0',0d,'tc',0.,'cy',0d)
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  IF KEYWORD_SET(nr500) THEN BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d,'r500',0d,'t500',0d,'mass500',0d,'cy500',0d, $
                            'rn500',0d,'tn500',0d,'massn500',0d,'cyn500',0d)
  ENDIF ELSE BEGIN
    catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rs',0d,'rv',0d,'tau0',0d,'y0',0d,'ts',0.,'tv',0.,'cy',0d)
  ENDELSE
ENDIF ELSE BEGIN
  catalog = CREATE_STRUCT(mf_cat(0),'Te',0d,'ne0',0d,'rc',0d,'rv',0d,'tau0',0d,'y0',0d,'tc',0.,'tv',0.,'cy',0d)
ENDELSE
catalog = REPLICATE(catalog,N_ELEMENTS(mf_cat))

tagnames = TAG_NAMES(mf_cat)
nnn = N_ELEMENTS(tagnames)
FOR i=0, nnn-1 DO catalog.(i) = mf_cat.(i)
tagnames = TAG_NAMES(model)
FOR i=0, N_ELEMENTS(tagnames)-1 DO catalog.(i+nnn) = model.(i)

da = ANG_DIST(catalog.z, cosmo=cosmo)
IF KEYWORD_SET(chandra) OR KEYWORD_SET(xmm) THEN BEGIN
  catalog.ts = catalog.rs/!dtor/da*60.
ENDIF ELSE BEGIN
  catalog.tc = catalog.rc/!dtor/da*60.
ENDELSE
catalog.tv = catalog.rv/!dtor/da*60.

catalog.cy = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster,chandra=chandra, norm=norm, xmm=xmm, nstd=nstd)

IF (KEYWORD_SET(chandra) OR KEYWORD_SET(xmm)) AND KEYWORD_SET(nr500) THEN BEGIN
  catalog.r500 = c500*catalog.rs
  catalog.t500 = c500*catalog.ts
  FAST_CONVERT_MASS_RADIUS, catalog.mass, catalog.z, INTERPOL(cluster.big_delta_c_array, cluster.z_array, catalog.z), REPLICATE(500.,N_ELEMENTS(catalog)), $
                            REPLICATE(cm500,N_ELEMENTS(catalog)),rvir,r500,m500,cosmo=cosmo,/five_hundred
  catalog.mass500 = m500
  catalog.cy500 = FAST_INT_Y(catalog.z, catalog.mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=1., nstd=nstd)
  catalog.rn500 = nr500*catalog.r500
  catalog.tn500 = nr500*catalog.t500
  catalog.massn500 = (ALOG(1.+nr500*cm500)-nr500*cm500/(1.+nr500*cm500))/(ALOG(1.+cm500)-cm500/(1.+cm500))*catalog.mass500
  factn500 = FAST_INT_Y(catalog[0].z, catalog[0].mass, cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, nstd=nstd)/catalog[0].cy500
  catalog.cyn500=factn500[0]*catalog.cy500
ENDIF

mf_cat = catalog
catalog = 0B

;make SZ maps
;------------

z_limits = [0.,10.]
which = WHERE( (mf_cat.z GE z_limits(0)) AND (mf_cat.z LE z_limits(1)) )
catalog = mf_cat(which)
mf_cat = 0B

nclu = N_ELEMENTS(catalog)
whn = WHERE(strcompress(catalog.name,/rem) EQ '', nwhn)
IF nwhn GE 1 THEN BEGIN
  defname=STRTRIM(STRING(LINDGEN(nclu)),1)
  catalog[whn].name=defname[whn]
ENDIF

IF NOT KEYWORD_SET(nomap) THEN BEGIN

PRINT, ''
PRINT, '  GENERATING THE SZ MAPS'

IF STRLOWCASE(map_type) EQ 'small-map' THEN BEGIN

CREATE_Y_TAUBETA_MAP, map_hd, catalog, y_map, taubeta_map, $
                          taubeta2_q_map, taubeta2_u_map, $
                          cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, $
                          polar=polar, nstd=nstd

ENDIF

IF STRLOWCASE(map_type) EQ 'all-sky' THEN BEGIN

  nside = SXPAR(map_hd, 'NSIDE')
		
  n_i_cat = N_ELEMENTS(catalog)
  tagnames = TAG_NAMES(catalog)
  cat = CREATE_STRUCT(catalog(0),'pixel',0L)
  cat = REPLICATE(cat,n_i_cat)
  FOR i=0, N_ELEMENTS(tagnames)-1 DO cat.(i) = catalog.(i)
  theta = !pi/2.-!dtor*cat.lat
  phi = !dtor*cat.lon
  ang2pix_ring, nside, theta, phi, pixel_cat
  theta = 0B
  phi = 0B
  cat.pixel = pixel_cat
  catalog=cat
  cat=0B

  CREATE_Y_TAUBETA_MAP_ALLSKY, nside, catalog, y_map, taubeta_map, $
                             taubeta2_q_map, taubeta2_u_map, $
                             cosmo=cosmo, cluster=cluster, chandra=chandra, xmm=xmm, norm=norm, nr500=nr500, $
                             polar=polar, nstd=nstd

ENDIF 

ENDIF

;visualisation
;-------------
;simplevisu, y_map, mag=2, /no
;simplevisu, taubeta_map, mag=2, /no
;radial(catalog.pixel) = vc_x(which)
;simplevisu, radial, mag=2, /no

;maxlength = 12	;length of mark in pixels
;factor = FLOAT(maxlength)/MAX(catalog.v_transverse)/[nx,ny]
;FOR i=0, N_ELEMENTS(catalog)-1 DO BEGIN & $
;	veloc =[catalog[i].v_transverse*COS(catalog[i].angle),catalog[i].v_transverse*SIN(catalog[i].angle)] & $
;	start = [(catalog[i].pixel MOD nx)/FLOAT(nx),(catalog[i].pixel/nx)/FLOAT(ny)] & $
;	ending = start+veloc*factor & $
;	xxx = [start(0),ending(0)] & $
;	yyy = [start(1),ending(1)] & $
;	plot, xxx,yyy,/noerase,position=[0,0,1,1],xrange=[0,1],yrange=[0,1] & $
;ENDFOR
;
;simplevisu, taubeta2_q_map, mag=2, /no
;simplevisu, taubeta2_u_map, mag=2, /no

closing:
END
