; -----------------------------------------------------------------------------
;
; Copyright (C) 2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	GREYBODY
;
; PURPOSE:
;	...
;
;-
FUNCTION GREYBODY, nu, p, help=help

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 2) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'GREYBODY: a greybody (modified blackbody) emission'
     PSM_INFO_MESSAGE, 'SYNTAX: result = GREYBODY(nu, p)'
     PSM_INFO_MESSAGE, 'nu is frequency in Hz'
     PSM_INFO_MESSAGE, 'p is a 7-element vector of parameters of the powerlaw and greybody emissions'
     PSM_INFO_MESSAGE, 'p[0] is the reference frequency for the greybody'
     PSM_INFO_MESSAGE, 'p[1] is the flux at reference frequency for the greybody'
     PSM_INFO_MESSAGE, 'p[2] is the spectral index of the greybody'
     PSM_INFO_MESSAGE, 'p[3] is the temperature of the greybody'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  flux = p[1] * PSM_EMLAW(nu, emission_name='GREYBODY', nuref=p[0], specind=p[2], temp=p[3])

  RETURN, flux

END

