; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2011
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ROUTINE_NAME
;
; PURPOSE:
;	...
;
;-
PRO DESTROY_DATA_REC, data

  ;; list of data types which are not recursive
  ;;--------------------------------------------
  nonrec = [0,1,2,3,4,5,6,7,9,12,13,14,15]


  ;; look recursively in structures and pointers
  ;;---------------------------------------------
  IF DEFINED(data) THEN BEGIN

     IF ISPOINTER(data) THEN BEGIN

        nel = N_ELEMENTS(data)
        IF nel NE 1 THEN BEGIN
           FOR j=0L, nel-1 DO BEGIN
              DESTROY_DATA_REC, data[j]
              PTR_FREE, data[j]
           ENDFOR
        ENDIF ELSE BEGIN

           IF PTR_VALID(data) THEN BEGIN
              
              s = SIZE(*data,/type)
              wh = WHERE(nonrec EQ s,nwh)
              IF nwh NE 0 THEN BEGIN
                 PTR_FREE, data
              ENDIF ELSE BEGIN 
                 DESTROY_DATA_REC, *data
                 PTR_FREE, data
              ENDELSE
              
           ENDIF ELSE BEGIN
              PTR_FREE, data
           ENDELSE

        ENDELSE

     ENDIF

     IF ISSTRUCT(data) THEN BEGIN
        tags = TAG_NAMES(data)
        ntags = N_ELEMENTS(tags)
        FOR i=0, ntags-1 DO ok = EXECUTE('DESTROY_DATA_REC, data.'+tags[i])
     ENDIF
        
  ENDIF

END


PRO DESTROY_DATA, data, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'DESTROY_DATA: procedure to erase data from memory'
     PSM_INFO_MESSAGE, 'SYNTAX: DESTROY_DATA, data'
     PSM_INFO_MESSAGE, "The input data will be completely destroyed (pointers freed, arrays erased, etc.)"
     PSM_INFO_MESSAGE, "Warning: To be used with some care!"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  IF DEFINED(data) THEN BEGIN

     DESTROY_DATA_REC, data
     IF ISPOINTER(data) THEN BEGIN
        IF ISARRAY(data) THEN BEGIN
           FOR i=0L, N_ELEMENTS(data)-1L DO PTR_FREE, data[i]
           a = TEMPORARY(data)
        ENDIF ELSE BEGIN
           PTR_FREE, data
        ENDELSE
     ENDIF ELSE BEGIN
        a = TEMPORARY(data)
     ENDELSE
  ENDIF

END


