; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	FIX_PRECISION
;
; PURPOSE:
;	Convert data to single or double precision type.
;
; CATEGORY:
;	Put a category (or categories) here.  
;
; CALLING SEQUENCE:
;	FIX_PRECISION, Data
;
; INPUTS:
;	Data:	Input data (can be an array) of floating point or
;	        double precision, or pointer to such data.
;
; KEYWORD PARAMETERS:
;	DOUBLE:	Define DOUBLE=0 to force conversion of data to
;	floating point precision, and DOUBLE=1 to force conversion to
;	double precision. WARNING: the command: 
;          IDL> FIX_PRECISION, data, double=0 
;	is thus different in effect from 
;          IDL> FIX_PRECISION, data
;	If the DOUBLE keyword is not defined, the output precision is
;	obtained with the PRECISION_ISDOUBLE function.
;
; OUTPUTS:
;	The modified data (type changed to double or single precision).
;
; SIDE EFFECTS:
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	11/10/2010:	Jacques Delabrouille, First version.
;	January 2012:	JD, Now code leaves bytes, integers, and long
;	                integers unmodified. 
;-
PRO FIX_PRECISION, data, double=double

  typ = SIZE(data,/type)
  IF DEFINED(double) THEN db=double ELSE db = PRECISION_ISDOUBLE()

  IF typ LE 3 THEN GOTO, closing

  IF typ LE 5 THEN BEGIN
     IF db THEN data = DOUBLE(TEMPORARY(data)) ELSE data = FLOAT(TEMPORARY(data))
     RETURN
  ENDIF

  IF typ EQ 10 THEN BEGIN
     
     pnttyp = SIZE(*data,/type)
     IF typ LE 5 THEN BEGIN
        IF db THEN *data = DOUBLE(TEMPORARY(*data)) ELSE *data = FLOAT(TEMPORARY(*data))     
     ENDIF
     RETURN
  ENDIF
  
closing:
END



