; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2011, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	ISFITSFILE
;
; PURPOSE:
;	...
;
;-
FUNCTION ISFITSFILE, name, help=help

  IF (KEYWORD_SET(help) NE 0) OR (N_PARAMS() NE 1) THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'ISFITSFILE: function which returns 1 if name is a valid fits file, and 0 otherwise'
     PSM_INFO_MESSAGE, 'SYNTAX: result = ISANUMBER(name)'
     RETURN, 0
  ENDIF

  ON_ERROR, 2

  ;; First check that the input is a string
  ;;----------------------------------------
  s = SIZE(name,/type)
  IF s NE 7 THEN RETURN, 0B

  ;; If so then check whether a file with that name exists
  ;;-------------------------------------------------------
  exists = FILE_TEST(name)     
  IF NOT exists THEN RETURN, 0B
  
  ;; If a file exists then check that it contains a valid header,
  ;; in which case it is assumed to be a valid fits file
  ;;--------------------------------------------------------------
  h = HEADFITS(name, errmsg=errmsg, /silent)
  IF STRLEN(errmsg) EQ 0 THEN RETURN, 1B ELSE RETURN, 0B

END
