; -----------------------------------------------------------------------------
;
; Copyright (C) 2007-2012
;   Jacques Delabrouille
;   Guillaume Castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	VECTOR_ROTATION
;
; PURPOSE:
;	Rotate a vector in 3-D space.
;
; CATEGORY:
;	Mathematics
;
; CALLING SEQUENCE:
;	VECTOR_ROTATION, Vector, Rotvec, Angle, New_vector
;
; INPUTS:
;	Vector: 	Vector to be rotated (in 3D space).
;
;	Rotvec: 	Direction of the axis of rotation (oriented).
;	
;	Angle:  	Angle of rotation (radians).
;	
; OUTPUTS:
;	New_vector:  	The vector after the rotation.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple mathematics.
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	03 July 2009:	Guillaume Castex, First version.
;	January 2012:	Jacques Delabrouille, some general clean-up,
;	                updated header documentation. 
;-

PRO VECTOR_ROTATION, vector, rotvec, angle, new_vector

  ;; First normalize the rotation vector
  ;;-------------------------------------
  squa_length = TOTAL(rotvec^2)
  rotation_vector = rotvec/SQRT(squa_length)
  
  ;; Compute the new vector
  ;;------------------------
  cos_angle = COS(angle)
  second_term = (1-cos_angle)*(TOTAL(rotation_vector*vector))
  new_vector = 0.*vector           ; keep the type (float, double) unchanged 
  new_vector[*]= cos_angle*vector + second_term*rotation_vector + SIN(angle)*crossp(rotation_vector,vector)
  
END
