; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HPX_OUTLIERS
;
; PURPOSE:
;	...
;
;-
PRO HPX_OUTLIERS, input_map, outliers, noutliers, ordering=ordering, sigmathreshold=sigmathreshold, $
                  getstats=getstats, localmean=localmean, localsigma=localsigma, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HPX_OUTLIERS: procedure to find outliers in a healpix map'
     PSM_INFO_MESSAGE, 'SYNTAX: HPX_OUTLIERS, input_map, outliers, noutliers, ordering=, sigmathreshold=, /getstats,'
     PSM_INFO_MESSAGE, 'localmean=, localsigma='
     PSM_INFO_MESSAGE, "input_map is the input HEALPix file"
     PSM_INFO_MESSAGE, "outliers is the list of pixel numbers for outliers"
     PSM_INFO_MESSAGE, 'noutliers is the number of outliers found'
     PSM_INFO_MESSAGE, "RING ordering is assumed, unless 'ordering' keyword is set to 'NESTED'"
     PSM_INFO_MESSAGE, "set 'sigmathreshold' to the minimum number of (local) standard deviations to flag the pixel"
     PSM_INFO_MESSAGE, "[default=10]"
     PSM_INFO_MESSAGE, "set /getstats to output local mean and local sigma in the corresponding keywords"

     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF


  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'
  IF NOT KEYWORD_SET(sigmathreshold) THEN sigmathreshold=10
  nside = GET_NSIDE(input_map)

  niter = 1 & degrade=8
  mean = HPX_LOCALMEAN(input_map, ordering=ordering, degrade=degrade, niter=niter, minpts=64)

  whok = WHERE((input_map NE !healpix.bad_value) AND (mean NE !healpix.bad_value), nwhok, complement=whbad, ncomplement=nwhbad)
  sigma2 = input_map
  IF nwhok NE 0 THEN sigma2[whok] = (sigma2[whok]-mean[whok])^2
  IF nwhbad NE 0 THEN sigma2[whbad] = 0
  outliers = WHERE(sigma2 GE sigmathreshold^2, noutliers)

  IF KEYWORD_SET(getstats) NE 0 THEN BEGIN
     localmean = TEMPORARY(mean)
     localsigma = SQRT(TEMPORARY(sigma2))
     IF nwhbad NE 0 THEN localsigma[whbad] = !healpix.bad_value
  ENDIF

  RETURN


END
