PRO HPX_PIX_MEDIAN, map_in, map_out, ordering=ordering, width=width, silent=silent

  ;; Default values for keywords
  ;;-----------------------------
  IF NOT KEYWORD_SET(ordering) THEN ordering='RING'
  IF NOT KEYWORD_SET(width) THEN width=3
  IF (ISEVEN(width) OR (width LE 1)) THEN MESSAGE, 'The width of the median filter must be an even number (3, 5, 7, etc.)'

  ;; Create output array
  ;;---------------------
  map_out = map_in
  nside = GET_NSIDE(map_in)

  ;; Smooth all faces
  ;;------------------
  FOR face=0, 11 DO BEGIN
     
     IF NOT KEYWORD_SET(silent) THEN PRINT, 'Median filtering for face '+NUMBER2STRING(face)

     ;; Find pixels for face
     pixface = HPX_FACE_PIX(nside, face=face, ordering=ordering)

     ;; Read map_in values
     face_in = map_in[pixface]

     ;; Median of the face
     face_in = MEDIAN(TEMPORARY(face_in),width)

     ;; Assign values to output map
     map_out[pixface] = TEMPORARY(face_in)

     ;; Take care of the borders, which are wrong for the moment
     ;; Find face border pixels
     nhalf = LONG((width+1)/2)
     borderpix1 = pixface[0:(nhalf-2),0:(nside-nhalf)]
     borderpix2 = pixface[0:(nside-nhalf),(nside-nhalf+1):(nside-1)]
     borderpix3 = pixface[(nside-nhalf+1):(nside-1),(nhalf-1):(nside-1)]
     borderpix4 = pixface[(nhalf-1):(nside-1),0:(nhalf-2)]
     
     borderpix = [borderpix1[*],borderpix2[*],borderpix3[*],borderpix4[*]]
     nb = N_ELEMENTS(borderpix)
     
     ;; Loop on border pixels, using hpx_pix_neighbours
     FOR p=0L, nb-1 DO BEGIN
        ipix = borderpix[p]
        HPX_PIX_NEIGHBOURS, ipix, remote, list, maxremote=1, nside=nside, ordering=ordering
        map_out[ipix] = MEDIAN(map_in[list])
     ENDFOR
     
  ENDFOR

END
