; -----------------------------------------------------------------------------
;
; Copyright (C) 2011-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PIX2THETA_RING
;
; PURPOSE:
;	...
;
;-
PRO PIX2THETA_RING, nside, ipix, theta
  ;; Adapted from HEALPix routine PIX2ANG_RING 
  ;; by Jacques Delabrouille, 12 July 2011

  routine = 'PIX2THETA_RING'
  if N_params() lt 3 then begin
      message,' syntax: '+routine+', nside, ipix, theta'
  endif

  if (N_ELEMENTS(nside) GT 1) then message,'Nside should be a scalar in '+routine
  npix = nside2npix(nside, error = error)
  if (error ne 0) then message,'Invalid Nside '+string(nside)

  assert_pixindex_type, ipix[0], /warning ; warning if ipix is not integer
  min_pix = MIN(ipix, MAX = max_pix)
  IF (min_pix LT 0) THEN BEGIN
      PRINT,'pixel index : ',min_pix,FORMAT='(A,I18)'
      PRINT,'is out of range : ',0,npix-1,FORMAT='(A,I2,I18)'
      message,'Abort'
  ENDIF
  IF (max_pix GT npix-1) THEN BEGIN
      PRINT,'pixel index : ',max_pix,FORMAT='(A,I18)'
      PRINT,'is out of range : ',0,npix-1,FORMAT='(A,I2,I18)'
      message,'Abort'
  ENDIF

  nl1 = LONG(nside)
  nl2 = 2L*nl1
  if (nl1 gt 8192) then begin
      nl4 = 4LL*nl1
      ncap = nl2*(nl1-1LL)
      nsup = nl2*(5LL*nl1+1LL)
      one = 1LL
      four = 4LL
      l64 = 1
  endif else begin
      nl4 = 4L*nl1
      ncap = nl2*(nl1-1L)
      nsup = nl2*(5L*nl1+1L)
      one = 1L
      four = 4L
      l64 = 0
  endelse
  fact1 = 1.5d0*nl1
  fact2 = (3.d0*nl1)*nl1
  np = N_ELEMENTS(ipix)
  theta = DBLARR(np, /NoZero)

  ;; North polar cap
  ;;-----------------
  pix_np = WHERE(ipix LT ncap,   n_np)
  IF (n_np GT 0) THEN BEGIN     
      ip = ROUND(ipix[pix_np], L64=l64) + one
      iring = LONG( SQRT( ip/2.d0 - SQRT(ip/2) ) ) + 1L ; counted from NORTH pole, starting at 1
      ip=0
      theta[pix_np] = ACOS( 1.d0 - double(iring)^2 / fact2 )
      ip = 0 & iring =0         ; free memory
      pix_np = 0                ; free memory
  ENDIF                         

  ;; Equatorial strip
  ;;------------------
  pix_eq = WHERE(ipix GE ncap AND ipix LT nsup,  n_eq) 
  IF (n_eq GT 0) THEN BEGIN     
      ip    = ROUND(ipix[pix_eq], L64=l64) - ncap
      iring = LONG( ip / nl4) + nl1 ; counted from NORTH pole in {Nside, 3*Nside}
      ip=0
      theta[pix_eq] = ACOS( (nl2 - iring) / fact1 )
      ip = 0 & iring =0         ; free memory
      pix_eq = 0                ; free memory
  ENDIF                         

  ;; South polar cap
  ;;-----------------
  pix_sp = WHERE(ipix GE nsup,   n_sp) 
  IF (n_np + n_sp + n_eq) NE np THEN message,'error in '+routine
  IF (n_sp GT 0) THEN BEGIN     
      ip =  npix - ROUND(ipix[pix_sp], L64=l64)
      iring = LONG( SQRT( ip/2.d0 - SQRT(ip/2) ) ) + 1L ; counted from SOUTH pole, starting at 1
      ip=0
      theta[pix_sp] = ACOS( - 1.d0 + double(iring)^2 / fact2 )
      ip = 0 & iring =0         ; free memory
      pix_sp = 0                ; free memory

  ENDIF                         

  RETURN
END
