; -----------------------------------------------------------------------------
;
; Copyright (C) 2010
;   Jacques Delabrouille
;   Soumen Basak
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	QUERY_DIST
;
; PURPOSE:
;	...
;
;-
PRO QUERY_DIST, nside, vector0, radius_in, listpix, nlist, distpix, deg=deg, inclusive=inclusive, help=help, nested=nested

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'QUERY_DIST: procedure to get pixels within certain distance of a point, and their distance'
     PSM_INFO_MESSAGE, 'SYNTAX: QUERY_DIST, nside, vector0, radius_in, listpix, nlist, distpix, /deg, /inclusive, /nested'
     PSM_INFO_MESSAGE, "nside is the HEALPix nside"
     PSM_INFO_MESSAGE, "vector0 is an input 3 element array [x,y,z]"
     PSM_INFO_MESSAGE, 'listpix is the list of pixels within that max distance'
     PSM_INFO_MESSAGE, 'set /deg for radius_in and distpix in degrees (radian is the default)'
     PSM_INFO_MESSAGE, 'ring ordering is assumed, unless /nested is set'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN
  ENDIF

  QUERY_DISC, nside, vector0, radius_in, listpix, nlist, deg=deg, inclusive=inclusive, nested=nested
  care = GET_CAREFULNESS()
  IF care GE 1 THEN BEGIN
     magnitude = TOTAL(vector0^2)
     IF magnitude EQ 0 THEN BEGIN
        MESSAGE, 'Input vector must have non vanishing magnitude', /info
        GOTO, closing
     ENDIF
  ENDIF

  IF nlist NE 0 THEN BEGIN

     IF NOT KEYWORD_SET(nested) THEN BEGIN
        PIX2VEC_RING, nside, listpix, vec_out  
     ENDIF ELSE BEGIN
        PIX2VEC_NEST, nside, listpix, vec_out  
     ENDELSE
     cosang = vec_out # REFORM(vector0)
     IF care GE 1 THEN cosang = cosang/magnitude
     distpix = ACOS(cosang<1)
     IF KEYWORD_SET(deg) NE 0 THEN distpix = distpix*180d0/!dpi

  ENDIF ELSE BEGIN

     distpix = 0.

  ENDELSE

closing:
END

