; -----------------------------------------------------------------------------
;
; Copyright (C) 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	HASLAM_ZONES
;
; PURPOSE:
;	...
;
;-
FUNCTION HASLAM_ZONES, nzones=nzones, nside=nside, scaledeg=scaledeg, ordering=ordering, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'HASLAM_ZONES: function to cut the sky into zones of increasing Haslam 408 GHz emission'
     PSM_INFO_MESSAGE, 'SYNTAX: result = HASLAM_ZONES(nzones=,nside=,scaledeg=,ordering=)'
     PSM_INFO_MESSAGE, "set 'nzones' to the number of (equal area) zones expected [default=4]"
     PSM_INFO_MESSAGE, "set 'scaledeg' to the scale (in deg.) at which the mask is constructed [min=1, default=2] "
     PSM_INFO_MESSAGE, "Specify optionally the output nside [defaut=2048] and ordering [default='RING']"
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  ;; Set default values for keywords
  ;;---------------------------------
  IF NOT KEYWORD_SET(nside) THEN nside = 2048
  IF NOT KEYWORD_SET(scaledeg) THEN scaledeg = 2. ELSE IF scaledeg LE 1. THEN scaledeg=1.
  IF NOT KEYWORD_SET(ordering) THEN ordering = 'RING'
  IF NOT KEYWORD_SET(nzones) THEN nzones = 4

  mapfile = GETPSMDIR('psmdata')+'ancillary/observations/Haslam/lambda_haslam408_dsds_zerolevel_512.fits'
  ok = TEST_PSM_DATA(mapfile,/get)
  IF NOT ok THEN BEGIN
     PSM_INFO_MESSAGE, 'Sorry, Haslam file for synchrotron mask is not available. Returning.'
     RETURN, -1
  ENDIF ELSE BEGIN
     CHANGE_MAP_RESOLUTION, mapfile, map1deg, beam_in=60., beam_out=60.*scaledeg, nside_out=nside, ordering_in='RING', ordering_out=ordering, lmax=650
  ENDELSE
  s = SORT(map1deg)
  npix = NSIDE2NPIX(nside)
  zones = REPLICATE(nzones,npix)
  nperzone = DOUBLE(npix)/nzones
  endzone = -1L
  FOR i=1, nzones-1 DO BEGIN
     beginzone = endzone+1L
     endzone = LONG(i*nperzone)-1L
     zones[s[beginzone:endzone]] = i
  ENDFOR

  RETURN, zones

END
