; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;   Guillaume castex
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	CL_STRUCTURE
;
; PURPOSE:
;	...
;
;-
FUNCTION CL_STRUCTURE, nfields=nfields, fieldnames=fieldnames, units=units, lmin=lmin, lmax=lmax, lvalues=lvalues, double=double, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'CL_STRUCTURE: function to create a cl structure'
     PSM_INFO_MESSAGE, 'SYNTAX: result = CL_STRUCTURE(nfields=, fieldnames=, units=, lmin=, lmax=, lvalues=lvalues, /double)'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  IF KEYWORD_SET(fieldnames) NE 0 THEN BEGIN
     nfield = N_ELEMENTS(fieldnames)
     IF defined(nfields) THEN BEGIN
        IF nfield NE nfields THEN MESSAGE, 'Number of fields incompatible with dimension of fieldnames vector'
     ENDIF ELSE BEGIN
        nfields = nfield
     ENDELSE
  ENDIF ELSE BEGIN
     IF NOT DEFINED(nfields) THEN nfields=1
     fieldnames = NUMBER2STRING(LINDGEN(nfields)+1)
  ENDELSE
  
  IF NOT KEYWORD_SET(lvalues) THEN BEGIN
     IF NOT KEYWORD_SET(lmin) THEN lmin=0L
     IF NOT KEYWORD_SET(lmax) THEN lmax=3000L
     lvalues = lmin+LINDGEN(lmax-lmin+1)
  ENDIF ELSE BEGIN
  ENDELSE

  IF NOT KEYWORD_SET(units) THEN units='unknown'
  nunits = N_ELEMENTS(units)
  IF nunits NE nfields THEN BEGIN
     IF nunits EQ 1 THEN units = REPLICATE(units, nfields) ELSE MESSAGE, 'Provided units vector has incompatible dimensions'
  ENDIF

  creation_string = 'cl = {field:fieldnames,units:units,l:lvalues'
    creation_string = creation_string + '}' 
    ok = EXECUTE(creation_string)

    a=FLTARR(N_ELEMENTS(lvalues))
    b=DBLARR(N_ELEMENTS(lvalues))
    IF NOT KEYWORD_SET(double) THEN FOR i=0, nfields-1 DO FOR j=i, nfields-1 DO $
       cl=create_struct(cl, 'c_'+ NUMBER2STRING(i+1) + '_' + NUMBER2STRING(j+1), a) $
    ELSE FOR i=0, nfields-1 DO FOR j=i, nfields-1 DO $
       cl=create_struct(cl, 'c_'+ NUMBER2STRING(i+1) + '_' + NUMBER2STRING(j+1), b)

  RETURN, cl

END

