; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
; NAME:
;	NUMBER2STRING
;
; PURPOSE:
;	Function for converting a number (integer, long, float,
;	double)  into a string 'as usually written'. 
;
; CATEGORY:
;	String handling
;
; CALLING SEQUENCE:
;	result = NUMBER2STRING(Number)
;
; INPUTS:
;	Number:	A number (byte, integer, long integer, floating point
;	        number, double precision number...).
;
; OPTIONAL INPUTS:
;	None
;	
; KEYWORD PARAMETERS:
;	None
;
; OUTPUTS:
;	The result is a string that corresponds to the number
;	'as usually written'.
;
; RESTRICTIONS:
;       Report any bugs to Jacques Delabrouille
;       delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;
; EXAMPLE:
;
; MODIFICATION HISTORY:
; 	18 January 2009:	Jacques Delabrouille, First version
;	27 April 2009:  	Bug fix. Now writes correct number string for bytes
; 
;-
FUNCTION NUMBER2STRING, number, blank4plus=blank4plus

  type = SIZE(number,/type)
  IF type EQ 1 THEN sts = STRTRIM(STRING(FIX(number)),2) ELSE sts = STRTRIM(STRING(number),2)
  
  FOR n=0L, N_ELEMENTS(sts)-1 DO BEGIN
     st=sts(n)
     decimal = STR_MATCH(st,'.')
     
     ;; if decimal then remove trailing zeros
     IF decimal THEN BEGIN
        len = STRLEN(st)
        WHILE ((len GE 1) AND (STRMID(st,len-1,1) EQ '0')) DO BEGIN
           st = STRMID(st,0,len-1)
           len = len-1
        ENDWHILE
     ENDIF
     
     ;; remove final dot if necessary
     len = STRLEN(st)
     IF STRMID(st, len-1,1) EQ '.' THEN st = STRMID(st,0,len-1)
     
     sts(n)=st
     
  ENDFOR
  
  ;; add leading blank for positive numbers if needed
  IF KEYWORD_SET(blank4plus) THEN BEGIN
     whplus = WHERE(number GE 0, nwh)
     IF nwh NE 0 THEN sts[whplus] = ' '+sts[whplus]
  ENDIF
  

RETURN, sts

END
