; -----------------------------------------------------------------------------
;
; Copyright (C) 2009, 2010, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	STRUCT_UNIQ
;
; PURPOSE:
;	...
;
;-
FUNCTION STRUCT_UNIQ, structarr, index=index, help=help

  IF KEYWORD_SET(help) NE 0 THEN BEGIN
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     PRINT, 'STRUCT_UNIQ: function which returns the subscripts of the unique elements in an array of structures.'
     PSM_INFO_MESSAGE, 'SYNTAX: result = STRUCT_UNIQ(structarr, index=)'
     PSM_INFO_MESSAGE, 'structarr is an input array of structures, all of which have the same tag tree.'
     PSM_INFO_MESSAGE, 'index is an output keyword, which indexes the elements of structarr'
     PSM_INFO_MESSAGE, '(permits to find which ones are equal).'
     PSM_INFO_MESSAGE, 'This function similar in spirit to UNIQ, but sorting is not necessary.'
     PRINT, PSM_STRING_TO_LENGTH('-',LINE_LENGTH(),character='-')
     RETURN, 0
  ENDIF

  nel = N_ELEMENTS(structarr)

  unique = [0]
  index = INTARR(nel)

  IF nel GE 2 THEN BEGIN
     FOR i=1, nel-1 DO BEGIN
        j=0 & cont=1
        WHILE cont DO BEGIN
           egal = STRUCT_EQUAL(structarr[i],structarr[unique[j]])
           IF egal THEN BEGIN
              unique[j] = i
              index[i] = j
              cont=0
           ENDIF ELSE BEGIN
              j = j+1
              index[i] = j
           ENDELSE
           IF j EQ N_ELEMENTS(unique) THEN BEGIN
              cont = 0
              unique = [unique,i]
           ENDIF
        ENDWHILE

     ENDFOR
  ENDIF

  RETURN, unique[SORT(unique)]

END
