; -----------------------------------------------------------------------------
;
; Copyright (C) 2011-2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	MAKE_NG_FIGURES
;
; PURPOSE:
;	...
;
;-
PRO MAKE_NG_FIGURES, ng_map_number, resol_arcmin, lmax=lmax, nside=nside, output=output

  PSM_DEBUG_MESSAGE, 'Rewrite this code:', routine='MAKE_NG_FIGURES', infolimit=3, stoplimit=3
  GOTO, closing

  IF NOT DEFINED(lmax) THEN lmax=3500
  IF NOT DEFINED(nside) THEN nside=2048

  IF NOT KEYWORD_SET(output) THEN BEGIN
     STOP
  ENDIF

  ;; Check and get data 
  ;;--------------------
  cmbdata = GETPSMDIR('psmdata')+'ancillary/simulations/cmb/nongaussian-elsner/'
  ;IF lmax LE 3500 THEN cmbdata = cmbdata+'lmax3500/' ELSE MESSAGE, 'Maximum ell too high for this CMB model'
  ;; map number
  mapnumber = PSM_STRING_TO_LENGTH(NUMBER2STRING(ng_map_number),4,/strict,/leading,character='0')
  ;; linear part
  alm_lin = cmbdata+'alm_l_'+mapnumber+'.fits'
  ok = TEST_PSM_DATA(alm_lin,/get)
  IF NOT ok THEN  MESSAGE,'ERROR : NON-GAUSSIANITY DATA NOT AVAILABLE'
  ;; non linear part
  alm_nonlin = cmbdata+'alm_nl_'+mapnumber+'.fits'
  ok = TEST_PSM_DATA(alm_nonlin,/get)
  IF NOT ok THEN  MESSAGE,'ERROR : NON-GAUSSIANITY DATA NOT AVAILABLE'
  
  ;; Get gaussian beam 
  ;;------------------- 
  bl = GAUSSBEAM(resol_arcmin, lmax, 3)

  ni = LMAX2NALM(lmax)
  globalfactor = CONVERSION_FACTOR('K/KCMB','uK_CMB')

  ;; Linear map
  ;;------------
  CHANGE_ALM_RESOLUTION, alm_lin, alm, beam_in=GET_BEAM_STRUCT(0), beam_out=resol_arcmin, globalfactor=globalfactor
  IF ni LT N_ELEMENTS(alm[*,0,0]) THEN alm = TEMPORARY(alm[0:ni-1,*,*])
  PSM_ALM2MAP, TEMPORARY(alm), map_lin, hpx_nside = nside, /polar
  CASE output OF
     'png': MOLLVIEW, map_lin, title='Linear CMB map', units='uK_CMB', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'CMB_map_NG_LinearPart.png'
     'ps': MOLLVIEW, map_lin, title='Linear CMB map', units='uK_CMB', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'CMB_map_NG_LinearPart.ps'
     'screen': MOLLVIEW, map_lin, title='Linear CMB map', units='uK_CMB'
     ELSE: ;; do nothing
  ENDCASE
  map_lin = 0

  ;; Non linear map
  ;;---------------
  CHANGE_ALM_RESOLUTION, alm_nonlin, alm, beam_in=GET_BEAM_STRUCT(0), beam_out=resol_arcmin, globalfactor=globalfactor
  IF ni LT N_ELEMENTS(alm[*,0,0]) THEN alm = TEMPORARY(alm[0:ni-1,*,*])
  PSM_ALM2MAP, TEMPORARY(alm), map_nonlin, hpx_nside = nside, /polar
  CASE output OF
     'png': MOLLVIEW, map_nonlin, title='Non-linear CMB correction (f!DNL!N = 1)', units='uK_CMB', png=FIX_SEPARATOR(GETPSMDIR('visu'))+'CMB_map_NG_NLcorrection.png'
     'ps': MOLLVIEW, map_nonlin, title='Non-linear CMB correction (f!DNL!N = 1)', units='uK_CMB', ps=FIX_SEPARATOR(GETPSMDIR('visu'))+'CMB_map_NG_NLcorrection.ps'
     'screen': MOLLVIEW, map_nonlin, title='Non-linear CMB correction (f!DNL!N = 1)', units='uK_CMB'
     ELSE: ;; do nothing
  ENDCASE
  map_lin = 0
  
closing:

END
