; -----------------------------------------------------------------------------
;
; Copyright (C) 2010-2012
;   Guillaume Castex
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	PLOT_SOURCE
;
; PURPOSE:
;	...
;
;-
PRO PLOT_SOURCE, source, title=title, xtitle=xtitle, ytitle=ytitle, oplot=oplot, line=line,$
                 thick=thick, xthick=xthick, ythick=ythick,  xrange=xrange,  yrange=yrange, charsize=charsize

IF (NOT KEYWORD_SET(xtitle)) THEN xtitle='FREQUENCY (GHz)'
IF (NOT KEYWORD_SET(ytitle)) THEN ytitle='EMISSION (mJy)'
IF (NOT KEYWORD_SET(oplot)) THEN oplot=0
IF (NOT KEYWORD_SET(line)) THEN line=1
IF (NOT KEYWORD_SET(thick)) THEN thick=1
IF (NOT KEYWORD_SET(xthick)) THEN xthick=1
IF (NOT KEYWORD_SET(ythick)) THEN ythick=1
IF (NOT KEYWORD_SET(charsize)) THEN charsize=1

  tags = TAG_NAMES(source)

  ;; Find number of emission laws
  ;;------------------------------
  n_emlaw = 0
  next=1
  WHILE next DO BEGIN
     wh = WHERE(tags EQ 'E'+NUMBER2STRING(n_emlaw+1), nwh)
     IF nwh EQ 1 THEN n_emlaw = n_emlaw+1 ELSE next=0
  ENDWHILE
  
  ;; Plot model 
  ;;------------
  IF n_emlaw NE 0 THEN BEGIN
     
     nu = (FINDGEN(10000)+10.) * 3e8
     em = FLTARR(10000)
     
     FOR j=1, n_emlaw DO BEGIN
        ok = EXECUTE('emi = source.e'+NUMBER2STRING(j))
        CASE emi.ampl.unit OF
           'mJy': factor=1
           'Jy': factor=1000
        ENDCASE
        em = em + PSM_EMLAW(nu,emi) * (*emi.ampl.value) * factor
     ENDFOR
     IF oplot EQ 0 THEN BEGIN
        PLOT, nu/1e9, em, /xlog, /ylog, title=title, xtitle=xtitle, ytitle=ytitle, line=line, thick=thick, xthick=xthick, ythick=ythick, xrange=xrange, yrange=yrange, charsize=charsize
     ENDIF ELSE BEGIN
        OPLOT, nu/1e9, em, line=line, thick=thick
     ENDELSE
  ENDIF

END

