; -----------------------------------------------------------------------------
;
; Copyright (C) 1997, 2000, 2001, 2012
;   Jacques Delabrouille
;
; This file is part of the Planck Sky Model (PSM).
;
; The Planck Sky Model is free software: you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation, either version 3 of the License, or
; (at your option) any later version.
;
; The Planck Sky Model is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with the Planck Sky Model.  If not, see <http://www.gnu.org/licenses/>.
;
; For more information about the PSM see:
; http://www.apc.univ-paris7.fr/~delabrou/PSM/psm.html
; or contact Jacques Delabrouille (delabrouille@apc.univ-paris7.fr)
;
; -----------------------------------------------------------------------------
;+
; NAME:
;	SIMPLEVISU
;
; PURPOSE:
;	Visualise an image (open free window of the appropriate size and display image).
;
; CATEGORY
;	Visualisation 
;
; CALLING SEQUENCE:
;	SIMPLEVISU, Image
;
; INPUTS:
;	Image:	a two-dimensional array.
;
; KEYWORD PARAMETERS:
;	MAGNIFICATION :	A factor by which to enlarge (or decrease if <1) the size of the display.
;	NUM_WINDOW :	Number of the window in which display is made.
;	NONEW : 	If this keyword is set display is in current window.
;	DONT_SCALE : 	If this keyword is set display is made with TV.
;	                instead of TVSCL.
;	TITLE : 	Set this keyword for naming the window.
;
; OUTPUTS:
;	None.
;
; SIDE EFFECTS:
;	Opens a new window.
;
; RESTRICTIONS:
;	Report any bugs to Jacques Delabrouille
;	delabrouille@apc.univ-paris7.fr
;
; PROCEDURE:
;	Simple IDL commands
;
; EXAMPLE:
;	SIMPLEVISU, findgen(300)#replicate(1,200), magnification = 1.5
;
; REVISION HISTORY
;	January 1997:	Jacques Delabrouille, First version.
;	May 2000:	JD, MAGNIFICATION keyword added.
;	January 2001:	JD, NUM_WINDOW keyword added.
;	March 2001:	JD, NONEW keyword added.
;	April 2001:	JD, Interactive help added.
;	06/06/2001:	JD, DONT_SCALE keyword added.
; 	09/01/2012:	JD, doc and PSM license header.
;-
PRO SIMPLEVISU, image, magnification = magnification, num_window = num_window, nonew = nonew, $
	dont_scale=dont_scale, title=title

IF ( N_PARAMS() EQ 0 ) THEN BEGIN
	print,'  CALL IS : SIMPLEVISU, image'
	print,'  Keywords : magnification=, num_window= , nonew= , dont_scale=, title=title'
     GOTO, closing
ENDIF

n1=N_ELEMENTS((REFORM(image))(*,0))
n2=N_ELEMENTS((REFORM(image))(0,*))
IF NOT KEYWORD_SET(magnification) THEN magnification = 1

IF magnification EQ 1 THEN BEGIN
	IF NOT KEYWORD_SET(nonew) THEN BEGIN
		IF DEFINED(num_window) EQ 0 THEN WINDOW,/free, xs=n1, ys=n2, ret=2, title=title $
		ELSE WINDOW,num_window, xs=n1, ys=n2, ret=2, title=title
	ENDIF
	IF NOT KEYWORD_SET(dont_scale) THEN TVSCL, (REFORM(image)) ELSE TV, (REFORM(image))
ENDIF ELSE BEGIN
	IF NOT KEYWORD_SET(nonew) THEN BEGIN
		IF DEFINED(num_window) EQ 0 THEN WINDOW,/free, xs=n1*magnification, ys=n2*magnification, ret=2, title=title $
		ELSE WINDOW, num_window, xs=n1*magnification, ys=n2*magnification, ret=2, title=title
	ENDIF
	IF NOT KEYWORD_SET(dont_scale) THEN TVSCL, CONGRID((REFORM(image)),n1*magnification,n2*magnification) $
		ELSE TV, CONGRID((REFORM(image)),n1*magnification,n2*magnification)
ENDELSE

closing:
END
